import { createSlice, createSelector, configureStore } from '@reduxjs/toolkit';
import require$$0, { useState, useCallback, useEffect, useRef, useMemo } from 'react';
import { LocalMediaStream, ZocksSDK, RoomState, ConnectionQuality, StreamSource, StreamKind, ZocksEvent, MediaDeviceManager } from '@zocks-communications/client-sdk-js';
import { useSelector, useDispatch, Provider } from 'react-redux';
import { Popover, ArrowContainer } from 'react-tiny-popover';
import Linkify from 'linkify-react';

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    enumerableOnly && (symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    })), keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = null != arguments[i] ? arguments[i] : {};
    i % 2 ? ownKeys(Object(source), !0).forEach(function (key) {
      _defineProperty(target, key, source[key]);
    }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) {
      Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
    });
  }

  return target;
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

var ChatMessageType;

(function (ChatMessageType) {
  ChatMessageType["PUBLIC"] = "public";
  ChatMessageType["PRIVATE"] = "private";
})(ChatMessageType || (ChatMessageType = {}));

const initialState$2 = {
  publicChatMessages: [],
  privateChatMessages: {}
};
const chatSlice = createSlice({
  name: 'chat',
  initialState: initialState$2,
  reducers: {
    savePublicChatMessage: (state, action) => {
      const chatMessage = action.payload;
      state.publicChatMessages.push(_objectSpread2({
        type: ChatMessageType.PUBLIC
      }, chatMessage));
    },
    privateChatMessageReceived: (state, action) => {
      const chatMessage = action.payload;
      const {
        privateChatMessages
      } = state;
      const {
        from: {
          id: senderId
        }
      } = chatMessage;
      addPrivateMessageForParticipant(privateChatMessages, chatMessage, senderId);
    },
    privateChatMessageSent: (state, action) => {
      const chatMessage = action.payload;
      const {
        privateChatMessages
      } = state;
      const {
        to: {
          id: recipientId
        } = {}
      } = chatMessage;
      addPrivateMessageForParticipant(privateChatMessages, chatMessage, recipientId);
    },
    clearChatMessages: state => {
      state.publicChatMessages = [];
      state.privateChatMessages = {};
    }
  }
});

const addPrivateMessageForParticipant = (privateChatMessages, chatMessage, remoteParticipantId) => {
  if (!remoteParticipantId) {
    return;
  }

  if (!Object.keys(privateChatMessages).includes(remoteParticipantId)) {
    privateChatMessages[remoteParticipantId] = [];
  }

  privateChatMessages[remoteParticipantId].push(_objectSpread2({
    type: ChatMessageType.PRIVATE
  }, chatMessage));
};

const {
  savePublicChatMessage,
  privateChatMessageReceived,
  privateChatMessageSent,
  clearChatMessages
} = chatSlice.actions;
const selectPublicChatMessages = state => state.chat.publicChatMessages;
const selectPrivateChatMessages = state => state.chat.privateChatMessages;
var chatReducer = chatSlice.reducer;

const useLocalMedia = (videoEnabledByDefault, audioEnabledByDefault) => {
  const [audioDevice, setAudioDevice] = useState();
  const [videoDevice, setVideoDevice] = useState();
  const [localAudioStream, setLocalAudioStream] = useState();
  const [localVideoStream, setLocalVideoStream] = useState();
  const [videoStreamIsLoading, setVideoStreamIsLoading] = useState(false);
  const startLocalAudioStream = useCallback(deviceId => {
    LocalMediaStream.create({
      audio: {
        deviceId
      },
      video: false
    }).then(audioStream => setLocalAudioStream(audioStream));
  }, []);
  const startLocalVideoStream = useCallback(deviceId => {
    setVideoStreamIsLoading(true);
    LocalMediaStream.create({
      audio: false,
      video: {
        deviceId
      }
    }).then(videoStream => setLocalVideoStream(videoStream)).finally(() => setVideoStreamIsLoading(false));
  }, []);

  const toggleLocalAudioStream = () => {
    if (localAudioStream) {
      localAudioStream.stopStream();
      setLocalAudioStream(undefined);
    } else {
      startLocalAudioStream(audioDevice === null || audioDevice === void 0 ? void 0 : audioDevice.deviceId);
    }
  };

  const toggleLocalVideoStream = () => {
    if (localVideoStream) {
      localVideoStream.stopStream();
      setLocalVideoStream(undefined);
    } else {
      startLocalVideoStream(videoDevice === null || videoDevice === void 0 ? void 0 : videoDevice.deviceId);
    }
  };

  const selectAudioDevice = audioDevice => {
    var _a;

    setAudioDevice(audioDevice);

    if (localAudioStream) {
      if (((_a = localAudioStream.mediaStreamTrack) === null || _a === void 0 ? void 0 : _a.getSettings().deviceId) === audioDevice.deviceId) {
        return;
      }

      localAudioStream.stopStream();
      startLocalAudioStream(audioDevice.deviceId);
    }
  };

  const selectVideoDevice = videoDevice => {
    var _a;

    setVideoDevice(videoDevice);

    if (localVideoStream) {
      if (((_a = localVideoStream.mediaStreamTrack) === null || _a === void 0 ? void 0 : _a.getSettings().deviceId) === videoDevice.deviceId) {
        return;
      }

      localVideoStream.stopStream();
      setLocalVideoStream(undefined);
      startLocalVideoStream(videoDevice.deviceId);
    }
  };

  useEffect(() => {
    if (videoEnabledByDefault) {
      startLocalVideoStream();
    }
  }, [videoEnabledByDefault, startLocalVideoStream]);
  useEffect(() => {
    if (audioEnabledByDefault) {
      startLocalAudioStream();
    }
  }, [audioEnabledByDefault, startLocalAudioStream]);
  return {
    localAudioStream,
    localVideoStream,
    videoStreamIsLoading,
    toggleLocalAudioStream,
    toggleLocalVideoStream,
    selectAudioDevice,
    selectVideoDevice
  };
};

const useZocksDispatch = () => useDispatch();
const useZocksStore = useSelector;

const getSessionById = sessionId => {
  if (!sessionId) {
    return;
  }

  return ZocksSDK.getSessionById(sessionId);
};
const getTrackById = (participant, trackId) => {
  return participant && trackId && participant.tracks.find(t => t.id === trackId) || null;
};

const createNewParticipant = function () {
  let id = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;
  let name = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
  return {
    id,
    name,
    tracks: {
      videoTracks: [],
      audioTracks: [],
      screenShareTrack: null
    },
    connectionQuality: ConnectionQuality.EXCELLENT
  };
};

const initialState$1 = {
  sessionId: null,
  roomState: RoomState.DISCONNECTED,
  isConnecting: false,
  localParticipant: createNewParticipant(),
  remoteParticipants: {},
  activeSpeakers: []
};
const sessionSlice = createSlice({
  name: 'session',
  initialState: initialState$1,
  reducers: {
    setRoomState: (state, action) => {
      state.roomState = action.payload;
    },
    setSessionId: (state, action) => {
      state.sessionId = action.payload;
    },
    setConnecting: (state, action) => {
      state.isConnecting = action.payload;
    },
    clearSessionId: state => {
      state.sessionId = null;
    },
    setLocalParticipantId: (state, action) => {
      state.localParticipant.id = action.payload;
    },
    setLocalParticipantName: (state, action) => {
      state.localParticipant.name = action.payload || null;
    },
    clearParticipants: state => {
      state.localParticipant = createNewParticipant();
      state.remoteParticipants = {};
    },
    participantLocalTrackPublished: (state, action) => {
      const {
        trackId,
        trackKind,
        trackSource,
        trackIsMuted
      } = action.payload;
      addTrackForParticipant(state.localParticipant, trackKind, trackSource, trackId, trackIsMuted);
    },
    participantLocalTrackUnPublished: (state, action) => {
      const {
        trackId,
        trackKind,
        trackSource
      } = action.payload;
      removeTrackFromParticipant(state.localParticipant, trackKind, trackSource, trackId);
    },
    addRemoteParticipant: (state, action) => {
      const {
        participantId,
        participantName
      } = action.payload;
      state.remoteParticipants[participantId] = createNewParticipant(participantId, participantName);
    },
    removeRemoteParticipant: (state, action) => {
      const {
        participantId
      } = action.payload;
      delete state.remoteParticipants[participantId];
    },
    subscribeToParticipantTrack: (state, action) => {
      const {
        participantId,
        trackId,
        trackKind,
        trackSource,
        trackIsMuted
      } = action.payload;
      const remoteParticipant = state.remoteParticipants[participantId];

      if (remoteParticipant) {
        removeTrackFromParticipant(remoteParticipant, trackKind, trackSource, trackId);
        addTrackForParticipant(remoteParticipant, trackKind, trackSource, trackId, trackIsMuted);
      }
    },
    participantTrackUnPublished: (state, action) => {
      const {
        participantId,
        trackId,
        trackKind,
        trackSource
      } = action.payload;
      const remoteParticipant = state.remoteParticipants[participantId];

      if (remoteParticipant) {
        removeTrackFromParticipant(remoteParticipant, trackKind, trackSource, trackId);
      }
    },
    participantTrackMuted: (state, action) => {
      const {
        participantId,
        trackId
      } = action.payload;
      const participant = getParticipantById(state, participantId);
      muteParticipantTrack(participant, trackId);
    },
    participantTrackUnMuted: (state, action) => {
      const {
        participantId,
        trackId
      } = action.payload;
      const participant = getParticipantById(state, participantId);
      unMuteParticipantTrack(participant, trackId);
    },
    connectionQualityChanged: (state, action) => {
      const {
        participantId,
        connectionQuality
      } = action.payload;
      const participant = getParticipantById(state, participantId);

      if (participant) {
        participant.connectionQuality = connectionQuality;
      }
    },
    setActiveSpeakers: (state, action) => {
      state.activeSpeakers = action.payload;
    }
  }
});
const {
  setRoomState,
  setSessionId,
  setConnecting,
  clearSessionId,
  setLocalParticipantId,
  setLocalParticipantName,
  participantLocalTrackPublished,
  participantLocalTrackUnPublished,
  addRemoteParticipant,
  removeRemoteParticipant,
  clearParticipants,
  subscribeToParticipantTrack,
  participantTrackMuted,
  participantTrackUnMuted,
  participantTrackUnPublished,
  connectionQualityChanged,
  setActiveSpeakers
} = sessionSlice.actions;
const selectSessionId = state => state.session.sessionId;
const selectRoomState = state => state.session.roomState;
const selectIsConnecting = state => state.session.isConnecting;
const selectLocalParticipant = state => state.session.localParticipant;
const selectActiveSpeakers = state => state.session.activeSpeakers;
const selectRemoteParticipants = createSelector([state => state.session.remoteParticipants], remoteParticipants => {
  return Object.values(remoteParticipants);
});
const selectLocalParticipantIsConnected = createSelector([selectRoomState, selectLocalParticipant], (roomState, localParticipant) => {
  return roomState === RoomState.CONNECTED && !!localParticipant.id;
});
const selectIsScreenSharing = createSelector([selectLocalParticipant, selectRemoteParticipants], (localParticipant, remoteParticipants) => {
  const localParticipantScreenShare = !!localParticipant.tracks.screenShareTrack;
  const remoteParticipantScreenShare = remoteParticipants.some(remoteParticipant => !!remoteParticipant.tracks.screenShareTrack);
  return localParticipantScreenShare || remoteParticipantScreenShare;
});
const selectScreenSharingParticipant = createSelector([selectLocalParticipant, state => state.session.remoteParticipants], (localParticipant, remoteParticipants) => {
  if (localParticipant.tracks.screenShareTrack) {
    return localParticipant;
  }

  return Object.values(remoteParticipants).find(participant => !!participant.tracks.screenShareTrack) || null;
});
const selectSpeakingRemoteParticipantAudioTracks = createSelector([selectSessionId, selectRoomState, state => state.session.remoteParticipants], (sessionId, roomState, remoteParticipants) => {
  const zocksSession = getSessionById(sessionId);

  if (roomState !== RoomState.CONNECTED || !zocksSession) {
    return [];
  }

  const speakingParticipantAudioTracks = [];

  for (const [participantId, participant] of Object.entries(remoteParticipants)) {
    const {
      tracks: {
        audioTracks
      }
    } = participant;

    if (audioTracks.length && !audioTracks[0].isMuted) {
      const zocksParticipant = zocksSession === null || zocksSession === void 0 ? void 0 : zocksSession.participants.get(participantId);
      const zocksTrack = convertToTrackView(audioTracks[0], zocksParticipant);

      if (zocksTrack) {
        speakingParticipantAudioTracks.push(zocksTrack);
      }
    }
  }

  return speakingParticipantAudioTracks;
});
const selectTracksForParticipant = createSelector([selectSessionId, selectRoomState, (state, participant) => participant], (sessionId, roomState, participant) => {
  const {
    id: participantId,
    tracks: {
      audioTracks,
      videoTracks,
      screenShareTrack
    }
  } = participant;
  const zocksSession = getSessionById(sessionId);

  if (roomState !== RoomState.CONNECTED || !zocksSession || !participantId) {
    return {
      videoTracks: [],
      audioTracks: [],
      primaryVideoTrack: null,
      primaryAudioTrack: null,
      screenShareTrack: null
    };
  }

  const zocksParticipant = participantId === zocksSession.localParticipant.id ? zocksSession.localParticipant : zocksSession.participants.get(participantId);
  const videoTrackViews = convertToTrackViews(videoTracks, zocksParticipant);
  const primaryVideoTrack = videoTrackViews.find(videoTrack => !videoTrack.isMuted) || videoTrackViews[0] || null;
  const audioTrackViews = convertToTrackViews(audioTracks, zocksParticipant);
  const primaryAudioTrack = audioTrackViews.find(audioTrack => !audioTrack.isMuted) || audioTrackViews[0] || null;
  return {
    videoTracks: videoTrackViews,
    audioTracks: audioTrackViews,
    primaryVideoTrack,
    primaryAudioTrack,
    screenShareTrack: screenShareTrack && convertToTrackView(screenShareTrack, zocksParticipant) || null
  };
});

const getParticipantById = (session, participantId) => {
  const {
    localParticipant,
    remoteParticipants
  } = session;
  return participantId === localParticipant.id ? localParticipant : remoteParticipants[participantId];
};

const addTrackForParticipant = (participant, trackKind, trackSource, trackId, isMuted) => {
  const {
    tracks
  } = participant;

  if (trackSource === StreamSource.SCREEN_SHARE) {
    tracks.screenShareTrack = {
      id: trackId,
      isMuted
    };
  } else if (trackKind === StreamKind.AUDIO) {
    tracks.audioTracks.push({
      id: trackId,
      isMuted
    });
  } else if (trackKind === StreamKind.VIDEO) {
    tracks.videoTracks.push({
      id: trackId,
      isMuted
    });
  }
};

const removeTrackFromParticipant = (participant, trackKind, trackSource, trackId) => {
  const {
    tracks
  } = participant;

  if (trackSource === StreamSource.SCREEN_SHARE) {
    tracks.screenShareTrack = null;
  } else if (trackKind === StreamKind.AUDIO) {
    tracks.audioTracks = tracks.audioTracks.filter(track => track.id !== trackId);
  } else if (trackKind === StreamKind.VIDEO) {
    tracks.videoTracks = tracks.videoTracks.filter(track => track.id !== trackId);
  }
};

const muteParticipantTrack = (participant, trackId) => {
  changeParticipantTrackMuteStatus(true, participant, trackId);
};

const unMuteParticipantTrack = (participant, trackId) => {
  changeParticipantTrackMuteStatus(false, participant, trackId);
};

const changeParticipantTrackMuteStatus = (isMuted, participant, trackId) => {
  if (!participant || !trackId) {
    return;
  }

  const {
    audioTracks,
    videoTracks
  } = participant.tracks;
  const track = audioTracks.find(track => track.id === trackId) || videoTracks.find(track => track.id === trackId);

  if (track) {
    track.isMuted = isMuted;
  }
};

const convertToTrackViews = (tracks, zocksParticipant) => {
  return tracks.reduce((result, track) => {
    const zocksTrack = getTrackById(zocksParticipant, track.id);

    if (zocksTrack) {
      result.push({
        track: zocksTrack,
        isMuted: track.isMuted
      });
    }

    return result;
  }, []);
};

const convertToTrackView = (track, zocksParticipant) => {
  const zocksTrack = getTrackById(zocksParticipant, track.id);

  if (zocksTrack) {
    return {
      track: zocksTrack,
      isMuted: track.isMuted
    };
  }

  return null;
};

var sessionReducer = sessionSlice.reducer;

const useZocksActions = zocksConfig => {
  const dispatch = useZocksDispatch();
  const [session, setSession] = useState();

  const setupEventHandlers = session => {
    session.on(ZocksEvent.ParticipantLocalTrackPublished, track => {
      if (track.isAvailable) {
        dispatch(participantLocalTrackPublished({
          trackId: track.id,
          trackKind: track.kind,
          trackSource: track.source,
          trackIsMuted: track.isMuted
        }));
      }
    });
    session.on(ZocksEvent.ParticipantLocalTrackUnpublished, track => {
      if (track.isAvailable) {
        dispatch(participantLocalTrackUnPublished({
          trackId: track.id,
          trackKind: track.kind,
          trackSource: track.source
        }));
      }
    });
    session.on(ZocksEvent.ParticipantTrackUnPublished, (track, participant) => {
      if (track.isAvailable) {
        dispatch(participantTrackUnPublished({
          participantId: participant.id,
          trackId: track.id,
          trackKind: track.kind,
          trackSource: track.source
        }));
      }
    });
    session.on(ZocksEvent.SessionConnected, () => {
      dispatch(setSessionId(session.id));
    });
    session.on(ZocksEvent.SessionStateChanged, roomState => {
      dispatch(setRoomState(roomState));

      if (roomState === RoomState.DISCONNECTED) {
        dispatch(clearParticipants());
        dispatch(clearSessionId());
        dispatch(clearChatMessages());
      }
    });
    session.on(ZocksEvent.LocalParticipantConnected, localParticipant => {
      dispatch(setLocalParticipantId(localParticipant.id));
      dispatch(setLocalParticipantName(localParticipant.name));
    });
    session.on(ZocksEvent.ParticipantConnected, participant => {
      dispatch(addRemoteParticipant({
        participantId: participant.id,
        participantName: participant.name
      }));
    });
    session.on(ZocksEvent.ParticipantDisconnected, participantId => {
      if (participantId) {
        dispatch(removeRemoteParticipant({
          participantId
        }));
      }
    });
    session.on(ZocksEvent.ParticipantTrackSubscribed, (track, participant) => {
      if (participant.id && track.isAvailable) {
        dispatch(subscribeToParticipantTrack({
          participantId: participant.id,
          trackId: track.id,
          trackKind: track.kind,
          trackSource: track.source,
          trackIsMuted: track.isMuted
        }));
      }
    });
    session.on(ZocksEvent.ConnectionQualityChanged, (connectionQuality, participant) => {
      dispatch(connectionQualityChanged({
        participantId: participant.id,
        connectionQuality
      }));
    });
    session.on(ZocksEvent.ParticipantTrackMuted, (participant, track) => {
      dispatch(participantTrackMuted({
        participantId: participant.id,
        trackId: track.id
      }));
    });
    session.on(ZocksEvent.ParticipantTrackUnMuted, (participant, track) => {
      dispatch(participantTrackUnMuted({
        participantId: participant.id,
        trackId: track.id
      }));
    });
    session.on(ZocksEvent.ActiveSpeakersChanged, participants => {
      dispatch(setActiveSpeakers(participants.filter(p => !!p.id).map(p => p.id)));
    });
    session.on(ZocksEvent.PrivateChatMessageReceived, chatMessage => {
      const {
        message,
        timestamp,
        from,
        attachment
      } = chatMessage;
      dispatch(privateChatMessageReceived({
        message,
        timestamp,
        from,
        to: {
          id: session.localParticipant.id,
          name: session.localParticipant.name
        },
        attachment
      }));
    });
    session.on(ZocksEvent.PublicChatMessageReceived, chatMessage => {
      const {
        message,
        timestamp,
        from,
        attachment
      } = chatMessage;
      dispatch(savePublicChatMessage({
        message,
        timestamp,
        from,
        attachment
      }));
    });
  };

  const createRoom = () => {
    const zocksSDK = ZocksSDK.init(zocksConfig);
    return zocksSDK.API.room.create();
  };

  const join = joinOptions => {
    const {
      roomId,
      autoCreate,
      audioDeviceSettings,
      videoDeviceSettings,
      afterJoinCallback,
      joinErrorCallback
    } = joinOptions;
    const zocksSDK = ZocksSDK.init(zocksConfig);
    const zocksSession = zocksSDK.createNewSession();
    setSession(zocksSession);
    setupEventHandlers(zocksSession);

    const joinRoom = async () => {
      dispatch(setConnecting(true));
      await zocksSession.join(roomId, autoCreate);
    };

    joinRoom().then(() => {
      afterJoinCallback && afterJoinCallback();
      audioDeviceSettings && zocksSession.localParticipant.publishAudio(audioDeviceSettings);
      videoDeviceSettings && zocksSession.localParticipant.publishVideo(videoDeviceSettings);
    }).catch(e => joinErrorCallback && joinErrorCallback(e)).finally(() => dispatch(setConnecting(false)));
  };

  const leave = () => {
    session === null || session === void 0 ? void 0 : session.disconnect();
  };

  const publishVideo = mediaTrackSettings => {
    session === null || session === void 0 ? void 0 : session.localParticipant.publishVideo(mediaTrackSettings);
  };

  const unPublishVideo = mediaTrackSettings => {
    session === null || session === void 0 ? void 0 : session.localParticipant.unPublishVideo(mediaTrackSettings);
  };

  const publishAudio = mediaTrackSettings => {
    session === null || session === void 0 ? void 0 : session.localParticipant.publishAudio(mediaTrackSettings);
  };

  const unPublishAudio = mediaTrackSettings => {
    session === null || session === void 0 ? void 0 : session.localParticipant.unPublishAudio(mediaTrackSettings);
  };

  const publishScreenShare = () => {
    session === null || session === void 0 ? void 0 : session.localParticipant.publishScreenShare().catch(console.error);
  };

  const unPublishScreenShare = () => {
    session === null || session === void 0 ? void 0 : session.localParticipant.unPublishScreenShare();
  };

  const switchActiveDevice = (kind, deviceId) => {
    session === null || session === void 0 ? void 0 : session.switchActiveDevice(kind, deviceId);
  };

  const sendPublicMessage = (message, attachment) => {
    if (!session) {
      throw Error('Cannot send public message without a session!');
    }

    session.sendMessageToRoom(message, attachment);
    dispatch(savePublicChatMessage({
      message,
      timestamp: Date.now(),
      from: {
        id: session.localParticipant.id,
        name: session.localParticipant.name
      },
      attachment
    }));
  };

  const sendPrivateMessage = (message, remoteParticipant, attachment) => {
    if (!remoteParticipant.id || !session) {
      throw Error('Cannot send private message without a session or recipient!');
    }

    session.sendMessageToParticipant(message, remoteParticipant.id, attachment);
    dispatch(privateChatMessageSent({
      message,
      timestamp: Date.now(),
      from: {
        id: session.localParticipant.id,
        name: session.localParticipant.name
      },
      to: {
        id: remoteParticipant.id,
        name: remoteParticipant.name
      },
      attachment
    }));
  };

  const uploadAttachment = async (file, remoteParticipant) => {
    if (remoteParticipant && !remoteParticipant.id || !session) {
      throw Error('Cannot upload attachment without a session or with empty recipient id!');
    }

    const recipientId = (remoteParticipant === null || remoteParticipant === void 0 ? void 0 : remoteParticipant.id) || undefined;
    return session.uploadAttachment(file, recipientId);
  };

  const getAttachmentLocation = async attachmentFileId => {
    if (!session) {
      throw Error('Cannot get attachment location without a session!');
    }

    return session.getFileDownloadLocation(attachmentFileId);
  };

  return {
    session,
    sessionActions: {
      createRoom,
      join,
      leave
    },
    localMediaActions: {
      publishVideo,
      publishAudio,
      unPublishVideo,
      unPublishAudio,
      publishScreenShare,
      unPublishScreenShare,
      switchActiveDevice
    },
    chatActions: {
      sendPublicMessage,
      sendPrivateMessage,
      uploadAttachment,
      getAttachmentLocation
    }
  };
};
const useZocksSession = () => {
  const sessionId = useZocksStore(selectSessionId);
  const roomState = useZocksStore(selectRoomState);
  const isConnecting = useZocksStore(selectIsConnecting);
  const localParticipant = useZocksStore(selectLocalParticipant);
  const remoteParticipants = useZocksStore(selectRemoteParticipants);
  const localParticipantIsConnected = useZocksStore(selectLocalParticipantIsConnected);
  return {
    sessionId,
    roomState,
    isConnecting,
    localParticipant,
    remoteParticipants,
    localParticipantIsConnected
  };
};
const useParticipant = participantModel => {
  const tracks = useZocksStore(state => selectTracksForParticipant(state, participantModel));
  const localParticipant = useZocksStore(selectLocalParticipant);
  const activeSpeakers = useZocksStore(selectActiveSpeakers);
  const isLocal = participantModel.id === localParticipant.id;
  const isSpeaking = activeSpeakers.some(participantId => participantId === participantModel.id);
  const {
    id,
    name,
    connectionQuality
  } = participantModel;
  return {
    id,
    name,
    tracks,
    isLocal,
    connectionQuality,
    isSpeaking
  };
};
const useLeaveOnUnload = leave => {
  useEffect(() => {
    window.onunload = () => {
      leave();
    };
  }, [leave]);
};
const useSynchronizeParticipants = () => {
  const dispatch = useZocksDispatch();
  const stateParticipants = useZocksStore(selectRemoteParticipants);
  const sessionId = useZocksStore(selectSessionId);
  const zocksSession = getSessionById(sessionId);

  const synchronizeParticipants = () => {
    if (!zocksSession) {
      return;
    }

    const {
      participants: zocksParticipants
    } = zocksSession;

    for (const stateParticipant of stateParticipants) {
      if (stateParticipant.id && !zocksParticipants.has(stateParticipant.id)) {
        removeObsoleteParticipant(stateParticipant.id);
      }
    }

    for (const zocksParticipant of zocksParticipants.values()) {
      const {
        id: zocksParticipantId,
        name: zocksParticipantName,
        tracks: zocksParticipantTracks
      } = zocksParticipant;
      const stateParticipant = stateParticipants.find(participant => participant.id === zocksParticipantId);

      if (!stateParticipant) {
        addMissingParticipant(zocksParticipantTracks, zocksParticipantId, zocksParticipantName);
      } else {
        synchronizeExistingParticipant(stateParticipant, zocksParticipantTracks);
      }
    }
  };

  const removeObsoleteParticipant = participantId => {
    dispatch(removeRemoteParticipant({
      participantId
    }));
  };

  const addMissingParticipant = (participantTracks, participantId, participantName) => {
    dispatch(addRemoteParticipant({
      participantId,
      participantName
    }));

    for (const zocksTrack of participantTracks) {
      subscribeToTrack(participantId, zocksTrack);
    }
  };

  const synchronizeExistingParticipant = (stateParticipant, zocksTracks) => {
    fixMuteStatusOfTracks(stateParticipant, zocksTracks);
    addMissingTracks(stateParticipant, zocksTracks);
    removeObsoleteTracks(stateParticipant, zocksTracks);
  };

  const fixMuteStatusOfTracks = (stateParticipant, zocksTracks) => {
    const {
      tracks: {
        videoTracks,
        audioTracks
      }
    } = stateParticipant;

    for (const stateVideoTrack of videoTracks) {
      const zocksVideoTrack = zocksTracks.find(t => t.id === stateVideoTrack.id);

      if (zocksVideoTrack && stateVideoTrack.isMuted !== zocksVideoTrack.isMuted) {
        changeMuteStatusOfTrack(stateParticipant.id, stateVideoTrack.id, zocksVideoTrack.isMuted);
      }
    }

    for (const stateAudioTrack of audioTracks) {
      const zocksAudioTrack = zocksTracks.find(t => t.id === stateAudioTrack.id);

      if (zocksAudioTrack && stateAudioTrack.isMuted !== zocksAudioTrack.isMuted) {
        changeMuteStatusOfTrack(stateParticipant.id, stateAudioTrack.id, zocksAudioTrack.isMuted);
      }
    }
  };

  const addMissingTracks = (stateParticipant, zocksParticipantTracks) => {
    const {
      id,
      tracks: {
        videoTracks,
        audioTracks,
        screenShareTrack
      }
    } = stateParticipant;

    for (const zocksTrack of zocksParticipantTracks) {
      const trackIsMissingFromState = !videoTracks.some(t => t.id === zocksTrack.id) && !audioTracks.some(t => t.id === zocksTrack.id) && (screenShareTrack === null || screenShareTrack === void 0 ? void 0 : screenShareTrack.id) !== zocksTrack.id;

      if (trackIsMissingFromState) {
        subscribeToTrack(id, zocksTrack);
      }
    }
  };

  const removeObsoleteTracks = (stateParticipant, zocksTracks) => {
    const {
      tracks: {
        videoTracks,
        audioTracks,
        screenShareTrack
      }
    } = stateParticipant;

    for (const stateVideoTrack of videoTracks) {
      if (!zocksTracks.some(t => t.id === stateVideoTrack.id)) {
        unsubscribeFromTrack(stateParticipant.id, stateVideoTrack.id, StreamSource.CAMERA, StreamKind.VIDEO);
      }
    }

    for (const stateAudioTrack of audioTracks) {
      if (!zocksTracks.some(t => t.id === stateAudioTrack.id)) {
        unsubscribeFromTrack(stateParticipant.id, stateAudioTrack.id, StreamSource.MICROPHONE, StreamKind.AUDIO);
      }
    }

    if (screenShareTrack && !zocksTracks.some(t => t.id === screenShareTrack.id)) {
      unsubscribeFromTrack(stateParticipant.id, screenShareTrack.id, StreamSource.SCREEN_SHARE, StreamKind.VIDEO);
    }
  };

  const subscribeToTrack = (participantId, track) => {
    dispatch(subscribeToParticipantTrack({
      participantId: participantId,
      trackId: track.id,
      trackKind: track.kind,
      trackSource: track.source,
      trackIsMuted: track.isMuted
    }));
  };

  const unsubscribeFromTrack = (participantId, trackId, trackSource, trackKind) => {
    dispatch(participantTrackUnPublished({
      participantId,
      trackId,
      trackSource,
      trackKind
    }));
  };

  const changeMuteStatusOfTrack = (participantId, trackId, shouldMute) => {
    if (shouldMute) {
      dispatch(participantTrackMuted({
        participantId,
        trackId
      }));
    } else {
      dispatch(participantTrackUnMuted({
        participantId,
        trackId
      }));
    }
  };

  return {
    synchronizeParticipants
  };
};

var ParticipantsViewType;

(function (ParticipantsViewType) {
  ParticipantsViewType[ParticipantsViewType["Grid"] = 0] = "Grid";
  ParticipantsViewType[ParticipantsViewType["Speaker"] = 1] = "Speaker";
})(ParticipantsViewType || (ParticipantsViewType = {}));

const initialState = {
  participantsView: {
    currentView: ParticipantsViewType.Grid
  }
};
const viewSlice = createSlice({
  name: 'view',
  initialState,
  reducers: {
    changeParticipantsView: (state, action) => {
      state.participantsView.currentView = action.payload;
    }
  }
});
const {
  changeParticipantsView
} = viewSlice.actions;
const selectCurrentParticipantsView = state => state.view.participantsView.currentView;
var viewReducer = viewSlice.reducer;

var jsxRuntime = {exports: {}};

var reactJsxRuntime_production_min = {};

/**
 * @license React
 * react-jsx-runtime.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

var f = require$$0,
    k = Symbol.for("react.element"),
    l = Symbol.for("react.fragment"),
    m = Object.prototype.hasOwnProperty,
    n = f.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED.ReactCurrentOwner,
    p = {
  key: !0,
  ref: !0,
  __self: !0,
  __source: !0
};

function q(c, a, g) {
  var b,
      d = {},
      e = null,
      h = null;
  void 0 !== g && (e = "" + g);
  void 0 !== a.key && (e = "" + a.key);
  void 0 !== a.ref && (h = a.ref);

  for (b in a) m.call(a, b) && !p.hasOwnProperty(b) && (d[b] = a[b]);

  if (c && c.defaultProps) for (b in a = c.defaultProps, a) void 0 === d[b] && (d[b] = a[b]);
  return {
    $$typeof: k,
    type: c,
    key: e,
    ref: h,
    props: d,
    _owner: n.current
  };
}

reactJsxRuntime_production_min.Fragment = l;
reactJsxRuntime_production_min.jsx = q;
reactJsxRuntime_production_min.jsxs = q;

var reactJsxRuntime_development = {};

/**
 * @license React
 * react-jsx-runtime.development.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

if (process.env.NODE_ENV !== "production") {
  (function () {

    var React = require$$0; // ATTENTION
    // When adding new symbols to this file,
    // Please consider also adding to 'react-devtools-shared/src/backend/ReactSymbols'
    // The Symbol used to tag the ReactElement-like types.

    var REACT_ELEMENT_TYPE = Symbol.for('react.element');
    var REACT_PORTAL_TYPE = Symbol.for('react.portal');
    var REACT_FRAGMENT_TYPE = Symbol.for('react.fragment');
    var REACT_STRICT_MODE_TYPE = Symbol.for('react.strict_mode');
    var REACT_PROFILER_TYPE = Symbol.for('react.profiler');
    var REACT_PROVIDER_TYPE = Symbol.for('react.provider');
    var REACT_CONTEXT_TYPE = Symbol.for('react.context');
    var REACT_FORWARD_REF_TYPE = Symbol.for('react.forward_ref');
    var REACT_SUSPENSE_TYPE = Symbol.for('react.suspense');
    var REACT_SUSPENSE_LIST_TYPE = Symbol.for('react.suspense_list');
    var REACT_MEMO_TYPE = Symbol.for('react.memo');
    var REACT_LAZY_TYPE = Symbol.for('react.lazy');
    var REACT_OFFSCREEN_TYPE = Symbol.for('react.offscreen');
    var MAYBE_ITERATOR_SYMBOL = Symbol.iterator;
    var FAUX_ITERATOR_SYMBOL = '@@iterator';

    function getIteratorFn(maybeIterable) {
      if (maybeIterable === null || typeof maybeIterable !== 'object') {
        return null;
      }

      var maybeIterator = MAYBE_ITERATOR_SYMBOL && maybeIterable[MAYBE_ITERATOR_SYMBOL] || maybeIterable[FAUX_ITERATOR_SYMBOL];

      if (typeof maybeIterator === 'function') {
        return maybeIterator;
      }

      return null;
    }

    var ReactSharedInternals = React.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED;

    function error(format) {
      {
        {
          for (var _len2 = arguments.length, args = new Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
            args[_key2 - 1] = arguments[_key2];
          }

          printWarning('error', format, args);
        }
      }
    }

    function printWarning(level, format, args) {
      // When changing this logic, you might want to also
      // update consoleWithStackDev.www.js as well.
      {
        var ReactDebugCurrentFrame = ReactSharedInternals.ReactDebugCurrentFrame;
        var stack = ReactDebugCurrentFrame.getStackAddendum();

        if (stack !== '') {
          format += '%s';
          args = args.concat([stack]);
        } // eslint-disable-next-line react-internal/safe-string-coercion


        var argsWithFormat = args.map(function (item) {
          return String(item);
        }); // Careful: RN currently depends on this prefix

        argsWithFormat.unshift('Warning: ' + format); // We intentionally don't use spread (or .apply) directly because it
        // breaks IE9: https://github.com/facebook/react/issues/13610
        // eslint-disable-next-line react-internal/no-production-logging

        Function.prototype.apply.call(console[level], console, argsWithFormat);
      }
    } // -----------------------------------------------------------------------------


    var enableScopeAPI = false; // Experimental Create Event Handle API.

    var enableCacheElement = false;
    var enableTransitionTracing = false; // No known bugs, but needs performance testing

    var enableLegacyHidden = false; // Enables unstable_avoidThisFallback feature in Fiber
    // stuff. Intended to enable React core members to more easily debug scheduling
    // issues in DEV builds.

    var enableDebugTracing = false; // Track which Fiber(s) schedule render work.

    var REACT_MODULE_REFERENCE;
    {
      REACT_MODULE_REFERENCE = Symbol.for('react.module.reference');
    }

    function isValidElementType(type) {
      if (typeof type === 'string' || typeof type === 'function') {
        return true;
      } // Note: typeof might be other than 'symbol' or 'number' (e.g. if it's a polyfill).


      if (type === REACT_FRAGMENT_TYPE || type === REACT_PROFILER_TYPE || enableDebugTracing || type === REACT_STRICT_MODE_TYPE || type === REACT_SUSPENSE_TYPE || type === REACT_SUSPENSE_LIST_TYPE || enableLegacyHidden || type === REACT_OFFSCREEN_TYPE || enableScopeAPI || enableCacheElement || enableTransitionTracing) {
        return true;
      }

      if (typeof type === 'object' && type !== null) {
        if (type.$$typeof === REACT_LAZY_TYPE || type.$$typeof === REACT_MEMO_TYPE || type.$$typeof === REACT_PROVIDER_TYPE || type.$$typeof === REACT_CONTEXT_TYPE || type.$$typeof === REACT_FORWARD_REF_TYPE || // This needs to include all possible module reference object
        // types supported by any Flight configuration anywhere since
        // we don't know which Flight build this will end up being used
        // with.
        type.$$typeof === REACT_MODULE_REFERENCE || type.getModuleId !== undefined) {
          return true;
        }
      }

      return false;
    }

    function getWrappedName(outerType, innerType, wrapperName) {
      var displayName = outerType.displayName;

      if (displayName) {
        return displayName;
      }

      var functionName = innerType.displayName || innerType.name || '';
      return functionName !== '' ? wrapperName + "(" + functionName + ")" : wrapperName;
    } // Keep in sync with react-reconciler/getComponentNameFromFiber


    function getContextName(type) {
      return type.displayName || 'Context';
    } // Note that the reconciler package should generally prefer to use getComponentNameFromFiber() instead.


    function getComponentNameFromType(type) {
      if (type == null) {
        // Host root, text node or just invalid type.
        return null;
      }

      {
        if (typeof type.tag === 'number') {
          error('Received an unexpected object in getComponentNameFromType(). ' + 'This is likely a bug in React. Please file an issue.');
        }
      }

      if (typeof type === 'function') {
        return type.displayName || type.name || null;
      }

      if (typeof type === 'string') {
        return type;
      }

      switch (type) {
        case REACT_FRAGMENT_TYPE:
          return 'Fragment';

        case REACT_PORTAL_TYPE:
          return 'Portal';

        case REACT_PROFILER_TYPE:
          return 'Profiler';

        case REACT_STRICT_MODE_TYPE:
          return 'StrictMode';

        case REACT_SUSPENSE_TYPE:
          return 'Suspense';

        case REACT_SUSPENSE_LIST_TYPE:
          return 'SuspenseList';
      }

      if (typeof type === 'object') {
        switch (type.$$typeof) {
          case REACT_CONTEXT_TYPE:
            var context = type;
            return getContextName(context) + '.Consumer';

          case REACT_PROVIDER_TYPE:
            var provider = type;
            return getContextName(provider._context) + '.Provider';

          case REACT_FORWARD_REF_TYPE:
            return getWrappedName(type, type.render, 'ForwardRef');

          case REACT_MEMO_TYPE:
            var outerName = type.displayName || null;

            if (outerName !== null) {
              return outerName;
            }

            return getComponentNameFromType(type.type) || 'Memo';

          case REACT_LAZY_TYPE:
            {
              var lazyComponent = type;
              var payload = lazyComponent._payload;
              var init = lazyComponent._init;

              try {
                return getComponentNameFromType(init(payload));
              } catch (x) {
                return null;
              }
            }
          // eslint-disable-next-line no-fallthrough
        }
      }

      return null;
    }

    var assign = Object.assign; // Helpers to patch console.logs to avoid logging during side-effect free
    // replaying on render function. This currently only patches the object
    // lazily which won't cover if the log function was extracted eagerly.
    // We could also eagerly patch the method.

    var disabledDepth = 0;
    var prevLog;
    var prevInfo;
    var prevWarn;
    var prevError;
    var prevGroup;
    var prevGroupCollapsed;
    var prevGroupEnd;

    function disabledLog() {}

    disabledLog.__reactDisabledLog = true;

    function disableLogs() {
      {
        if (disabledDepth === 0) {
          /* eslint-disable react-internal/no-production-logging */
          prevLog = console.log;
          prevInfo = console.info;
          prevWarn = console.warn;
          prevError = console.error;
          prevGroup = console.group;
          prevGroupCollapsed = console.groupCollapsed;
          prevGroupEnd = console.groupEnd; // https://github.com/facebook/react/issues/19099

          var props = {
            configurable: true,
            enumerable: true,
            value: disabledLog,
            writable: true
          }; // $FlowFixMe Flow thinks console is immutable.

          Object.defineProperties(console, {
            info: props,
            log: props,
            warn: props,
            error: props,
            group: props,
            groupCollapsed: props,
            groupEnd: props
          });
          /* eslint-enable react-internal/no-production-logging */
        }

        disabledDepth++;
      }
    }

    function reenableLogs() {
      {
        disabledDepth--;

        if (disabledDepth === 0) {
          /* eslint-disable react-internal/no-production-logging */
          var props = {
            configurable: true,
            enumerable: true,
            writable: true
          }; // $FlowFixMe Flow thinks console is immutable.

          Object.defineProperties(console, {
            log: assign({}, props, {
              value: prevLog
            }),
            info: assign({}, props, {
              value: prevInfo
            }),
            warn: assign({}, props, {
              value: prevWarn
            }),
            error: assign({}, props, {
              value: prevError
            }),
            group: assign({}, props, {
              value: prevGroup
            }),
            groupCollapsed: assign({}, props, {
              value: prevGroupCollapsed
            }),
            groupEnd: assign({}, props, {
              value: prevGroupEnd
            })
          });
          /* eslint-enable react-internal/no-production-logging */
        }

        if (disabledDepth < 0) {
          error('disabledDepth fell below zero. ' + 'This is a bug in React. Please file an issue.');
        }
      }
    }

    var ReactCurrentDispatcher = ReactSharedInternals.ReactCurrentDispatcher;
    var prefix;

    function describeBuiltInComponentFrame(name, source, ownerFn) {
      {
        if (prefix === undefined) {
          // Extract the VM specific prefix used by each line.
          try {
            throw Error();
          } catch (x) {
            var match = x.stack.trim().match(/\n( *(at )?)/);
            prefix = match && match[1] || '';
          }
        } // We use the prefix to ensure our stacks line up with native stack frames.


        return '\n' + prefix + name;
      }
    }

    var reentry = false;
    var componentFrameCache;
    {
      var PossiblyWeakMap = typeof WeakMap === 'function' ? WeakMap : Map;
      componentFrameCache = new PossiblyWeakMap();
    }

    function describeNativeComponentFrame(fn, construct) {
      // If something asked for a stack inside a fake render, it should get ignored.
      if (!fn || reentry) {
        return '';
      }

      {
        var frame = componentFrameCache.get(fn);

        if (frame !== undefined) {
          return frame;
        }
      }
      var control;
      reentry = true;
      var previousPrepareStackTrace = Error.prepareStackTrace; // $FlowFixMe It does accept undefined.

      Error.prepareStackTrace = undefined;
      var previousDispatcher;
      {
        previousDispatcher = ReactCurrentDispatcher.current; // Set the dispatcher in DEV because this might be call in the render function
        // for warnings.

        ReactCurrentDispatcher.current = null;
        disableLogs();
      }

      try {
        // This should throw.
        if (construct) {
          // Something should be setting the props in the constructor.
          var Fake = function () {
            throw Error();
          }; // $FlowFixMe


          Object.defineProperty(Fake.prototype, 'props', {
            set: function () {
              // We use a throwing setter instead of frozen or non-writable props
              // because that won't throw in a non-strict mode function.
              throw Error();
            }
          });

          if (typeof Reflect === 'object' && Reflect.construct) {
            // We construct a different control for this case to include any extra
            // frames added by the construct call.
            try {
              Reflect.construct(Fake, []);
            } catch (x) {
              control = x;
            }

            Reflect.construct(fn, [], Fake);
          } else {
            try {
              Fake.call();
            } catch (x) {
              control = x;
            }

            fn.call(Fake.prototype);
          }
        } else {
          try {
            throw Error();
          } catch (x) {
            control = x;
          }

          fn();
        }
      } catch (sample) {
        // This is inlined manually because closure doesn't do it for us.
        if (sample && control && typeof sample.stack === 'string') {
          // This extracts the first frame from the sample that isn't also in the control.
          // Skipping one frame that we assume is the frame that calls the two.
          var sampleLines = sample.stack.split('\n');
          var controlLines = control.stack.split('\n');
          var s = sampleLines.length - 1;
          var c = controlLines.length - 1;

          while (s >= 1 && c >= 0 && sampleLines[s] !== controlLines[c]) {
            // We expect at least one stack frame to be shared.
            // Typically this will be the root most one. However, stack frames may be
            // cut off due to maximum stack limits. In this case, one maybe cut off
            // earlier than the other. We assume that the sample is longer or the same
            // and there for cut off earlier. So we should find the root most frame in
            // the sample somewhere in the control.
            c--;
          }

          for (; s >= 1 && c >= 0; s--, c--) {
            // Next we find the first one that isn't the same which should be the
            // frame that called our sample function and the control.
            if (sampleLines[s] !== controlLines[c]) {
              // In V8, the first line is describing the message but other VMs don't.
              // If we're about to return the first line, and the control is also on the same
              // line, that's a pretty good indicator that our sample threw at same line as
              // the control. I.e. before we entered the sample frame. So we ignore this result.
              // This can happen if you passed a class to function component, or non-function.
              if (s !== 1 || c !== 1) {
                do {
                  s--;
                  c--; // We may still have similar intermediate frames from the construct call.
                  // The next one that isn't the same should be our match though.

                  if (c < 0 || sampleLines[s] !== controlLines[c]) {
                    // V8 adds a "new" prefix for native classes. Let's remove it to make it prettier.
                    var _frame = '\n' + sampleLines[s].replace(' at new ', ' at '); // If our component frame is labeled "<anonymous>"
                    // but we have a user-provided "displayName"
                    // splice it in to make the stack more readable.


                    if (fn.displayName && _frame.includes('<anonymous>')) {
                      _frame = _frame.replace('<anonymous>', fn.displayName);
                    }

                    {
                      if (typeof fn === 'function') {
                        componentFrameCache.set(fn, _frame);
                      }
                    } // Return the line we found.

                    return _frame;
                  }
                } while (s >= 1 && c >= 0);
              }

              break;
            }
          }
        }
      } finally {
        reentry = false;
        {
          ReactCurrentDispatcher.current = previousDispatcher;
          reenableLogs();
        }
        Error.prepareStackTrace = previousPrepareStackTrace;
      } // Fallback to just using the name if we couldn't make it throw.


      var name = fn ? fn.displayName || fn.name : '';
      var syntheticFrame = name ? describeBuiltInComponentFrame(name) : '';
      {
        if (typeof fn === 'function') {
          componentFrameCache.set(fn, syntheticFrame);
        }
      }
      return syntheticFrame;
    }

    function describeFunctionComponentFrame(fn, source, ownerFn) {
      {
        return describeNativeComponentFrame(fn, false);
      }
    }

    function shouldConstruct(Component) {
      var prototype = Component.prototype;
      return !!(prototype && prototype.isReactComponent);
    }

    function describeUnknownElementTypeFrameInDEV(type, source, ownerFn) {
      if (type == null) {
        return '';
      }

      if (typeof type === 'function') {
        {
          return describeNativeComponentFrame(type, shouldConstruct(type));
        }
      }

      if (typeof type === 'string') {
        return describeBuiltInComponentFrame(type);
      }

      switch (type) {
        case REACT_SUSPENSE_TYPE:
          return describeBuiltInComponentFrame('Suspense');

        case REACT_SUSPENSE_LIST_TYPE:
          return describeBuiltInComponentFrame('SuspenseList');
      }

      if (typeof type === 'object') {
        switch (type.$$typeof) {
          case REACT_FORWARD_REF_TYPE:
            return describeFunctionComponentFrame(type.render);

          case REACT_MEMO_TYPE:
            // Memo may contain any component type so we recursively resolve it.
            return describeUnknownElementTypeFrameInDEV(type.type, source, ownerFn);

          case REACT_LAZY_TYPE:
            {
              var lazyComponent = type;
              var payload = lazyComponent._payload;
              var init = lazyComponent._init;

              try {
                // Lazy may contain any component type so we recursively resolve it.
                return describeUnknownElementTypeFrameInDEV(init(payload), source, ownerFn);
              } catch (x) {}
            }
        }
      }

      return '';
    }

    var hasOwnProperty = Object.prototype.hasOwnProperty;
    var loggedTypeFailures = {};
    var ReactDebugCurrentFrame = ReactSharedInternals.ReactDebugCurrentFrame;

    function setCurrentlyValidatingElement(element) {
      {
        if (element) {
          var owner = element._owner;
          var stack = describeUnknownElementTypeFrameInDEV(element.type, element._source, owner ? owner.type : null);
          ReactDebugCurrentFrame.setExtraStackFrame(stack);
        } else {
          ReactDebugCurrentFrame.setExtraStackFrame(null);
        }
      }
    }

    function checkPropTypes(typeSpecs, values, location, componentName, element) {
      {
        // $FlowFixMe This is okay but Flow doesn't know it.
        var has = Function.call.bind(hasOwnProperty);

        for (var typeSpecName in typeSpecs) {
          if (has(typeSpecs, typeSpecName)) {
            var error$1 = void 0; // Prop type validation may throw. In case they do, we don't want to
            // fail the render phase where it didn't fail before. So we log it.
            // After these have been cleaned up, we'll let them throw.

            try {
              // This is intentionally an invariant that gets caught. It's the same
              // behavior as without this statement except with a better message.
              if (typeof typeSpecs[typeSpecName] !== 'function') {
                // eslint-disable-next-line react-internal/prod-error-codes
                var err = Error((componentName || 'React class') + ': ' + location + ' type `' + typeSpecName + '` is invalid; ' + 'it must be a function, usually from the `prop-types` package, but received `' + typeof typeSpecs[typeSpecName] + '`.' + 'This often happens because of typos such as `PropTypes.function` instead of `PropTypes.func`.');
                err.name = 'Invariant Violation';
                throw err;
              }

              error$1 = typeSpecs[typeSpecName](values, typeSpecName, componentName, location, null, 'SECRET_DO_NOT_PASS_THIS_OR_YOU_WILL_BE_FIRED');
            } catch (ex) {
              error$1 = ex;
            }

            if (error$1 && !(error$1 instanceof Error)) {
              setCurrentlyValidatingElement(element);
              error('%s: type specification of %s' + ' `%s` is invalid; the type checker ' + 'function must return `null` or an `Error` but returned a %s. ' + 'You may have forgotten to pass an argument to the type checker ' + 'creator (arrayOf, instanceOf, objectOf, oneOf, oneOfType, and ' + 'shape all require an argument).', componentName || 'React class', location, typeSpecName, typeof error$1);
              setCurrentlyValidatingElement(null);
            }

            if (error$1 instanceof Error && !(error$1.message in loggedTypeFailures)) {
              // Only monitor this failure once because there tends to be a lot of the
              // same error.
              loggedTypeFailures[error$1.message] = true;
              setCurrentlyValidatingElement(element);
              error('Failed %s type: %s', location, error$1.message);
              setCurrentlyValidatingElement(null);
            }
          }
        }
      }
    }

    var isArrayImpl = Array.isArray; // eslint-disable-next-line no-redeclare

    function isArray(a) {
      return isArrayImpl(a);
    }
    /*
     * The `'' + value` pattern (used in in perf-sensitive code) throws for Symbol
     * and Temporal.* types. See https://github.com/facebook/react/pull/22064.
     *
     * The functions in this module will throw an easier-to-understand,
     * easier-to-debug exception with a clear errors message message explaining the
     * problem. (Instead of a confusing exception thrown inside the implementation
     * of the `value` object).
     */
    // $FlowFixMe only called in DEV, so void return is not possible.


    function typeName(value) {
      {
        // toStringTag is needed for namespaced types like Temporal.Instant
        var hasToStringTag = typeof Symbol === 'function' && Symbol.toStringTag;
        var type = hasToStringTag && value[Symbol.toStringTag] || value.constructor.name || 'Object';
        return type;
      }
    } // $FlowFixMe only called in DEV, so void return is not possible.


    function willCoercionThrow(value) {
      {
        try {
          testStringCoercion(value);
          return false;
        } catch (e) {
          return true;
        }
      }
    }

    function testStringCoercion(value) {
      // If you ended up here by following an exception call stack, here's what's
      // happened: you supplied an object or symbol value to React (as a prop, key,
      // DOM attribute, CSS property, string ref, etc.) and when React tried to
      // coerce it to a string using `'' + value`, an exception was thrown.
      //
      // The most common types that will cause this exception are `Symbol` instances
      // and Temporal objects like `Temporal.Instant`. But any object that has a
      // `valueOf` or `[Symbol.toPrimitive]` method that throws will also cause this
      // exception. (Library authors do this to prevent users from using built-in
      // numeric operators like `+` or comparison operators like `>=` because custom
      // methods are needed to perform accurate arithmetic or comparison.)
      //
      // To fix the problem, coerce this object or symbol value to a string before
      // passing it to React. The most reliable way is usually `String(value)`.
      //
      // To find which value is throwing, check the browser or debugger console.
      // Before this exception was thrown, there should be `console.error` output
      // that shows the type (Symbol, Temporal.PlainDate, etc.) that caused the
      // problem and how that type was used: key, atrribute, input value prop, etc.
      // In most cases, this console output also shows the component and its
      // ancestor components where the exception happened.
      //
      // eslint-disable-next-line react-internal/safe-string-coercion
      return '' + value;
    }

    function checkKeyStringCoercion(value) {
      {
        if (willCoercionThrow(value)) {
          error('The provided key is an unsupported type %s.' + ' This value must be coerced to a string before before using it here.', typeName(value));
          return testStringCoercion(value); // throw (to help callers find troubleshooting comments)
        }
      }
    }

    var ReactCurrentOwner = ReactSharedInternals.ReactCurrentOwner;
    var RESERVED_PROPS = {
      key: true,
      ref: true,
      __self: true,
      __source: true
    };
    var specialPropKeyWarningShown;
    var specialPropRefWarningShown;
    var didWarnAboutStringRefs;
    {
      didWarnAboutStringRefs = {};
    }

    function hasValidRef(config) {
      {
        if (hasOwnProperty.call(config, 'ref')) {
          var getter = Object.getOwnPropertyDescriptor(config, 'ref').get;

          if (getter && getter.isReactWarning) {
            return false;
          }
        }
      }
      return config.ref !== undefined;
    }

    function hasValidKey(config) {
      {
        if (hasOwnProperty.call(config, 'key')) {
          var getter = Object.getOwnPropertyDescriptor(config, 'key').get;

          if (getter && getter.isReactWarning) {
            return false;
          }
        }
      }
      return config.key !== undefined;
    }

    function warnIfStringRefCannotBeAutoConverted(config, self) {
      {
        if (typeof config.ref === 'string' && ReactCurrentOwner.current && self && ReactCurrentOwner.current.stateNode !== self) {
          var componentName = getComponentNameFromType(ReactCurrentOwner.current.type);

          if (!didWarnAboutStringRefs[componentName]) {
            error('Component "%s" contains the string ref "%s". ' + 'Support for string refs will be removed in a future major release. ' + 'This case cannot be automatically converted to an arrow function. ' + 'We ask you to manually fix this case by using useRef() or createRef() instead. ' + 'Learn more about using refs safely here: ' + 'https://reactjs.org/link/strict-mode-string-ref', getComponentNameFromType(ReactCurrentOwner.current.type), config.ref);
            didWarnAboutStringRefs[componentName] = true;
          }
        }
      }
    }

    function defineKeyPropWarningGetter(props, displayName) {
      {
        var warnAboutAccessingKey = function () {
          if (!specialPropKeyWarningShown) {
            specialPropKeyWarningShown = true;
            error('%s: `key` is not a prop. Trying to access it will result ' + 'in `undefined` being returned. If you need to access the same ' + 'value within the child component, you should pass it as a different ' + 'prop. (https://reactjs.org/link/special-props)', displayName);
          }
        };

        warnAboutAccessingKey.isReactWarning = true;
        Object.defineProperty(props, 'key', {
          get: warnAboutAccessingKey,
          configurable: true
        });
      }
    }

    function defineRefPropWarningGetter(props, displayName) {
      {
        var warnAboutAccessingRef = function () {
          if (!specialPropRefWarningShown) {
            specialPropRefWarningShown = true;
            error('%s: `ref` is not a prop. Trying to access it will result ' + 'in `undefined` being returned. If you need to access the same ' + 'value within the child component, you should pass it as a different ' + 'prop. (https://reactjs.org/link/special-props)', displayName);
          }
        };

        warnAboutAccessingRef.isReactWarning = true;
        Object.defineProperty(props, 'ref', {
          get: warnAboutAccessingRef,
          configurable: true
        });
      }
    }
    /**
     * Factory method to create a new React element. This no longer adheres to
     * the class pattern, so do not use new to call it. Also, instanceof check
     * will not work. Instead test $$typeof field against Symbol.for('react.element') to check
     * if something is a React Element.
     *
     * @param {*} type
     * @param {*} props
     * @param {*} key
     * @param {string|object} ref
     * @param {*} owner
     * @param {*} self A *temporary* helper to detect places where `this` is
     * different from the `owner` when React.createElement is called, so that we
     * can warn. We want to get rid of owner and replace string `ref`s with arrow
     * functions, and as long as `this` and owner are the same, there will be no
     * change in behavior.
     * @param {*} source An annotation object (added by a transpiler or otherwise)
     * indicating filename, line number, and/or other information.
     * @internal
     */


    var ReactElement = function (type, key, ref, self, source, owner, props) {
      var element = {
        // This tag allows us to uniquely identify this as a React Element
        $$typeof: REACT_ELEMENT_TYPE,
        // Built-in properties that belong on the element
        type: type,
        key: key,
        ref: ref,
        props: props,
        // Record the component responsible for creating this element.
        _owner: owner
      };
      {
        // The validation flag is currently mutative. We put it on
        // an external backing store so that we can freeze the whole object.
        // This can be replaced with a WeakMap once they are implemented in
        // commonly used development environments.
        element._store = {}; // To make comparing ReactElements easier for testing purposes, we make
        // the validation flag non-enumerable (where possible, which should
        // include every environment we run tests in), so the test framework
        // ignores it.

        Object.defineProperty(element._store, 'validated', {
          configurable: false,
          enumerable: false,
          writable: true,
          value: false
        }); // self and source are DEV only properties.

        Object.defineProperty(element, '_self', {
          configurable: false,
          enumerable: false,
          writable: false,
          value: self
        }); // Two elements created in two different places should be considered
        // equal for testing purposes and therefore we hide it from enumeration.

        Object.defineProperty(element, '_source', {
          configurable: false,
          enumerable: false,
          writable: false,
          value: source
        });

        if (Object.freeze) {
          Object.freeze(element.props);
          Object.freeze(element);
        }
      }
      return element;
    };
    /**
     * https://github.com/reactjs/rfcs/pull/107
     * @param {*} type
     * @param {object} props
     * @param {string} key
     */


    function jsxDEV(type, config, maybeKey, source, self) {
      {
        var propName; // Reserved names are extracted

        var props = {};
        var key = null;
        var ref = null; // Currently, key can be spread in as a prop. This causes a potential
        // issue if key is also explicitly declared (ie. <div {...props} key="Hi" />
        // or <div key="Hi" {...props} /> ). We want to deprecate key spread,
        // but as an intermediary step, we will use jsxDEV for everything except
        // <div {...props} key="Hi" />, because we aren't currently able to tell if
        // key is explicitly declared to be undefined or not.

        if (maybeKey !== undefined) {
          {
            checkKeyStringCoercion(maybeKey);
          }
          key = '' + maybeKey;
        }

        if (hasValidKey(config)) {
          {
            checkKeyStringCoercion(config.key);
          }
          key = '' + config.key;
        }

        if (hasValidRef(config)) {
          ref = config.ref;
          warnIfStringRefCannotBeAutoConverted(config, self);
        } // Remaining properties are added to a new props object


        for (propName in config) {
          if (hasOwnProperty.call(config, propName) && !RESERVED_PROPS.hasOwnProperty(propName)) {
            props[propName] = config[propName];
          }
        } // Resolve default props


        if (type && type.defaultProps) {
          var defaultProps = type.defaultProps;

          for (propName in defaultProps) {
            if (props[propName] === undefined) {
              props[propName] = defaultProps[propName];
            }
          }
        }

        if (key || ref) {
          var displayName = typeof type === 'function' ? type.displayName || type.name || 'Unknown' : type;

          if (key) {
            defineKeyPropWarningGetter(props, displayName);
          }

          if (ref) {
            defineRefPropWarningGetter(props, displayName);
          }
        }

        return ReactElement(type, key, ref, self, source, ReactCurrentOwner.current, props);
      }
    }

    var ReactCurrentOwner$1 = ReactSharedInternals.ReactCurrentOwner;
    var ReactDebugCurrentFrame$1 = ReactSharedInternals.ReactDebugCurrentFrame;

    function setCurrentlyValidatingElement$1(element) {
      {
        if (element) {
          var owner = element._owner;
          var stack = describeUnknownElementTypeFrameInDEV(element.type, element._source, owner ? owner.type : null);
          ReactDebugCurrentFrame$1.setExtraStackFrame(stack);
        } else {
          ReactDebugCurrentFrame$1.setExtraStackFrame(null);
        }
      }
    }

    var propTypesMisspellWarningShown;
    {
      propTypesMisspellWarningShown = false;
    }
    /**
     * Verifies the object is a ReactElement.
     * See https://reactjs.org/docs/react-api.html#isvalidelement
     * @param {?object} object
     * @return {boolean} True if `object` is a ReactElement.
     * @final
     */

    function isValidElement(object) {
      {
        return typeof object === 'object' && object !== null && object.$$typeof === REACT_ELEMENT_TYPE;
      }
    }

    function getDeclarationErrorAddendum() {
      {
        if (ReactCurrentOwner$1.current) {
          var name = getComponentNameFromType(ReactCurrentOwner$1.current.type);

          if (name) {
            return '\n\nCheck the render method of `' + name + '`.';
          }
        }

        return '';
      }
    }

    function getSourceInfoErrorAddendum(source) {
      {
        if (source !== undefined) {
          var fileName = source.fileName.replace(/^.*[\\\/]/, '');
          var lineNumber = source.lineNumber;
          return '\n\nCheck your code at ' + fileName + ':' + lineNumber + '.';
        }

        return '';
      }
    }
    /**
     * Warn if there's no key explicitly set on dynamic arrays of children or
     * object keys are not valid. This allows us to keep track of children between
     * updates.
     */


    var ownerHasKeyUseWarning = {};

    function getCurrentComponentErrorInfo(parentType) {
      {
        var info = getDeclarationErrorAddendum();

        if (!info) {
          var parentName = typeof parentType === 'string' ? parentType : parentType.displayName || parentType.name;

          if (parentName) {
            info = "\n\nCheck the top-level render call using <" + parentName + ">.";
          }
        }

        return info;
      }
    }
    /**
     * Warn if the element doesn't have an explicit key assigned to it.
     * This element is in an array. The array could grow and shrink or be
     * reordered. All children that haven't already been validated are required to
     * have a "key" property assigned to it. Error statuses are cached so a warning
     * will only be shown once.
     *
     * @internal
     * @param {ReactElement} element Element that requires a key.
     * @param {*} parentType element's parent's type.
     */


    function validateExplicitKey(element, parentType) {
      {
        if (!element._store || element._store.validated || element.key != null) {
          return;
        }

        element._store.validated = true;
        var currentComponentErrorInfo = getCurrentComponentErrorInfo(parentType);

        if (ownerHasKeyUseWarning[currentComponentErrorInfo]) {
          return;
        }

        ownerHasKeyUseWarning[currentComponentErrorInfo] = true; // Usually the current owner is the offender, but if it accepts children as a
        // property, it may be the creator of the child that's responsible for
        // assigning it a key.

        var childOwner = '';

        if (element && element._owner && element._owner !== ReactCurrentOwner$1.current) {
          // Give the component that originally created this child.
          childOwner = " It was passed a child from " + getComponentNameFromType(element._owner.type) + ".";
        }

        setCurrentlyValidatingElement$1(element);
        error('Each child in a list should have a unique "key" prop.' + '%s%s See https://reactjs.org/link/warning-keys for more information.', currentComponentErrorInfo, childOwner);
        setCurrentlyValidatingElement$1(null);
      }
    }
    /**
     * Ensure that every element either is passed in a static location, in an
     * array with an explicit keys property defined, or in an object literal
     * with valid key property.
     *
     * @internal
     * @param {ReactNode} node Statically passed child of any type.
     * @param {*} parentType node's parent's type.
     */


    function validateChildKeys(node, parentType) {
      {
        if (typeof node !== 'object') {
          return;
        }

        if (isArray(node)) {
          for (var i = 0; i < node.length; i++) {
            var child = node[i];

            if (isValidElement(child)) {
              validateExplicitKey(child, parentType);
            }
          }
        } else if (isValidElement(node)) {
          // This element was passed in a valid location.
          if (node._store) {
            node._store.validated = true;
          }
        } else if (node) {
          var iteratorFn = getIteratorFn(node);

          if (typeof iteratorFn === 'function') {
            // Entry iterators used to provide implicit keys,
            // but now we print a separate warning for them later.
            if (iteratorFn !== node.entries) {
              var iterator = iteratorFn.call(node);
              var step;

              while (!(step = iterator.next()).done) {
                if (isValidElement(step.value)) {
                  validateExplicitKey(step.value, parentType);
                }
              }
            }
          }
        }
      }
    }
    /**
     * Given an element, validate that its props follow the propTypes definition,
     * provided by the type.
     *
     * @param {ReactElement} element
     */


    function validatePropTypes(element) {
      {
        var type = element.type;

        if (type === null || type === undefined || typeof type === 'string') {
          return;
        }

        var propTypes;

        if (typeof type === 'function') {
          propTypes = type.propTypes;
        } else if (typeof type === 'object' && (type.$$typeof === REACT_FORWARD_REF_TYPE || // Note: Memo only checks outer props here.
        // Inner props are checked in the reconciler.
        type.$$typeof === REACT_MEMO_TYPE)) {
          propTypes = type.propTypes;
        } else {
          return;
        }

        if (propTypes) {
          // Intentionally inside to avoid triggering lazy initializers:
          var name = getComponentNameFromType(type);
          checkPropTypes(propTypes, element.props, 'prop', name, element);
        } else if (type.PropTypes !== undefined && !propTypesMisspellWarningShown) {
          propTypesMisspellWarningShown = true; // Intentionally inside to avoid triggering lazy initializers:

          var _name = getComponentNameFromType(type);

          error('Component %s declared `PropTypes` instead of `propTypes`. Did you misspell the property assignment?', _name || 'Unknown');
        }

        if (typeof type.getDefaultProps === 'function' && !type.getDefaultProps.isReactClassApproved) {
          error('getDefaultProps is only used on classic React.createClass ' + 'definitions. Use a static property named `defaultProps` instead.');
        }
      }
    }
    /**
     * Given a fragment, validate that it can only be provided with fragment props
     * @param {ReactElement} fragment
     */


    function validateFragmentProps(fragment) {
      {
        var keys = Object.keys(fragment.props);

        for (var i = 0; i < keys.length; i++) {
          var key = keys[i];

          if (key !== 'children' && key !== 'key') {
            setCurrentlyValidatingElement$1(fragment);
            error('Invalid prop `%s` supplied to `React.Fragment`. ' + 'React.Fragment can only have `key` and `children` props.', key);
            setCurrentlyValidatingElement$1(null);
            break;
          }
        }

        if (fragment.ref !== null) {
          setCurrentlyValidatingElement$1(fragment);
          error('Invalid attribute `ref` supplied to `React.Fragment`.');
          setCurrentlyValidatingElement$1(null);
        }
      }
    }

    function jsxWithValidation(type, props, key, isStaticChildren, source, self) {
      {
        var validType = isValidElementType(type); // We warn in this case but don't throw. We expect the element creation to
        // succeed and there will likely be errors in render.

        if (!validType) {
          var info = '';

          if (type === undefined || typeof type === 'object' && type !== null && Object.keys(type).length === 0) {
            info += ' You likely forgot to export your component from the file ' + "it's defined in, or you might have mixed up default and named imports.";
          }

          var sourceInfo = getSourceInfoErrorAddendum(source);

          if (sourceInfo) {
            info += sourceInfo;
          } else {
            info += getDeclarationErrorAddendum();
          }

          var typeString;

          if (type === null) {
            typeString = 'null';
          } else if (isArray(type)) {
            typeString = 'array';
          } else if (type !== undefined && type.$$typeof === REACT_ELEMENT_TYPE) {
            typeString = "<" + (getComponentNameFromType(type.type) || 'Unknown') + " />";
            info = ' Did you accidentally export a JSX literal instead of a component?';
          } else {
            typeString = typeof type;
          }

          error('React.jsx: type is invalid -- expected a string (for ' + 'built-in components) or a class/function (for composite ' + 'components) but got: %s.%s', typeString, info);
        }

        var element = jsxDEV(type, props, key, source, self); // The result can be nullish if a mock or a custom function is used.
        // TODO: Drop this when these are no longer allowed as the type argument.

        if (element == null) {
          return element;
        } // Skip key warning if the type isn't valid since our key validation logic
        // doesn't expect a non-string/function type and can throw confusing errors.
        // We don't want exception behavior to differ between dev and prod.
        // (Rendering will throw with a helpful message and as soon as the type is
        // fixed, the key warnings will appear.)


        if (validType) {
          var children = props.children;

          if (children !== undefined) {
            if (isStaticChildren) {
              if (isArray(children)) {
                for (var i = 0; i < children.length; i++) {
                  validateChildKeys(children[i], type);
                }

                if (Object.freeze) {
                  Object.freeze(children);
                }
              } else {
                error('React.jsx: Static children should always be an array. ' + 'You are likely explicitly calling React.jsxs or React.jsxDEV. ' + 'Use the Babel transform instead.');
              }
            } else {
              validateChildKeys(children, type);
            }
          }
        }

        if (type === REACT_FRAGMENT_TYPE) {
          validateFragmentProps(element);
        } else {
          validatePropTypes(element);
        }

        return element;
      }
    } // These two functions exist to still get child warnings in dev
    // even with the prod transform. This means that jsxDEV is purely
    // opt-in behavior for better messages but that we won't stop
    // giving you warnings if you use production apis.


    function jsxWithValidationStatic(type, props, key) {
      {
        return jsxWithValidation(type, props, key, true);
      }
    }

    function jsxWithValidationDynamic(type, props, key) {
      {
        return jsxWithValidation(type, props, key, false);
      }
    }

    var jsx = jsxWithValidationDynamic; // we may want to special case jsxs internally to take advantage of static children.
    // for now we can ship identical prod functions

    var jsxs = jsxWithValidationStatic;
    reactJsxRuntime_development.Fragment = REACT_FRAGMENT_TYPE;
    reactJsxRuntime_development.jsx = jsx;
    reactJsxRuntime_development.jsxs = jsxs;
  })();
}

if (process.env.NODE_ENV === 'production') {
  jsxRuntime.exports = reactJsxRuntime_production_min;
} else {
  jsxRuntime.exports = reactJsxRuntime_development;
}

const zocksStoreConfig = {
  reducer: {
    session: sessionReducer,
    view: viewReducer,
    chat: chatReducer
  }
};
configureStore(zocksStoreConfig);

const ZocksProvider = ({ children }) => {
    const store = configureStore(zocksStoreConfig);
    return jsxRuntime.exports.jsx(Provider, { store: store, children: children });
};

const useChat = chatActions => {
  const {
    sendPublicMessage,
    sendPrivateMessage,
    uploadAttachment,
    getAttachmentLocation
  } = chatActions;
  const publicChatMessages = useZocksStore(selectPublicChatMessages);
  const privateChatMessages = useZocksStore(selectPrivateChatMessages);
  return {
    sendPublicMessage,
    sendPrivateMessage,
    uploadAttachment,
    getAttachmentLocation,
    publicChatMessages,
    privateChatMessages
  };
};
const useChatAttachment = (attachment, getAttachmentLocation) => {
  const anchorRef = useRef(null);
  const [isAttachmentLoading, setAttachmentLoading] = useState(false);
  const [hasAttachmentError, setAttachmentError] = useState(false);
  const [attachmentUrl, setAttachmentUrl] = useState('');

  const handleDownloadFailed = error => {
    console.error(error);
    setAttachmentError(true);
  };

  const downloadAttachment = () => {
    setAttachmentLoading(true);
    getAttachmentLocation(attachment.fileId).then(url => {
      if (url && anchorRef && anchorRef.current) {
        setAttachmentUrl(url);
        anchorRef.current.href = url;
        anchorRef.current.click();
      }
    }).catch(error => handleDownloadFailed(error)).finally(() => setAttachmentLoading(false));
  };

  return {
    anchorRef,
    isAttachmentLoading,
    hasAttachmentError,
    attachmentUrl,
    downloadAttachment
  };
};

var classnames = {exports: {}};

/*!
  Copyright (c) 2018 Jed Watson.
  Licensed under the MIT License (MIT), see
  http://jedwatson.github.io/classnames
*/

(function (module) {
  /* global define */
  (function () {

    var hasOwn = {}.hasOwnProperty;

    function classNames() {
      var classes = [];

      for (var i = 0; i < arguments.length; i++) {
        var arg = arguments[i];
        if (!arg) continue;
        var argType = typeof arg;

        if (argType === 'string' || argType === 'number') {
          classes.push(arg);
        } else if (Array.isArray(arg)) {
          if (arg.length) {
            var inner = classNames.apply(null, arg);

            if (inner) {
              classes.push(inner);
            }
          }
        } else if (argType === 'object') {
          if (arg.toString === Object.prototype.toString) {
            for (var key in arg) {
              if (hasOwn.call(arg, key) && arg[key]) {
                classes.push(key);
              }
            }
          } else {
            classes.push(arg.toString());
          }
        }
      }

      return classes.join(' ');
    }

    if (module.exports) {
      classNames.default = classNames;
      module.exports = classNames;
    } else {
      window.classNames = classNames;
    }
  })();
})(classnames);

var classNames = classnames.exports;

const AttachmentUploader = ({ uploadAttachment, attachmentRecipient, onAttachmentUploadStarted, onAttachmentUploadFinished, onAttachmentUploadFailed, uploadAttachmentButtonElement, }) => {
    const fileInputRef = useRef(null);
    const [isUploading, setUploading] = useState(false);
    const [hasError, setHasError] = useState(false);
    const buttonClickHandler = () => {
        var _a;
        (_a = fileInputRef.current) === null || _a === void 0 ? void 0 : _a.click();
    };
    const uploadStarted = (file) => {
        setHasError(false);
        setUploading(true);
        onAttachmentUploadStarted && onAttachmentUploadStarted(file);
    };
    const uploadFinished = (file, fileId) => {
        if (fileId) {
            onAttachmentUploadFinished && onAttachmentUploadFinished(file, fileId);
        }
        else {
            uploadFailed(file, new Error('No file id returned.'));
        }
    };
    const uploadFailed = (file, error) => {
        console.error(error);
        setHasError(true);
        onAttachmentUploadFailed && onAttachmentUploadFailed(file);
    };
    const fileUploadHandler = (e) => {
        const { target: { files }, } = e;
        if (files === null || files === void 0 ? void 0 : files.length) {
            const file = files[0];
            uploadStarted(file);
            uploadAttachment(file, attachmentRecipient)
                .then((fileId) => uploadFinished(file, fileId))
                .catch((error) => uploadFailed(file, error))
                .finally(() => setUploading(false));
            e.target.value = '';
        }
    };
    const UploadAttachmentButtonElement = uploadAttachmentButtonElement !== null && uploadAttachmentButtonElement !== void 0 ? uploadAttachmentButtonElement : UploadAttachmentButton;
    return (jsxRuntime.exports.jsxs("div", { className: 'attachment-uploader__container', children: [jsxRuntime.exports.jsx(UploadAttachmentButtonElement, { onClick: buttonClickHandler, disabled: isUploading, hasError: hasError }), jsxRuntime.exports.jsx("input", { ref: fileInputRef, className: 'attachment-uploader__file-input', type: 'file', onChange: fileUploadHandler, disabled: isUploading, "aria-hidden": true, "data-testid": 'hidden-file-input' })] }));
};
const UploadAttachmentButton = ({ onClick, disabled, hasError }) => (jsxRuntime.exports.jsx("button", { className: classNames('attachment-uploader__open-button', {
        'attachment-uploader__open-button--error': hasError,
    }), "aria-label": 'Upload a file', onClick: onClick, disabled: disabled, children: "\uD83D\uDCCE" }));
const AttachmentDisplay = ({ attachment, onClearAttachment, hasAttachmentError, clearAttachmentButtonElement, }) => {
    if (hasAttachmentError) {
        return jsxRuntime.exports.jsx("div", { className: 'attachment-display__error', children: 'Error while uploading attachment.' });
    }
    const ClearAttachmentButtonElement = clearAttachmentButtonElement !== null && clearAttachmentButtonElement !== void 0 ? clearAttachmentButtonElement : ClearAttachmentButton;
    if (attachment) {
        return (jsxRuntime.exports.jsxs("div", { className: 'attachment-display__container', children: [jsxRuntime.exports.jsx("span", { className: 'attachment-display__name', title: attachment.fileName, children: attachment.fileName }), jsxRuntime.exports.jsx(ClearAttachmentButtonElement, { onClick: onClearAttachment })] }));
    }
    return null;
};
const ClearAttachmentButton = ({ onClick }) => (jsxRuntime.exports.jsx("button", { className: 'attachment-display__remove-button', onClick: onClick, "aria-label": 'Remove attachment', children: "\u274C" }));

const ChatMessageInput = require$$0.forwardRef(({ currentChatMessage, onChatMessageChanged, sendMessage, placeholderText = 'Type a message…' }, ref) => {
    const MESSAGE_MAX_LENGTH = 1000;
    const keyDownHandler = (evt) => {
        if (evt.key === 'Enter' && !evt.shiftKey) {
            evt.preventDefault();
            sendMessage && sendMessage();
        }
    };
    return (jsxRuntime.exports.jsx("textarea", { ref: ref, className: 'chat-input__message-textarea', value: currentChatMessage, onChange: (evt) => onChatMessageChanged(evt.target.value), onKeyDown: keyDownHandler, placeholder: placeholderText, maxLength: MESSAGE_MAX_LENGTH }));
});

const RecipientSelector = ({ remoteParticipants, onRecipientSelected, selectInfoText = 'To', allParticipantsLabel = 'Everyone', }) => {
    return (jsxRuntime.exports.jsxs("div", { className: 'chat-input__recipient-select-container', children: [jsxRuntime.exports.jsx("div", { children: selectInfoText }), jsxRuntime.exports.jsxs("select", { className: 'chat-input__recipient-select', defaultValue: '', onChange: (evt) => onRecipientSelected(evt.target.value), children: [jsxRuntime.exports.jsx("option", { value: '', children: allParticipantsLabel }), remoteParticipants &&
                        remoteParticipants
                            .filter((p) => !!p.id)
                            .map((p) => (jsxRuntime.exports.jsx("option", { value: p.id, children: p.name }, p.id)))] })] }));
};

const EmojiSelector = ({ emojis, onSelectEmoji, openButtonElement, currentMessage, }) => {
    const selectableEmojis = emojis !== null && emojis !== void 0 ? emojis : ['😂', '👍', '🎉', '😃', '👏'];
    const popoverStyle = {
        zIndex: '1001',
        paddingBottom: '0.25rem',
    };
    const [isPopoverVisible, setPopoverVisible] = useState(false);
    useEffect(() => {
        if (!currentMessage) {
            setPopoverVisible(false);
        }
    }, [currentMessage]);
    const EmojiSelectorElement = () => (jsxRuntime.exports.jsx("div", { className: 'emoji-selector__emoji-container', children: selectableEmojis.map((emoji, idx) => (jsxRuntime.exports.jsx("button", { className: 'emoji-selector__emoji-button', onClick: () => onSelectEmoji(emoji), "aria-label": `Insert ${emoji} emoji`, children: emoji }, idx))) }));
    const OpenButtonElement = openButtonElement !== null && openButtonElement !== void 0 ? openButtonElement : EmojiSelectorDefaultOpenButton;
    const openButtonClickHandler = () => setPopoverVisible(!isPopoverVisible);
    return (jsxRuntime.exports.jsx(Popover, { isOpen: isPopoverVisible, content: jsxRuntime.exports.jsx(EmojiSelectorElement, {}), onClickOutside: () => setPopoverVisible(false), containerStyle: popoverStyle, children: jsxRuntime.exports.jsx(OpenButtonElement, { onClick: openButtonClickHandler }) }));
};
const EmojiSelectorDefaultOpenButton = require$$0.forwardRef(({ onClick }, ref) => (jsxRuntime.exports.jsx("button", { ref: ref, className: 'emoji-selector__open-button', onClick: onClick, "aria-label": 'Toggle emoji selector', children: "\uD83D\uDE02" })));

const SendButton = ({ disabled, onClick, buttonText = 'Send' }) => (jsxRuntime.exports.jsx("button", { className: 'chat-input__send-button', disabled: disabled, onClick: onClick, children: buttonText }));

const CustomizableChatInput = ({ remoteParticipants, sendPublicMessage, sendPrivateMessage, uploadAttachment, chatMessageInputElement, sendButtonElement, recipientSelectorElement, emojiSelectorElement, attachmentUploaderElement, attachmentDisplayElement, }) => {
    const [currentMessage, setCurrentMessage] = useState('');
    const [messageRecipient, setMessageRecipient] = useState('');
    const [attachment, setAttachment] = useState();
    const [isUploading, setUploading] = useState(false);
    const [hasAttachmentError, setAttachmentError] = useState(false);
    const chatMessageInputRef = useRef(null);
    const recipientParticipant = remoteParticipants === null || remoteParticipants === void 0 ? void 0 : remoteParticipants.find((p) => p.id === messageRecipient);
    const sendMessage = () => {
        const messageToSend = currentMessage.trim();
        if (!messageToSend && !attachment) {
            return;
        }
        if (!messageRecipient) {
            sendPublicMessage && sendPublicMessage(messageToSend, attachment);
        }
        else if (recipientParticipant) {
            sendPrivateMessage && sendPrivateMessage(messageToSend, recipientParticipant, attachment);
        }
        setCurrentMessage('');
        clearAttachment();
    };
    const recipientSelectedHandler = (recipientId) => {
        setAttachment(undefined);
        setMessageRecipient(recipientId);
    };
    const emojiSelectedHandler = (emoji) => {
        var _a;
        if (!currentMessage) {
            setCurrentMessage(emoji);
        }
        else {
            setCurrentMessage((currentMessage) => `${currentMessage} ${emoji}`);
        }
        (_a = chatMessageInputRef.current) === null || _a === void 0 ? void 0 : _a.focus();
    };
    const attachmentUploadStartedHandler = () => {
        setUploading(true);
        setAttachmentError(false);
    };
    const attachmentUploadFinishedHandler = (file, fileId) => {
        var _a;
        setAttachment({ fileId, fileName: file.name, fileSize: file.size, contentType: file.type });
        setUploading(false);
        (_a = chatMessageInputRef.current) === null || _a === void 0 ? void 0 : _a.focus();
    };
    const attachmentUploadFailedHandler = () => {
        clearAttachment();
        setUploading(false);
        setAttachmentError(true);
    };
    const clearAttachment = () => {
        setAttachment(undefined);
    };
    useEffect(() => {
        if (messageRecipient && remoteParticipants && !remoteParticipants.some((p) => p.id === messageRecipient)) {
            setMessageRecipient('');
            setCurrentMessage('');
        }
    }, [messageRecipient, remoteParticipants]);
    const ChatMessageInputElement = chatMessageInputElement !== null && chatMessageInputElement !== void 0 ? chatMessageInputElement : ChatMessageInput;
    const RecipientSelectorElement = recipientSelectorElement !== null && recipientSelectorElement !== void 0 ? recipientSelectorElement : RecipientSelector;
    const EmojiSelectorElement = emojiSelectorElement !== null && emojiSelectorElement !== void 0 ? emojiSelectorElement : EmojiSelector;
    const SendButtonElement = sendButtonElement !== null && sendButtonElement !== void 0 ? sendButtonElement : SendButton;
    const AttachmentUploaderElement = attachmentUploaderElement !== null && attachmentUploaderElement !== void 0 ? attachmentUploaderElement : AttachmentUploader;
    const AttachmentDisplayElement = attachmentDisplayElement !== null && attachmentDisplayElement !== void 0 ? attachmentDisplayElement : AttachmentDisplay;
    return (jsxRuntime.exports.jsxs("div", { className: 'chat-input__input-group', children: [jsxRuntime.exports.jsxs("div", { className: 'chat-input__message-input-container', children: [jsxRuntime.exports.jsxs("div", { className: 'chat-input__top-bar', children: [jsxRuntime.exports.jsx(RecipientSelectorElement, { remoteParticipants: remoteParticipants, onRecipientSelected: recipientSelectedHandler }), jsxRuntime.exports.jsx(EmojiSelectorElement, { onSelectEmoji: emojiSelectedHandler, currentMessage: currentMessage }), uploadAttachment && (jsxRuntime.exports.jsx(AttachmentUploaderElement, { uploadAttachment: uploadAttachment, attachmentRecipient: recipientParticipant, onAttachmentUploadStarted: attachmentUploadStartedHandler, onAttachmentUploadFinished: attachmentUploadFinishedHandler, onAttachmentUploadFailed: attachmentUploadFailedHandler }))] }), jsxRuntime.exports.jsx(ChatMessageInputElement, { ref: chatMessageInputRef, currentChatMessage: currentMessage, onChatMessageChanged: (message) => setCurrentMessage(message), sendMessage: sendMessage }), jsxRuntime.exports.jsx(AttachmentDisplayElement, { attachment: attachment, onClearAttachment: clearAttachment, hasAttachmentError: hasAttachmentError })] }), jsxRuntime.exports.jsx(SendButtonElement, { disabled: !currentMessage.trim() && !attachment, onClick: sendMessage }), isUploading && (jsxRuntime.exports.jsx("div", { className: 'chat-input__loader-container', children: jsxRuntime.exports.jsx("div", { className: 'chat-input__loader' }) }))] }));
};

const ChatMessageAttachment = ({ attachment, getAttachmentLocation, }) => {
    const { anchorRef, isAttachmentLoading, hasAttachmentError, attachmentUrl, downloadAttachment } = useChatAttachment(attachment, getAttachmentLocation);
    return (jsxRuntime.exports.jsxs("div", { className: 'chat-message-attachment__container', children: [jsxRuntime.exports.jsx("div", { className: 'chat-message-attachment__name', children: `📎 ${attachment.fileName}` }), jsxRuntime.exports.jsx("button", { className: 'chat-message-attachment__download-button', disabled: isAttachmentLoading, onClick: downloadAttachment, children: "Download" }), jsxRuntime.exports.jsx("a", { ref: anchorRef, className: 'chat-message-attachment__hidden-link', href: attachmentUrl, download: attachment.fileName, target: '_blank', rel: 'noreferrer noopener', children: ' ' }), hasAttachmentError && jsxRuntime.exports.jsx("div", { children: "Error while downloading the attachment." })] }));
};

const linkifyOptions = {
    defaultProtocol: 'https',
    target: '_blank',
};
const ChatMessage = ({ chatMessage, getAttachmentLocation, sameThreadAsPrevious, localParticipantLabel = 'Me', chatMessageAttachmentElement, }) => {
    const { id: localParticipantId } = useZocksStore(selectLocalParticipant);
    const { from: { id: senderId, name: senderName }, to: { id: recipientId, name: recipientName } = {}, message, attachment, } = chatMessage;
    const sentByMe = senderId === localParticipantId;
    const forMe = recipientId === localParticipantId;
    const directMessage = !!recipientId;
    const headerText = `${sentByMe ? localParticipantLabel : senderName}` +
        (directMessage ? ` To ${forMe ? localParticipantLabel : recipientName}` : '');
    const ChatMessageAttachmentElement = chatMessageAttachmentElement !== null && chatMessageAttachmentElement !== void 0 ? chatMessageAttachmentElement : ChatMessageAttachment;
    return (jsxRuntime.exports.jsxs("div", { className: classNames('chat-message__container', {
            'chat-message__container--sent': sentByMe,
            'chat-message__container--received': !sentByMe,
        }), children: [!sameThreadAsPrevious && (jsxRuntime.exports.jsxs(jsxRuntime.exports.Fragment, { children: [jsxRuntime.exports.jsx("div", { className: 'chat-message__avatar', children: senderName.slice(0, 1) }), jsxRuntime.exports.jsx("div", { className: classNames('chat-message__header', {
                            'chat-message__header--public': !directMessage,
                            'chat-message__header--private': directMessage,
                        }), children: headerText })] })), message && (jsxRuntime.exports.jsx(Linkify, { options: linkifyOptions, children: jsxRuntime.exports.jsx("div", { className: 'chat-message__message', children: message }) })), attachment && getAttachmentLocation && (jsxRuntime.exports.jsx(ChatMessageAttachmentElement, { attachment: attachment, getAttachmentLocation: getAttachmentLocation }))] }));
};

const Chat = ({ chatActions, dismiss, onNewChatMessage, chatMessageElement = null, chatInputElement = null, }) => {
    const { sendPublicMessage, sendPrivateMessage, uploadAttachment, getAttachmentLocation, publicChatMessages, privateChatMessages, } = useChat(chatActions);
    const remoteParticipants = useZocksStore(selectRemoteParticipants);
    const messagesEndRef = useRef(null);
    const scrollToBottom = () => {
        var _a;
        (_a = messagesEndRef.current) === null || _a === void 0 ? void 0 : _a.scrollIntoView({ behavior: 'smooth' });
    };
    const allChatMessages = useMemo(() => [...publicChatMessages, ...Object.values(privateChatMessages).flat()].sort((a, b) => a.timestamp - b.timestamp), [publicChatMessages, privateChatMessages]);
    useEffect(() => {
        scrollToBottom();
    }, [allChatMessages]);
    useEffect(() => {
        if (allChatMessages.length) {
            onNewChatMessage && onNewChatMessage();
        }
    }, [allChatMessages.length, onNewChatMessage]);
    const sameSenderAndRecipient = (message, other) => {
        const { from: { id: messageSenderId }, to: { id: messageRecipientId } = {}, } = message;
        const { from: { id: otherSenderId }, to: { id: otherRecipientId } = {}, } = other;
        return messageSenderId === otherSenderId && messageRecipientId === otherRecipientId;
    };
    const ChatMessageElement = chatMessageElement !== null && chatMessageElement !== void 0 ? chatMessageElement : ChatMessage;
    const ChatInputElement = chatInputElement !== null && chatInputElement !== void 0 ? chatInputElement : CustomizableChatInput;
    return (jsxRuntime.exports.jsxs("div", { className: 'chat', children: [dismiss && (jsxRuntime.exports.jsx("div", { className: 'chat__header', children: jsxRuntime.exports.jsx("button", { onClick: dismiss, "aria-label": 'Dismiss' }) })), jsxRuntime.exports.jsxs("div", { className: 'chat__main-container', children: [jsxRuntime.exports.jsx("div", { className: 'chat__messages-container', children: jsxRuntime.exports.jsxs("div", { className: 'chat__messages-scroll', children: [allChatMessages.map((chatMessage, idx) => {
                                    const sameThreadAsPrevious = idx !== 0 && sameSenderAndRecipient(allChatMessages[idx], allChatMessages[idx - 1]);
                                    return (jsxRuntime.exports.jsx(ChatMessageElement, { chatMessage: chatMessage, sameThreadAsPrevious: sameThreadAsPrevious, getAttachmentLocation: getAttachmentLocation }, idx));
                                }), jsxRuntime.exports.jsx("div", { ref: messagesEndRef })] }) }), jsxRuntime.exports.jsx("div", { className: 'chat__footer', children: jsxRuntime.exports.jsx(ChatInputElement, { remoteParticipants: remoteParticipants, sendPublicMessage: sendPublicMessage, sendPrivateMessage: sendPrivateMessage, uploadAttachment: uploadAttachment }) })] })] }));
};

const useMedia = stream => {
  const mediaRef = useRef(null);
  useEffect(() => {
    const currentRef = mediaRef.current;

    if (stream && currentRef) {
      stream.attach(currentRef);
      return () => stream.detach(currentRef);
    }
  }, [stream]);
  return {
    mediaRef
  };
};

const useVideo = videoStream => {
  const {
    mediaRef: videoRef
  } = useMedia(videoStream);
  return {
    videoRef
  };
};
const useAudio = audioStream => {
  const {
    mediaRef: audioRef
  } = useMedia(audioStream);
  return {
    audioRef
  };
};

const Video = ({ videoStream }) => {
    const { videoRef } = useVideo(videoStream);
    return videoStream && jsxRuntime.exports.jsx("video", { ref: videoRef, "data-testid": 'videoElement' });
};

const Audio = ({ audioStream }) => {
    const { audioRef } = useAudio(audioStream);
    return audioStream && jsxRuntime.exports.jsx("audio", { ref: audioRef, "data-testid": 'audioElement' });
};

const SelectButton = ({ menuItems, onMenuItemSelected, disabled, className, buttonLabel }) => {
    const [popoverVisible, setPopoverVisible] = useState(false);
    const handleMenuItemClick = (item) => {
        setPopoverVisible(false);
        onMenuItemSelected && onMenuItemSelected(item);
    };
    const menu = (jsxRuntime.exports.jsx("div", { className: 'select-button__popover-container', children: jsxRuntime.exports.jsx("ul", { className: 'select-button__popover-list', children: menuItems === null || menuItems === void 0 ? void 0 : menuItems.map((item, i) => {
                return (jsxRuntime.exports.jsx("li", { className: 'select-button__popover-item', onClick: () => handleMenuItemClick(item), children: item.label }, i));
            }) }) }));
    return (jsxRuntime.exports.jsx("div", { className: className, children: jsxRuntime.exports.jsx(Popover, { isOpen: popoverVisible, positions: ['top'], content: menu, onClickOutside: () => setPopoverVisible(false), children: jsxRuntime.exports.jsx("div", { className: 'select-button__button-container', children: jsxRuntime.exports.jsx("button", { disabled: disabled, className: 'select-button__button', onClick: () => setPopoverVisible(!popoverVisible), "aria-label": buttonLabel, children: jsxRuntime.exports.jsx("div", { className: 'select-button__button-icon' }) }) }) }) }));
};

const DeviceSelectButton = ({ deviceKind, onDeviceSelected, className, disabled, label }) => {
    const [sources, setSources] = useState([]);
    const [menuItems, setMenuItems] = useState([]);
    const listDevices = useCallback(async () => {
        const devices = await MediaDeviceManager.getDevices(deviceKind);
        setSources(devices);
        setMenuItems(devices.map((item) => {
            return { label: item.label };
        }));
    }, [deviceKind]);
    useEffect(() => {
        listDevices();
        navigator.mediaDevices.addEventListener('devicechange', listDevices);
        return () => {
            navigator.mediaDevices.removeEventListener('devicechange', listDevices);
        };
    }, [listDevices]);
    const onMenuItemSelected = (item) => {
        const device = sources.find((d) => d.label === item.label);
        device && onDeviceSelected(device);
    };
    return (jsxRuntime.exports.jsx(SelectButton, { disabled: disabled, className: className, buttonLabel: label, menuItems: menuItems, onMenuItemSelected: onMenuItemSelected }));
};

const JoinPage = ({ joinAction, createRoomAction, initialRoomId = '', enableAudioText = 'Unmute', disableAudioText = 'Mute', enableVideoText = 'Enable Video', disableVideoText = 'Disable Video', joinText = 'Join', newMeetingText = 'New Meeting', createNewMeetingText = 'Create New Meeting', newMeetingDisabled = false, roomIdInputLabel = 'Room id', roomIdInputErrorLabel = 'Invalid room id!', roomIdInputDisabled = false, roomIdPlaceholder = 'Enter a room id', videoEnabledByDefault = false, audioEnabledByDefault = false, onNewRoomCreated = null, }) => {
    var _a, _b;
    const isConnecting = useZocksStore(selectIsConnecting);
    const [roomId, setRoomId] = useState(initialRoomId);
    const [hasError, setHasError] = useState(false);
    const [isPopoverOpen, setPopoverOpen] = useState(false);
    const { localAudioStream, localVideoStream, videoStreamIsLoading, toggleLocalAudioStream, toggleLocalVideoStream, selectAudioDevice, selectVideoDevice, } = useLocalMedia(videoEnabledByDefault, audioEnabledByDefault);
    const audioEnabled = !!localAudioStream;
    const videoEnabled = !!localVideoStream;
    const audioDeviceSettings = (_a = localAudioStream === null || localAudioStream === void 0 ? void 0 : localAudioStream.mediaStreamTrack) === null || _a === void 0 ? void 0 : _a.getSettings();
    const videoDeviceSettings = (_b = localVideoStream === null || localVideoStream === void 0 ? void 0 : localVideoStream.mediaStreamTrack) === null || _b === void 0 ? void 0 : _b.getSettings();
    const afterJoinCallback = () => {
        localAudioStream && toggleLocalAudioStream();
        localVideoStream && toggleLocalVideoStream();
    };
    const joinErrorCallback = () => {
        setPopoverOpen(true);
        setHasError(true);
    };
    const baseJoinOptions = {
        roomId,
        autoCreate: false,
        audioDeviceSettings,
        videoDeviceSettings,
        afterJoinCallback,
        joinErrorCallback,
    };
    const onRoomIdInputChange = (evt) => {
        setHasError(false);
        setPopoverOpen(false);
        setRoomId(evt.target.value);
    };
    const onRoomIdInputKeyUp = (evt) => {
        if (roomId && evt.key === 'Enter') {
            joinAction(baseJoinOptions);
        }
    };
    const createNewMeeting = () => {
        createRoomAction().then((roomInfo) => onNewRoomCreated && onNewRoomCreated(roomInfo));
    };
    const joinErrorTooltip = (position, childRect, popoverRect) => (jsxRuntime.exports.jsx("div", { className: 'join_page__room-id-input-popover', children: jsxRuntime.exports.jsx(ArrowContainer, { position: position, childRect: childRect, popoverRect: popoverRect, arrowColor: 'black', arrowSize: 10, className: 'popover-arrow-container', arrowClassName: 'popover-arrow', children: jsxRuntime.exports.jsx("div", { className: 'join-page__room-id-input-popover-text', children: roomIdInputErrorLabel }) }) }));
    return (jsxRuntime.exports.jsxs("div", { className: 'join-page', children: [jsxRuntime.exports.jsxs("div", { className: 'join-page__local-streams', children: [videoEnabled ? (jsxRuntime.exports.jsx("div", { className: 'join-page__video-container', children: jsxRuntime.exports.jsx("div", { className: 'join-page__video-preview', children: jsxRuntime.exports.jsx(Video, { videoStream: localVideoStream }) }) })) : (jsxRuntime.exports.jsx("div", { className: 'join-page__no-video-icon' })), jsxRuntime.exports.jsxs("div", { className: 'join-page__local-stream-buttons', children: [jsxRuntime.exports.jsxs("div", { className: 'join-page__audio-button-container', children: [jsxRuntime.exports.jsx("button", { className: classNames('join-page__button join-page__audio-button', {
                                            'join-page__audio-button--enabled': audioEnabled,
                                            'join-page__audio-button--disabled': !audioEnabled,
                                        }), onClick: toggleLocalAudioStream, children: audioEnabled ? disableAudioText : enableAudioText }), jsxRuntime.exports.jsx(DeviceSelectButton, { className: 'join-page--audio-select-button', deviceKind: 'audioinput', onDeviceSelected: selectAudioDevice, label: 'Audio devices' })] }), jsxRuntime.exports.jsxs("div", { className: 'join-page__video-button-container', children: [jsxRuntime.exports.jsx("button", { disabled: videoStreamIsLoading, className: classNames('join-page__button join-page__video-button', {
                                            'join-page__video-button--enabled': videoEnabled,
                                            'join-page__video-button--disabled': !videoEnabled,
                                        }), onClick: toggleLocalVideoStream, children: videoEnabled ? disableVideoText : enableVideoText }), jsxRuntime.exports.jsx(DeviceSelectButton, { disabled: videoStreamIsLoading, className: 'join-page--video-select-button', deviceKind: 'videoinput', onDeviceSelected: selectVideoDevice, label: 'Video devices' })] })] })] }), jsxRuntime.exports.jsxs("div", { className: 'join-page__controls', children: [jsxRuntime.exports.jsxs("div", { className: 'join-page__join-room-container', children: [jsxRuntime.exports.jsx(Popover, { isOpen: isPopoverOpen, content: ({ position, childRect, popoverRect }) => joinErrorTooltip(position, childRect, popoverRect), positions: ['top'], children: jsxRuntime.exports.jsx("input", { onMouseEnter: () => setPopoverOpen(hasError), onMouseLeave: () => setPopoverOpen(false), className: classNames({ 'join-page__room-id-input--error': hasError }), type: 'text', onChange: onRoomIdInputChange, onKeyUp: onRoomIdInputKeyUp, value: roomId, placeholder: roomIdPlaceholder, "aria-label": roomIdInputLabel, disabled: roomIdInputDisabled }) }), jsxRuntime.exports.jsx("button", { className: 'join-page__button join-page__join-button', onClick: () => joinAction(baseJoinOptions), disabled: !roomId || isConnecting || hasError, children: joinText })] }), jsxRuntime.exports.jsx("div", { className: 'join-page__controls-divider' }), jsxRuntime.exports.jsxs("div", { className: 'join-page__new-meeting-container', children: [jsxRuntime.exports.jsx("button", { className: 'join-page__button join-page__button join-page__new-meeting-button', onClick: () => joinAction({ ...baseJoinOptions, roomId: '', autoCreate: true }), disabled: isConnecting || newMeetingDisabled, children: newMeetingText }), jsxRuntime.exports.jsx("button", { className: 'join-page__button join-page__button join-page__create-new-meeting-button', onClick: createNewMeeting, disabled: isConnecting || newMeetingDisabled, children: createNewMeetingText })] })] })] }));
};

const Participant = ({ participant, withAudio = false, }) => {
    const { name, tracks: { primaryVideoTrack, primaryAudioTrack }, isLocal, connectionQuality, isSpeaking, } = useParticipant(participant);
    const videoIsMuted = !primaryVideoTrack || primaryVideoTrack.isMuted;
    const audioIsMuted = !primaryAudioTrack || primaryAudioTrack.isMuted;
    return (jsxRuntime.exports.jsxs("div", { className: 'participant', children: [jsxRuntime.exports.jsx("div", { className: 'participant__name', children: name }), jsxRuntime.exports.jsx("div", { className: classNames('participant__video-container', {
                    'participant__video-container--speaking': isSpeaking,
                    'participant__video-container--local': isLocal,
                }), children: videoIsMuted ? (jsxRuntime.exports.jsx("div", { className: 'participant__avatar', children: (name === null || name === void 0 ? void 0 : name.slice(0, 1)) || '' })) : (jsxRuntime.exports.jsx(Video, { videoStream: primaryVideoTrack.track })) }), jsxRuntime.exports.jsxs("div", { className: 'participant__status-icons', children: [videoIsMuted && jsxRuntime.exports.jsx("div", { className: 'participant__no-video-icon' }), audioIsMuted && jsxRuntime.exports.jsx("div", { className: 'participant__no-audio-icon' }), jsxRuntime.exports.jsx("div", { className: `participant__${connectionQuality}-quality-icon` })] }), withAudio && !audioIsMuted && !isLocal && jsxRuntime.exports.jsx(Audio, { audioStream: primaryAudioTrack.track })] }));
};

const sortParticipantsBySpeakers = (activeSpeakers, participants) => {
  const reversedSpeakers = [...activeSpeakers].reverse();
  return participants.filter(p => !!p.id).sort((p1, p2) => reversedSpeakers.indexOf(p2.id) - reversedSpeakers.indexOf(p1.id));
};

const usePagedParticipants = (participants, pageSize) => {
  const [page, setPage] = useState(0);
  const start = page * pageSize;
  const end = start + pageSize;
  const displayedParticipants = participants.slice(start, end);

  const nextPage = () => {
    setPage(page => page + 1);
  };

  const previousPage = () => {
    setPage(page => page - 1);
  };

  const hasNextPage = end < participants.length;
  const hasPreviousPage = start > 0;
  return {
    displayedParticipants,
    hasNextPage,
    hasPreviousPage,
    nextPage,
    previousPage
  };
};
const useSortRemoteParticipantsBySpeakers = () => {
  const activeSpeakers = useZocksStore(selectActiveSpeakers);
  const remoteParticipants = useZocksStore(selectRemoteParticipants);
  const [lastSpeakers, setLastSpeakers] = useState(activeSpeakers);
  useEffect(() => {
    if (activeSpeakers.length) {
      setLastSpeakers(lastSpeakers => [...new Set([...activeSpeakers, ...lastSpeakers])]);
    }
  }, [activeSpeakers]);
  const remoteParticipantsSortedBySpeakers = sortParticipantsBySpeakers(lastSpeakers, remoteParticipants);
  return {
    remoteParticipantsSortedBySpeakers
  };
};
const useSortRemoteParticipantsForGridView = pageSize => {
  const activeSpeakers = useZocksStore(selectActiveSpeakers);
  const remoteParticipants = useZocksStore(selectRemoteParticipants);
  const [lastSpeakers, setLastSpeakers] = useState(activeSpeakers);
  useEffect(() => {
    if (activeSpeakers.length) {
      setLastSpeakers(lastSpeakers => [...new Set([...activeSpeakers, ...lastSpeakers])]);
    }
  }, [activeSpeakers]);

  if (remoteParticipants.length < pageSize) {
    return remoteParticipants;
  }

  const lastSpeakingParticipantsNotOnFirstPage = remoteParticipants.slice(pageSize - 1).filter(p => p.id && lastSpeakers.includes(p.id));
  const participantsToMove = [];

  for (const lastSpeaker of lastSpeakers) {
    const participantIndex = remoteParticipants.findIndex(p => p.id === lastSpeaker);

    if (participantIndex < 0) {
      continue;
    }

    if (lastSpeakingParticipantsNotOnFirstPage.some(p => p.id === lastSpeaker) || participantIndex + lastSpeakingParticipantsNotOnFirstPage.length >= pageSize - 1) {
      participantsToMove.push(remoteParticipants[participantIndex]);
    }
  }

  const others = remoteParticipants.filter(p => !participantsToMove.some(p2 => p2.id === p.id));
  return [...participantsToMove, ...others];
};

const ParticipantsGridView = ({ pageSize = 25 }) => {
    const localParticipant = useZocksStore(selectLocalParticipant);
    const remoteParticipants = useSortRemoteParticipantsForGridView(pageSize);
    const allParticipants = [localParticipant, ...remoteParticipants];
    const { displayedParticipants, hasNextPage, hasPreviousPage, nextPage, previousPage } = usePagedParticipants(allParticipants, pageSize);
    const gridClass = getGridClassForParticipants(Math.min(pageSize, allParticipants.length));
    return (jsxRuntime.exports.jsxs("div", { className: 'participants-grid-view', children: [hasPreviousPage && jsxRuntime.exports.jsx(PreviousButton$2, { onClick: previousPage }), jsxRuntime.exports.jsx("div", { className: `participants-grid-view__grid ${gridClass}`, children: displayedParticipants.map((participant) => (jsxRuntime.exports.jsx(Participant, { participant: participant, withAudio: true }, participant.id))) }), hasNextPage && jsxRuntime.exports.jsx(NextButton$2, { onClick: nextPage })] }));
};
const getGridClassForParticipants = (numberOfParticipants) => {
    switch (true) {
        case numberOfParticipants <= 1:
            return 'participants-grid-view__grid--1x1';
        case numberOfParticipants === 2:
            return 'participants-grid-view__grid--2x1';
        case numberOfParticipants <= 4:
            return 'participants-grid-view__grid--2x2';
        case numberOfParticipants <= 6:
            return 'participants-grid-view__grid--3x2';
        case numberOfParticipants <= 9:
            return 'participants-grid-view__grid--3x3';
        case numberOfParticipants <= 16:
            return 'participants-grid-view__grid--4x4';
        default:
            return 'participants-grid-view__grid--5x5';
    }
};
const PreviousButton$2 = ({ onClick }) => (jsxRuntime.exports.jsx("button", { className: 'participants-grid-view__button participants-grid-view__previous-button', onClick: onClick, "aria-label": 'Previous', children: jsxRuntime.exports.jsx("div", { className: 'participants-grid-view__previous-button-icon' }) }));
const NextButton$2 = ({ onClick }) => (jsxRuntime.exports.jsx("button", { className: 'participants-grid-view__button participants-grid-view__next-button', onClick: onClick, "aria-label": 'Next', children: jsxRuntime.exports.jsx("div", { className: 'participants-grid-view__next-button-icon' }) }));

const ParticipantsSpeakerView = ({ pageSize = 4 }) => {
    const localParticipant = useZocksStore(selectLocalParticipant);
    const { remoteParticipantsSortedBySpeakers } = useSortRemoteParticipantsBySpeakers();
    const remoteParticipantsExist = !!remoteParticipantsSortedBySpeakers.length;
    const mainParticipant = remoteParticipantsExist ? remoteParticipantsSortedBySpeakers[0] : localParticipant;
    const allParticipants = [localParticipant, ...remoteParticipantsSortedBySpeakers];
    const { displayedParticipants, hasNextPage, hasPreviousPage, nextPage, previousPage } = usePagedParticipants(allParticipants, pageSize);
    return (jsxRuntime.exports.jsxs("div", { className: 'participants-speaker-view', children: [jsxRuntime.exports.jsx("div", { className: classNames('participants-speaker-view__main-speaker', {
                    'participants-speaker-view__only-speaker': !remoteParticipantsExist,
                }), children: mainParticipant && jsxRuntime.exports.jsx(MainParticipant, { mainParticipant: mainParticipant }) }), remoteParticipantsExist && (jsxRuntime.exports.jsxs(jsxRuntime.exports.Fragment, { children: [hasPreviousPage && jsxRuntime.exports.jsx(PreviousButton$1, { onClick: previousPage }), jsxRuntime.exports.jsx("div", { className: 'participants-speaker-view__others-container', children: jsxRuntime.exports.jsx("div", { className: 'participants-speaker-view__others', children: displayedParticipants.map((participant) => (jsxRuntime.exports.jsx(Participant, { participant: participant }, participant.id))) }) }), hasNextPage && jsxRuntime.exports.jsx(NextButton$1, { onClick: nextPage })] }))] }));
};
const MainParticipant = ({ mainParticipant }) => {
    const SPEAKER_CHANGE_DELAY = 500;
    const [shouldDisplayMainSpeaker, setShouldDisplayMainSpeaker] = useState(true);
    const [lastSpeaker, setLastSpeaker] = useState(mainParticipant);
    useEffect(() => {
        setShouldDisplayMainSpeaker(false);
        const timeout = setTimeout(() => {
            setShouldDisplayMainSpeaker(true);
            setLastSpeaker(mainParticipant);
        }, SPEAKER_CHANGE_DELAY);
        return () => clearTimeout(timeout);
    }, [mainParticipant]);
    return (jsxRuntime.exports.jsxs(jsxRuntime.exports.Fragment, { children: [jsxRuntime.exports.jsx("div", { className: classNames('main-speaker__participant', {
                    'main-speaker__participant--hidden': !shouldDisplayMainSpeaker,
                }), "aria-hidden": !shouldDisplayMainSpeaker, "data-testid": 'mainSpeakerElement', children: jsxRuntime.exports.jsx(Participant, { participant: mainParticipant }) }), lastSpeaker && (jsxRuntime.exports.jsx("div", { className: classNames('main-speaker__participant', {
                    'main-speaker__participant--hidden': shouldDisplayMainSpeaker,
                }), "aria-hidden": shouldDisplayMainSpeaker, "data-testid": 'lastSpeakerElement', children: jsxRuntime.exports.jsx(Participant, { participant: lastSpeaker }) }))] }));
};
const PreviousButton$1 = ({ onClick }) => (jsxRuntime.exports.jsx("button", { className: 'participants-speaker-view__button participants-speaker-view__previous-button', onClick: onClick, "aria-label": 'Previous', children: jsxRuntime.exports.jsx("div", { className: 'participants-speaker-view__previous-button-icon' }) }));
const NextButton$1 = ({ onClick }) => (jsxRuntime.exports.jsx("button", { className: 'participants-speaker-view__button participants-speaker-view__next-button', onClick: onClick, "aria-label": 'Next', children: jsxRuntime.exports.jsx("div", { className: 'participants-speaker-view__next-button-icon' }) }));

const ScreenShare = ({ participant }) => {
    const { tracks: { screenShareTrack, primaryAudioTrack }, isLocal, } = useParticipant(participant);
    return (jsxRuntime.exports.jsxs("div", { className: 'screen-share', children: [screenShareTrack && jsxRuntime.exports.jsx(Video, { videoStream: screenShareTrack.track }), !isLocal && primaryAudioTrack && !primaryAudioTrack.isMuted && jsxRuntime.exports.jsx(Audio, { audioStream: primaryAudioTrack.track })] }));
};

const ParticipantsScreenShareView = ({ pageSize = 4 }) => {
    const localParticipant = useZocksStore(selectLocalParticipant);
    const screenSharingParticipant = useZocksStore(selectScreenSharingParticipant);
    const { remoteParticipantsSortedBySpeakers } = useSortRemoteParticipantsBySpeakers();
    const allParticipants = [localParticipant, ...remoteParticipantsSortedBySpeakers];
    const { displayedParticipants, hasNextPage, hasPreviousPage, nextPage, previousPage } = usePagedParticipants(allParticipants, pageSize);
    return (jsxRuntime.exports.jsxs("div", { className: 'participants-screen-share-view', children: [jsxRuntime.exports.jsx("div", { className: 'participants-screen-share-view__current-share', children: screenSharingParticipant && jsxRuntime.exports.jsx(ScreenShare, { participant: screenSharingParticipant }) }), hasPreviousPage && jsxRuntime.exports.jsx(PreviousButton, { onClick: previousPage }), jsxRuntime.exports.jsx("div", { className: 'participants-screen-share-view__participants-container', children: jsxRuntime.exports.jsx("div", { className: 'participants-screen-share-view__participants', children: displayedParticipants.map((participant) => (jsxRuntime.exports.jsx(Participant, { participant: participant }, participant.id))) }) }), hasNextPage && jsxRuntime.exports.jsx(NextButton, { onClick: nextPage })] }));
};
const PreviousButton = ({ onClick }) => (jsxRuntime.exports.jsx("button", { className: 'participants-screen-share-view__button participants-screen-share-view__previous-button', onClick: onClick, "aria-label": 'Previous', children: jsxRuntime.exports.jsx("div", { className: 'participants-screen-share-view__previous-button-icon' }) }));
const NextButton = ({ onClick }) => (jsxRuntime.exports.jsx("button", { className: 'participants-screen-share-view__button participants-screen-share-view__next-button', onClick: onClick, "aria-label": 'Next', children: jsxRuntime.exports.jsx("div", { className: 'participants-screen-share-view__next-button-icon' }) }));

const ParticipantAudio = () => {
    const audioTracks = useZocksStore(selectSpeakingRemoteParticipantAudioTracks);
    return (jsxRuntime.exports.jsx(jsxRuntime.exports.Fragment, { children: audioTracks.map(({ track }) => (jsxRuntime.exports.jsx(Audio, { audioStream: track }, track.id))) }));
};

const Participants = ({ gridPageSize = 25, speakerPageSize = 4, }) => {
    const isScreenSharing = useZocksStore(selectIsScreenSharing);
    const currentView = useZocksStore(selectCurrentParticipantsView);
    const getCurrentView = () => {
        switch (true) {
            case isScreenSharing:
                return jsxRuntime.exports.jsx(ParticipantsScreenShareView, {});
            case currentView === ParticipantsViewType.Grid:
                return jsxRuntime.exports.jsx(ParticipantsGridView, { pageSize: gridPageSize });
            case currentView === ParticipantsViewType.Speaker:
                return jsxRuntime.exports.jsx(ParticipantsSpeakerView, { pageSize: speakerPageSize });
        }
    };
    return (jsxRuntime.exports.jsxs("div", { className: 'participants', children: [getCurrentView(), jsxRuntime.exports.jsx(ParticipantAudio, {})] }));
};

const useViewSwitcher = () => {
  const dispatch = useZocksDispatch();
  const currentView = useZocksStore(selectCurrentParticipantsView);

  const changeView = newView => dispatch(changeParticipantsView(newView));

  return {
    currentView,
    changeView
  };
};

const ViewSwitcher = () => {
    const { currentView, changeView } = useViewSwitcher();
    return (jsxRuntime.exports.jsxs("div", { className: 'participants__buttons-container', children: [jsxRuntime.exports.jsx("button", { className: 'participants__button', onClick: () => changeView(ParticipantsViewType.Grid), disabled: currentView === ParticipantsViewType.Grid, "aria-label": 'Change to grid view', children: jsxRuntime.exports.jsx("div", { className: 'participants__grid-icon' }) }), jsxRuntime.exports.jsx("button", { className: 'participants__button', onClick: () => changeView(ParticipantsViewType.Speaker), disabled: currentView === ParticipantsViewType.Speaker, "aria-label": 'Change to speaker view', children: jsxRuntime.exports.jsx("div", { className: 'participants__speaker-icon' }) })] }));
};

const ParticipantsList = ({ participants, dismiss, ParticipantElement = SimpleParticipantsListElement }) => {
    return (jsxRuntime.exports.jsxs("div", { className: 'participants-list', children: [dismiss && (jsxRuntime.exports.jsx("div", { className: 'participants-list__dismiss-button', children: jsxRuntime.exports.jsx("button", { onClick: dismiss, "aria-label": 'Dismiss' }) })), jsxRuntime.exports.jsx("div", { className: 'participants-list__participants-container', children: participants.map((participant) => (jsxRuntime.exports.jsx(ParticipantElement, { participant: participant }, participant.id))) })] }));
};
const SimpleParticipantsListElement = ({ participant }) => {
    var _a;
    return (jsxRuntime.exports.jsxs("div", { className: 'participants-list__participant-container', children: [jsxRuntime.exports.jsx("div", { className: 'participants-list__participant-avatar', children: ((_a = participant.name) === null || _a === void 0 ? void 0 : _a.slice(0, 1)) || '' }), jsxRuntime.exports.jsx("div", { className: 'participants-list__participant-name', children: participant.name })] }));
};

const OnlineParticipantsList = ({ dismiss }) => {
    const localParticipant = useZocksStore(selectLocalParticipant);
    const remoteParticipants = useZocksStore(selectRemoteParticipants);
    const allParticipants = [localParticipant, ...remoteParticipants];
    return (jsxRuntime.exports.jsx(ParticipantsList, { participants: allParticipants, dismiss: dismiss, ParticipantElement: OnlineParticipantsListElement }));
};
const OnlineParticipantsListElement = ({ participant }) => {
    var _a;
    const activeSpeakers = useZocksStore(selectActiveSpeakers);
    const isSpeaking = activeSpeakers.some((participantId) => participantId === participant.id);
    const noAudio = participant.tracks.audioTracks.every((t) => t.isMuted);
    const noVideo = participant.tracks.videoTracks.every((t) => t.isMuted);
    return (jsxRuntime.exports.jsxs("div", { className: 'participants-list__participant-container', children: [jsxRuntime.exports.jsx("div", { className: 'participants-list__participant-avatar', children: ((_a = participant.name) === null || _a === void 0 ? void 0 : _a.slice(0, 1)) || '' }), jsxRuntime.exports.jsx("div", { className: 'participants-list__participant-name', children: participant.name }), jsxRuntime.exports.jsx("div", { className: classNames({
                    'participants-list__participant-video-icon': !noVideo,
                    'participants-list__participant-no-video-icon': noVideo,
                }) }), jsxRuntime.exports.jsx("div", { className: classNames({
                    'participants-list__participant-audio-icon': !noAudio,
                    'participants-list__participant-no-audio-icon': noAudio,
                    'participants-list__participant-speaking': isSpeaking,
                }) })] }));
};

const useRoomControls = mediaActions => {
  const {
    publishAudio,
    publishVideo,
    unPublishAudio,
    unPublishVideo,
    switchActiveDevice
  } = mediaActions;
  const localParticipant = useZocksStore(selectLocalParticipant);
  const isScreenSharing = useZocksStore(selectIsScreenSharing);
  const screenSharingParticipant = useZocksStore(selectScreenSharingParticipant);
  const isLocalScreenShare = screenSharingParticipant === localParticipant;
  const audioIsOn = !!(localParticipant.tracks.audioTracks.length && !localParticipant.tracks.audioTracks[0].isMuted);
  const videoIsOn = !!(localParticipant.tracks.videoTracks.length && !localParticipant.tracks.videoTracks[0].isMuted);

  const toggleAudio = () => {
    if (audioIsOn) {
      unPublishAudio();
    } else {
      publishAudio();
    }
  };

  const toggleVideo = () => {
    if (videoIsOn) {
      unPublishVideo();
    } else {
      publishVideo();
    }
  };

  const selectAudioDevice = audioDevice => {
    switchActiveDevice('audioinput', audioDevice.deviceId);
  };

  const selectVideoDevice = videoDevice => {
    switchActiveDevice('videoinput', videoDevice.deviceId);
  };

  return {
    audioIsOn,
    videoIsOn,
    isScreenSharing,
    isLocalScreenShare,
    toggleAudio,
    toggleVideo,
    selectAudioDevice,
    selectVideoDevice
  };
};

const RoomControls = ({ mediaActions, leaveAction }) => {
    const { audioIsOn, videoIsOn, isScreenSharing, isLocalScreenShare, toggleAudio, toggleVideo, selectAudioDevice, selectVideoDevice, } = useRoomControls(mediaActions);
    return (jsxRuntime.exports.jsxs("div", { className: 'room-controls', children: [jsxRuntime.exports.jsxs("div", { className: 'room-controls__audio-button-container', children: [jsxRuntime.exports.jsxs("button", { id: 'room-controls__audio-button', className: 'room-controls__control-button', onClick: toggleAudio, children: [jsxRuntime.exports.jsx("div", { className: classNames({
                                    'room-controls__audio-button--unmuted': audioIsOn,
                                    'room-controls__audio-button--muted': !audioIsOn,
                                }) }), jsxRuntime.exports.jsx("label", { htmlFor: 'room-controls__audio-button', children: audioIsOn ? 'Mute' : 'Unmute' })] }), jsxRuntime.exports.jsx(DeviceSelectButton, { className: 'room-controls--audio-select-button', deviceKind: 'audioinput', onDeviceSelected: selectAudioDevice, label: 'Audio devices' })] }), jsxRuntime.exports.jsxs("div", { className: 'room-controls__video-button-container', children: [jsxRuntime.exports.jsxs("button", { id: 'room-controls__video-button', className: 'room-controls__control-button', onClick: toggleVideo, children: [jsxRuntime.exports.jsx("div", { className: classNames({
                                    'room-controls__video-button--unmuted': videoIsOn,
                                    'room-controls__video-button--muted': !videoIsOn,
                                }) }), jsxRuntime.exports.jsx("label", { htmlFor: 'room-controls__video-button', children: videoIsOn ? 'Disable Video' : 'Enable Video' })] }), jsxRuntime.exports.jsx(DeviceSelectButton, { className: 'room-controls--video-select-button', deviceKind: 'videoinput', onDeviceSelected: selectVideoDevice, label: 'Video devices' })] }), jsxRuntime.exports.jsxs("button", { disabled: isScreenSharing && !isLocalScreenShare, id: 'room-controls__screen-share-button', className: 'room-controls__control-button room-controls__screen-share-button', onClick: isScreenSharing ? mediaActions.unPublishScreenShare : mediaActions.publishScreenShare, children: [jsxRuntime.exports.jsx("div", { className: classNames({
                            'room-controls__screen-share-button--share-on': !isScreenSharing || !isLocalScreenShare,
                            'room-controls__screen-share-button--share-off': isScreenSharing && isLocalScreenShare,
                        }) }), jsxRuntime.exports.jsx("label", { htmlFor: 'room-controls__screen-share-button', children: isLocalScreenShare ? 'Stop Sharing' : 'Share Screen' })] }), jsxRuntime.exports.jsx("button", { className: 'room-controls__leave-button', onClick: leaveAction, children: "Leave" })] }));
};

export { AttachmentDisplay, AttachmentUploader, Audio, Chat, ChatMessage, ChatMessageAttachment, ChatMessageInput, CustomizableChatInput, EmojiSelector, JoinPage, OnlineParticipantsList, Participant, ParticipantAudio, Participants, ParticipantsGridView, ParticipantsList, ParticipantsScreenShareView, ParticipantsSpeakerView, ParticipantsViewType, RecipientSelector, RoomControls, SendButton, SimpleParticipantsListElement, Video, ViewSwitcher, ZocksProvider, changeParticipantsView, selectActiveSpeakers, selectCurrentParticipantsView, selectIsConnecting, selectIsScreenSharing, selectLocalParticipant, selectLocalParticipantIsConnected, selectPrivateChatMessages, selectPublicChatMessages, selectRemoteParticipants, selectRoomState, selectScreenSharingParticipant, selectSessionId, selectSpeakingRemoteParticipantAudioTracks, selectTracksForParticipant, useAudio, useChat, useChatAttachment, useLeaveOnUnload, useLocalMedia, usePagedParticipants, useParticipant, useSortRemoteParticipantsBySpeakers, useSynchronizeParticipants, useVideo, useViewSwitcher, useZocksActions, useZocksSession, useZocksStore };
//# sourceMappingURL=zocks-client-sdk-react.esm.mjs.map
