import React, { useEffect, useMemo, useRef } from 'react'
import { selectRemoteParticipants } from '../../store'
import { ChatMessageModel } from '../../store/chat/models'
import { useZocksStore } from '../../app'
import { ChatProps } from './models'
import { useChat } from './hooks'
import { CustomizableChatInput } from './ChatInput'
import { ChatMessage } from './ChatMessage'
import './Chat.scss'

export const Chat: React.FC<ChatProps> = ({
  chatActions,
  dismiss,
  onNewChatMessage,
  chatMessageElement = null,
  chatInputElement = null,
}) => {
  const {
    sendPublicMessage,
    sendPrivateMessage,
    uploadAttachment,
    getAttachmentLocation,
    publicChatMessages,
    privateChatMessages,
  } = useChat(chatActions)
  const remoteParticipants = useZocksStore(selectRemoteParticipants)

  const messagesEndRef = useRef<HTMLDivElement>(null)
  const scrollToBottom = () => {
    messagesEndRef.current?.scrollIntoView({ behavior: 'smooth' })
  }

  const allChatMessages = useMemo(
    () =>
      [...publicChatMessages, ...Object.values(privateChatMessages).flat()].sort((a, b) => a.timestamp - b.timestamp),
    [publicChatMessages, privateChatMessages]
  )

  useEffect(() => {
    scrollToBottom()
  }, [allChatMessages])

  useEffect(() => {
    if (allChatMessages.length) {
      onNewChatMessage && onNewChatMessage()
    }
  }, [allChatMessages.length, onNewChatMessage])

  const sameSenderAndRecipient = (message: ChatMessageModel, other: ChatMessageModel): boolean => {
    const {
      from: { id: messageSenderId },
      to: { id: messageRecipientId } = {},
    } = message
    const {
      from: { id: otherSenderId },
      to: { id: otherRecipientId } = {},
    } = other
    return messageSenderId === otherSenderId && messageRecipientId === otherRecipientId
  }

  const ChatMessageElement = chatMessageElement ?? ChatMessage
  const ChatInputElement = chatInputElement ?? CustomizableChatInput

  return (
    <div className='chat'>
      {dismiss && (
        <div className='chat__header'>
          <button onClick={dismiss} aria-label='Dismiss' />
        </div>
      )}
      <div className='chat__main-container'>
        <div className='chat__messages-container'>
          <div className='chat__messages-scroll'>
            {allChatMessages.map((chatMessage, idx) => {
              const sameThreadAsPrevious =
                idx !== 0 && sameSenderAndRecipient(allChatMessages[idx], allChatMessages[idx - 1])
              return (
                <ChatMessageElement
                  key={idx}
                  chatMessage={chatMessage}
                  sameThreadAsPrevious={sameThreadAsPrevious}
                  getAttachmentLocation={getAttachmentLocation}
                />
              )
            })}
            <div ref={messagesEndRef} />
          </div>
        </div>
        <div className='chat__footer'>
          <ChatInputElement
            remoteParticipants={remoteParticipants}
            sendPublicMessage={sendPublicMessage}
            sendPrivateMessage={sendPrivateMessage}
            uploadAttachment={uploadAttachment}
          />
        </div>
      </div>
    </div>
  )
}
