import { render, screen } from '@testing-library/react'
import userEvent from '@testing-library/user-event'
import { JoinPage } from './JoinPage'
import { Provider } from 'react-redux'
import { configureStore, EnhancedStore } from '@reduxjs/toolkit'
import { zocksStoreConfig } from '../../app/store'
import { setConnecting } from '../../store/session/sessionSlice'
import { setupMediaDevicesMock, setupMockLocalMediaStream } from '../../testUtils'
import { mock } from 'jest-mock-extended'
import { RoomInfo } from '@zocks-communications/client-sdk-js'

beforeEach(() => setupMediaDevicesMock())

const loadDevices = async () => {
  const audioDevicesButton = screen.getByRole('button', { name: 'Audio devices' })
  userEvent.click(audioDevicesButton)
  expect(await screen.findByText('Test Microphone 1')).toBeInTheDocument()
  userEvent.click(audioDevicesButton)
}

const renderWithState = async (store: EnhancedStore, joinAction: () => void, props?: any) => {
  render(
    <Provider store={store}>
      <JoinPage joinAction={joinAction} {...props} />
    </Provider>
  )
  await loadDevices()
}

test('renders a local stream preview, a Room Id input, Join and a New Meeting button', async () => {
  const store = configureStore(zocksStoreConfig)
  await renderWithState(store, jest.fn())

  expect(screen.getByRole('textbox', { name: 'Room id' })).toBeInTheDocument()
  expect(screen.getByRole('button', { name: 'Join' })).toBeInTheDocument()
  expect(screen.getByRole('button', { name: 'New Meeting' })).toBeInTheDocument()
  expect(screen.getByRole('button', { name: 'Unmute' })).toBeInTheDocument()
  expect(screen.getByRole('button', { name: 'Enable Video' })).toBeInTheDocument()

  userEvent.click(screen.getByRole('button', { name: 'Audio devices' }))
  expect(await screen.findByText('Test Microphone 1')).toBeInTheDocument()
  userEvent.click(screen.getByRole('button', { name: 'Video devices' }))
  expect(await screen.findByText('Test Camera 1')).toBeInTheDocument()
})

test('sets room id when typing into the RoomId input field', async () => {
  const store = configureStore(zocksStoreConfig)
  await renderWithState(store, jest.fn())

  const roomIdInput = screen.getByLabelText('Room id')
  userEvent.clear(roomIdInput)
  userEvent.type(roomIdInput, 'test room id')

  expect(roomIdInput).toHaveValue('test room id')
})

test('starts the local audio stream when the Unmute button is clicked', async () => {
  const { mockCreateMediaStream } = setupMockLocalMediaStream()
  const store = configureStore(zocksStoreConfig)
  await renderWithState(store, jest.fn())

  const unmuteAudioButton = screen.getByRole('button', { name: 'Unmute' })
  userEvent.click(unmuteAudioButton)

  expect(mockCreateMediaStream).toHaveBeenCalledTimes(1)
  expect(mockCreateMediaStream).toHaveBeenCalledWith({ audio: { deviceId: undefined }, video: false })
  expect(await screen.findByRole('button', { name: 'Mute' })).toBeInTheDocument()
})

test('stops the local audio stream when the Mute button is clicked', async () => {
  const { mockLocalMediaStream: mockLocalAudioStream } = setupMockLocalMediaStream()
  const store = configureStore(zocksStoreConfig)
  await renderWithState(store, jest.fn())
  const unmuteAudioButton = screen.getByRole('button', { name: 'Unmute' })
  userEvent.click(unmuteAudioButton)

  const muteAudioButton = await screen.findByRole('button', { name: 'Mute' })
  userEvent.click(muteAudioButton)

  expect(mockLocalAudioStream.stopStream).toHaveBeenCalledTimes(1)
  expect(screen.getByRole('button', { name: 'Unmute' })).toBeInTheDocument()
})

test('starts the local video stream when the Enable Video button is clicked', async () => {
  const { mockCreateMediaStream } = setupMockLocalMediaStream()

  const store = configureStore(zocksStoreConfig)
  await renderWithState(store, jest.fn())

  const enableVideoButton = screen.getByRole('button', { name: 'Enable Video' })
  userEvent.click(enableVideoButton)

  expect(mockCreateMediaStream).toHaveBeenCalledTimes(1)
  expect(mockCreateMediaStream).toHaveBeenCalledWith({ audio: false, video: { deviceId: undefined } })
  expect(await screen.findByRole('button', { name: 'Disable Video' })).toBeInTheDocument()
})

test('stops the local video stream when the Disable Video button is clicked', async () => {
  const { mockLocalMediaStream: mockLocalVideoStream } = setupMockLocalMediaStream()

  const store = configureStore(zocksStoreConfig)
  await renderWithState(store, jest.fn())
  const enableVideButton = screen.getByRole('button', { name: 'Enable Video' })
  userEvent.click(enableVideButton)

  const disableVideoButton = await screen.findByRole('button', { name: 'Disable Video' })
  userEvent.click(disableVideoButton)

  expect(mockLocalVideoStream.stopStream).toHaveBeenCalledTimes(1)
  expect(screen.getByRole('button', { name: 'Enable Video' })).toBeInTheDocument()
})

test('calls the join action with the room id, audio and video start and after join callback parameters when clicking on the Join button without any streams', async () => {
  const store = configureStore(zocksStoreConfig)
  const joinAction = jest.fn()
  await renderWithState(store, joinAction)
  const roomIdInput = screen.getByLabelText('Room id')
  userEvent.clear(roomIdInput)
  userEvent.type(roomIdInput, 'test room id')

  const joinButton = screen.getByRole('button', { name: 'Join' })
  userEvent.click(joinButton)

  expect(joinAction).toHaveBeenCalledTimes(1)
  expect(joinAction).toHaveBeenCalledWith({
    afterJoinCallback: expect.any(Function),
    joinErrorCallback: expect.any(Function),
    audioDeviceSettings: undefined,
    roomId: 'test room id',
    autoCreate: false,
    videoDeviceSettings: undefined,
  })
})

test('calls the join action with room id, audio and video start parameters when clicking on the Join button with audio and video streams', async () => {
  setupMockLocalMediaStream('testMediaDeviceId')
  const store = configureStore(zocksStoreConfig)
  const joinAction = jest.fn()
  await renderWithState(store, joinAction)
  const roomIdInput = screen.getByLabelText('Room id')
  userEvent.clear(roomIdInput)
  userEvent.type(roomIdInput, 'test room id')
  const unmuteAudioButton = screen.getByRole('button', { name: 'Unmute' })
  const enableVideButton = screen.getByRole('button', { name: 'Enable Video' })
  userEvent.click(unmuteAudioButton)
  userEvent.click(enableVideButton)

  expect(await screen.findByRole('button', { name: 'Disable Video' })).toBeInTheDocument()
  const joinButton = screen.getByRole('button', { name: 'Join' })
  userEvent.click(joinButton)

  expect(joinAction).toHaveBeenCalledTimes(1)
  expect(joinAction).toHaveBeenCalledWith({
    afterJoinCallback: expect.any(Function),
    joinErrorCallback: expect.any(Function),
    audioDeviceSettings: { deviceId: 'testMediaDeviceId' },
    roomId: 'test room id',
    autoCreate: false,
    videoDeviceSettings: { deviceId: 'testMediaDeviceId' },
  })
})

test('calls the join action without a room id when clicking on the New Meeting button', async () => {
  const store = configureStore(zocksStoreConfig)
  const joinAction = jest.fn()
  await renderWithState(store, joinAction)

  const newMeetingButton = screen.getByRole('button', { name: 'New Meeting' })
  userEvent.click(newMeetingButton)

  expect(joinAction).toHaveBeenCalledTimes(1)
  expect(joinAction).toHaveBeenCalledWith({
    afterJoinCallback: expect.any(Function),
    joinErrorCallback: expect.any(Function),
    audioDeviceSettings: undefined,
    roomId: '',
    autoCreate: true,
    videoDeviceSettings: undefined,
  })
})
test('calls the create room action when clicking on the Create New Meeting button, which calls the callback if provided when the room is created', async () => {
  const mockRoomInfo = mock<RoomInfo>()
  const createRoomAction = jest.fn().mockResolvedValue(mockRoomInfo)
  const onNewRoomCreated = jest.fn()
  const store = configureStore(zocksStoreConfig)
  await renderWithState(store, jest.fn(), { createRoomAction, onNewRoomCreated })

  const createMeetingButton = screen.getByRole('button', { name: 'Create New Meeting' })
  await userEvent.click(createMeetingButton)

  expect(createRoomAction).toHaveBeenCalledTimes(1)
  expect(onNewRoomCreated).toHaveBeenCalledTimes(1)
  expect(onNewRoomCreated).toHaveBeenCalledWith(mockRoomInfo)
})

test('the Join and New Meeting buttons are disabled if the participant is already connecting', async () => {
  const store = configureStore(zocksStoreConfig)
  store.dispatch(setConnecting(true))
  await renderWithState(store, jest.fn())

  expect(screen.getByRole('button', { name: 'Join' })).toBeDisabled()
  expect(screen.getByRole('button', { name: 'New Meeting' })).toBeDisabled()
  expect(screen.getByRole('button', { name: 'Create New Meeting' })).toBeDisabled()
})

test('the New Meeting button is disabled if set', async () => {
  const store = configureStore(zocksStoreConfig)
  render(
    <Provider store={store}>
      <JoinPage
        joinAction={jest.fn()}
        createRoomAction={jest.fn()}
        initialRoomId='testInitialRoomId'
        newMeetingDisabled={true}
      />
    </Provider>
  )
  await loadDevices()

  expect(screen.getByRole('button', { name: 'New Meeting' })).toBeDisabled()
})

test('the Room id input is disabled if set', async () => {
  const store = configureStore(zocksStoreConfig)
  render(
    <Provider store={store}>
      <JoinPage
        joinAction={jest.fn()}
        createRoomAction={jest.fn()}
        initialRoomId='testInitialRoomId'
        roomIdInputDisabled={true}
      />
    </Provider>
  )
  await loadDevices()

  expect(screen.getByRole('textbox', { name: 'Room id' })).toBeDisabled()
})

test('the room id field is filled if the initial room id is given', async () => {
  const store = configureStore(zocksStoreConfig)
  render(
    <Provider store={store}>
      <JoinPage joinAction={jest.fn()} createRoomAction={jest.fn()} initialRoomId='testInitialRoomId' />
    </Provider>
  )
  await loadDevices()

  expect(screen.getByRole('textbox', { name: 'Room id' })).toHaveValue('testInitialRoomId')
})

test('automatically starts the local audio stream if the audio is enabled by default', async () => {
  const { mockCreateMediaStream } = setupMockLocalMediaStream()
  const store = configureStore(zocksStoreConfig)
  await renderWithState(store, jest.fn(), { audioEnabledByDefault: true })

  expect(mockCreateMediaStream).toHaveBeenCalledTimes(1)
  expect(mockCreateMediaStream).toHaveBeenCalledWith({ audio: { deviceId: undefined }, video: false })
  expect(await screen.findByRole('button', { name: 'Mute' })).toBeInTheDocument()
})

test('automatically starts the local video stream if the video is enabled by default', async () => {
  const { mockCreateMediaStream } = setupMockLocalMediaStream()
  const store = configureStore(zocksStoreConfig)
  await renderWithState(store, jest.fn(), { videoEnabledByDefault: true })

  expect(mockCreateMediaStream).toHaveBeenCalledTimes(1)
  expect(mockCreateMediaStream).toHaveBeenCalledWith({ audio: false, video: { deviceId: undefined } })
  expect(await screen.findByRole('button', { name: 'Disable Video' })).toBeInTheDocument()
})

test('pressing the Enter key calls the join action if there is a meeting id entered', async () => {
  const store = configureStore(zocksStoreConfig)
  const joinAction = jest.fn()
  await renderWithState(store, joinAction)
  const roomIdInput = screen.getByLabelText('Room id')
  userEvent.clear(roomIdInput)
  userEvent.type(roomIdInput, 'test room id')

  userEvent.type(roomIdInput, '{enter}')

  expect(joinAction).toHaveBeenCalledTimes(1)
  expect(joinAction).toHaveBeenCalledWith({
    afterJoinCallback: expect.any(Function),
    joinErrorCallback: expect.any(Function),
    audioDeviceSettings: undefined,
    roomId: 'test room id',
    autoCreate: false,
    videoDeviceSettings: undefined,
  })
})

test('pressing the Enter key does not call the join action if there is no meeting id', async () => {
  const store = configureStore(zocksStoreConfig)
  const joinAction = jest.fn()
  await renderWithState(store, joinAction)
  const roomIdInput = screen.getByLabelText('Room id')
  userEvent.clear(roomIdInput)

  userEvent.type(roomIdInput, '{enter}')

  expect(joinAction).toHaveBeenCalledTimes(0)
})
