import React, { ChangeEvent, useState } from 'react'
import classNames from 'classnames'
import { ArrowContainer, Popover, PopoverPosition } from 'react-tiny-popover'
import { RoomInfo } from '@zocks-communications/client-sdk-js'
import { useZocksStore } from '../../app'
import { JoinOptions } from '../../models'
import { selectIsConnecting, useLocalMedia } from '../../store'
import { Video } from '../Media'
import { DeviceSelectButton } from '../DeviceSelectButton'
import './JoinPage.scss'

export const JoinPage: React.FC<{
  joinAction: (joinOptions: JoinOptions) => void
  createRoomAction: () => Promise<RoomInfo>
  initialRoomId?: string
  enableAudioText?: string
  disableAudioText?: string
  enableVideoText?: string
  disableVideoText?: string
  joinText?: string
  newMeetingText?: string
  createNewMeetingText?: string
  newMeetingDisabled?: boolean
  roomIdInputLabel?: string
  roomIdInputErrorLabel?: string
  roomIdInputDisabled?: boolean
  roomIdPlaceholder?: string
  videoEnabledByDefault?: boolean
  audioEnabledByDefault?: boolean
  onNewRoomCreated?: (roomInfo: RoomInfo) => void
}> = ({
  joinAction,
  createRoomAction,
  initialRoomId = '',
  enableAudioText = 'Unmute',
  disableAudioText = 'Mute',
  enableVideoText = 'Enable Video',
  disableVideoText = 'Disable Video',
  joinText = 'Join',
  newMeetingText = 'New Meeting',
  createNewMeetingText = 'Create New Meeting',
  newMeetingDisabled = false,
  roomIdInputLabel = 'Room id',
  roomIdInputErrorLabel = 'Invalid room id!',
  roomIdInputDisabled = false,
  roomIdPlaceholder = 'Enter a room id',
  videoEnabledByDefault = false,
  audioEnabledByDefault = false,
  onNewRoomCreated = null,
}) => {
  const isConnecting = useZocksStore(selectIsConnecting)
  const [roomId, setRoomId] = useState<string>(initialRoomId)
  const [hasError, setHasError] = useState(false)
  const [isPopoverOpen, setPopoverOpen] = useState(false)

  const {
    localAudioStream,
    localVideoStream,
    videoStreamIsLoading,
    toggleLocalAudioStream,
    toggleLocalVideoStream,
    selectAudioDevice,
    selectVideoDevice,
  } = useLocalMedia(videoEnabledByDefault, audioEnabledByDefault)

  const audioEnabled = !!localAudioStream
  const videoEnabled = !!localVideoStream
  const audioDeviceSettings = localAudioStream?.mediaStreamTrack?.getSettings()
  const videoDeviceSettings = localVideoStream?.mediaStreamTrack?.getSettings()

  const afterJoinCallback = () => {
    localAudioStream && toggleLocalAudioStream()
    localVideoStream && toggleLocalVideoStream()
  }

  const joinErrorCallback = () => {
    setPopoverOpen(true)
    setHasError(true)
  }

  const baseJoinOptions = {
    roomId,
    autoCreate: false,
    audioDeviceSettings,
    videoDeviceSettings,
    afterJoinCallback,
    joinErrorCallback,
  }

  const onRoomIdInputChange = (evt: ChangeEvent<HTMLInputElement>) => {
    setHasError(false)
    setPopoverOpen(false)
    setRoomId(evt.target.value)
  }

  const onRoomIdInputKeyUp = (evt: React.KeyboardEvent<HTMLInputElement>) => {
    if (roomId && evt.key === 'Enter') {
      joinAction(baseJoinOptions)
    }
  }

  const createNewMeeting = () => {
    createRoomAction().then((roomInfo: RoomInfo) => onNewRoomCreated && onNewRoomCreated(roomInfo))
  }

  const joinErrorTooltip = (position: PopoverPosition | undefined, childRect: ClientRect, popoverRect: ClientRect) => (
    <div className='join_page__room-id-input-popover'>
      <ArrowContainer
        position={position}
        childRect={childRect}
        popoverRect={popoverRect}
        arrowColor={'black'}
        arrowSize={10}
        className='popover-arrow-container'
        arrowClassName='popover-arrow'
      >
        <div className='join-page__room-id-input-popover-text'>{roomIdInputErrorLabel}</div>
      </ArrowContainer>
    </div>
  )

  return (
    <div className='join-page'>
      <div className='join-page__local-streams'>
        {videoEnabled ? (
          <div className='join-page__video-container'>
            <div className='join-page__video-preview'>
              <Video videoStream={localVideoStream} />
            </div>
          </div>
        ) : (
          <div className='join-page__no-video-icon' />
        )}
        <div className='join-page__local-stream-buttons'>
          <div className='join-page__audio-button-container'>
            <button
              className={classNames('join-page__button join-page__audio-button', {
                'join-page__audio-button--enabled': audioEnabled,
                'join-page__audio-button--disabled': !audioEnabled,
              })}
              onClick={toggleLocalAudioStream}
            >
              {audioEnabled ? disableAudioText : enableAudioText}
            </button>
            <DeviceSelectButton
              className='join-page--audio-select-button'
              deviceKind={'audioinput'}
              onDeviceSelected={selectAudioDevice}
              label='Audio devices'
            />
          </div>
          <div className='join-page__video-button-container'>
            <button
              disabled={videoStreamIsLoading}
              className={classNames('join-page__button join-page__video-button', {
                'join-page__video-button--enabled': videoEnabled,
                'join-page__video-button--disabled': !videoEnabled,
              })}
              onClick={toggleLocalVideoStream}
            >
              {videoEnabled ? disableVideoText : enableVideoText}
            </button>
            <DeviceSelectButton
              disabled={videoStreamIsLoading}
              className='join-page--video-select-button'
              deviceKind={'videoinput'}
              onDeviceSelected={selectVideoDevice}
              label='Video devices'
            />
          </div>
        </div>
      </div>
      <div className='join-page__controls'>
        <div className='join-page__join-room-container'>
          <Popover
            isOpen={isPopoverOpen}
            content={({ position, childRect, popoverRect }) => joinErrorTooltip(position, childRect, popoverRect)}
            positions={['top']}
          >
            <input
              onMouseEnter={() => setPopoverOpen(hasError)}
              onMouseLeave={() => setPopoverOpen(false)}
              className={classNames({ 'join-page__room-id-input--error': hasError })}
              type='text'
              onChange={onRoomIdInputChange}
              onKeyUp={onRoomIdInputKeyUp}
              value={roomId}
              placeholder={roomIdPlaceholder}
              aria-label={roomIdInputLabel}
              disabled={roomIdInputDisabled}
            />
          </Popover>
          <button
            className='join-page__button join-page__join-button'
            onClick={() => joinAction(baseJoinOptions)}
            disabled={!roomId || isConnecting || hasError}
          >
            {joinText}
          </button>
        </div>
        <div className='join-page__controls-divider' />
        <div className='join-page__new-meeting-container'>
          <button
            className='join-page__button join-page__button join-page__new-meeting-button'
            onClick={() => joinAction({ ...baseJoinOptions, roomId: '', autoCreate: true })}
            disabled={isConnecting || newMeetingDisabled}
          >
            {newMeetingText}
          </button>
          <button
            className='join-page__button join-page__button join-page__create-new-meeting-button'
            onClick={createNewMeeting}
            disabled={isConnecting || newMeetingDisabled}
          >
            {createNewMeetingText}
          </button>
        </div>
      </div>
    </div>
  )
}
