import { render, screen } from '@testing-library/react'
import { Participant } from './Participant'
import { ParticipantModel } from '../../models'
import { setActiveSpeakers, setLocalParticipantId, setRoomState, setSessionId } from '../../store/session/sessionSlice'
import { ConnectionQuality, RoomState } from '@zocks-communications/client-sdk-js'
import { setupZocksSdkMocks } from '../../testUtils'
import { configureStore, EnhancedStore } from '@reduxjs/toolkit'
import { Provider } from 'react-redux'
import { zocksStoreConfig } from '../../app/store'

const renderWithState = (store: EnhancedStore, participant: ParticipantModel, withAudio: boolean = true) => {
  return render(
    <Provider store={store}>
      <Participant participant={participant} withAudio={withAudio} />
    </Provider>
  )
}

const testLocalParticipant: ParticipantModel = {
  id: 'testLocalParticipantId',
  name: 'testLocalParticipantName',
  tracks: {
    audioTracks: [
      {
        id: 'testLocalAudioTrackId',
        isMuted: false,
      },
    ],
    videoTracks: [
      {
        id: 'testLocalVideoTrackId',
        isMuted: false,
      },
    ],
    screenShareTrack: null,
  },
  connectionQuality: ConnectionQuality.EXCELLENT,
}

const testRemoteParticipant: ParticipantModel = {
  id: 'testRemoteParticipantId',
  name: 'testRemoteParticipantName',
  tracks: {
    audioTracks: [
      {
        id: 'testRemoteAudioTrackId',
        isMuted: false,
      },
    ],
    videoTracks: [
      {
        id: 'testRemoteVideoTrackId',
        isMuted: false,
      },
    ],
    screenShareTrack: null,
  },
  connectionQuality: ConnectionQuality.GOOD,
}

test('renders the participant id and connection quality, and no media if they are not available', () => {
  const store = configureStore(zocksStoreConfig)
  store.dispatch(setRoomState(RoomState.CONNECTED))
  store.dispatch(setSessionId('testSessionId'))
  renderWithState(store, testLocalParticipant)

  expect(screen.getByText('testLocalParticipantName')).toBeInTheDocument()
  expect(screen.queryByTestId('videoElement')).not.toBeInTheDocument()
  expect(screen.queryByTestId('audioElement')).not.toBeInTheDocument()
})

test('renders audio and video tags for a remote participant if both are available, and attaches the corresponding tracks to them', () => {
  const { mockRemoteAudioTrack, mockRemoteVideoTrack } = setupZocksSdkMocks()
  const store = configureStore(zocksStoreConfig)
  store.dispatch(setRoomState(RoomState.CONNECTED))
  store.dispatch(setSessionId('testSessionId'))
  renderWithState(store, testRemoteParticipant)

  expect(screen.getByTestId('audioElement')).toBeInTheDocument()
  expect(screen.getByTestId('videoElement')).toBeInTheDocument()
  expect(mockRemoteAudioTrack.attach).toHaveBeenCalledTimes(1)
  expect(mockRemoteVideoTrack.attach).toHaveBeenCalledTimes(1)
  expect(mockRemoteAudioTrack.attach.mock.calls[0][0]).toBeInstanceOf(HTMLAudioElement)
  expect(mockRemoteVideoTrack.attach.mock.calls[0][0]).toBeInstanceOf(HTMLVideoElement)
})

test('renders only video tag for the local participant even if there is also audio, and attaches the corresponding track to it', () => {
  const { mockLocalAudioTrack, mockLocalVideoTrack } = setupZocksSdkMocks()
  const store = configureStore(zocksStoreConfig)
  store.dispatch(setRoomState(RoomState.CONNECTED))
  store.dispatch(setSessionId('testSessionId'))
  store.dispatch(setLocalParticipantId('testLocalParticipantId'))
  renderWithState(store, testLocalParticipant)

  expect(screen.getByTestId('videoElement')).toBeInTheDocument()
  expect(screen.queryByTestId('audioElement')).not.toBeInTheDocument()
  expect(mockLocalAudioTrack.attach).not.toHaveBeenCalled()
  expect(mockLocalVideoTrack.attach).toHaveBeenCalledTimes(1)
  expect(mockLocalVideoTrack.attach.mock.calls[0][0]).toBeInstanceOf(HTMLVideoElement)
})

test('does not render video or audio tags for muted tracks', () => {
  const { mockRemoteAudioTrack, mockRemoteVideoTrack } = setupZocksSdkMocks()
  const store = configureStore(zocksStoreConfig)
  store.dispatch(setRoomState(RoomState.CONNECTED))
  store.dispatch(setSessionId('testSessionId'))
  renderWithState(store, {
    ...testRemoteParticipant,
    tracks: {
      audioTracks: [
        {
          id: 'testRemoteAudioTrackId',
          isMuted: true,
        },
      ],
      videoTracks: [
        {
          id: 'testRemoteVideoTrackId',
          isMuted: true,
        },
      ],
      screenShareTrack: null,
    },
  })

  expect(screen.queryByTestId('audioElement')).not.toBeInTheDocument()
  expect(screen.queryByTestId('videoElement')).not.toBeInTheDocument()
  expect(mockRemoteAudioTrack.attach).not.toHaveBeenCalled()
  expect(mockRemoteVideoTrack.attach).not.toHaveBeenCalled()
})

test('has custom style when the participant is speaking', () => {
  const store = configureStore(zocksStoreConfig)
  store.dispatch(setRoomState(RoomState.CONNECTED))
  store.dispatch(setSessionId('testSessionId'))
  store.dispatch(setLocalParticipantId('testLocalParticipantId'))
  store.dispatch(setActiveSpeakers(['testLocalParticipantId']))
  const { container } = renderWithState(store, testLocalParticipant)

  expect(container.querySelector('.participant__video-container--speaking')).toBeInTheDocument()
})

test('does not render an audio tag if withAudio is false', () => {
  setupZocksSdkMocks()
  const store = configureStore(zocksStoreConfig)
  store.dispatch(setRoomState(RoomState.CONNECTED))
  store.dispatch(setSessionId('testSessionId'))
  renderWithState(store, testRemoteParticipant, false)

  expect(screen.queryByTestId('audioElement')).not.toBeInTheDocument()
  expect(screen.getByTestId('videoElement')).toBeInTheDocument()
})
