import React, { useEffect, useState } from 'react'
import classNames from 'classnames'
import { Participant } from '../../Participant'
import { useZocksStore } from '../../../app'
import { selectLocalParticipant } from '../../../store'
import { usePagedParticipants, useSortRemoteParticipantsBySpeakers } from '../hooks'
import { ParticipantModel } from '../../../models'
import './ParticipantsSpeakerView.scss'

export const ParticipantsSpeakerView: React.FC<{ pageSize?: number }> = ({ pageSize = 4 }) => {
  const localParticipant = useZocksStore(selectLocalParticipant)

  const { remoteParticipantsSortedBySpeakers } = useSortRemoteParticipantsBySpeakers()
  const remoteParticipantsExist = !!remoteParticipantsSortedBySpeakers.length

  const mainParticipant = remoteParticipantsExist ? remoteParticipantsSortedBySpeakers[0] : localParticipant
  const allParticipants = [localParticipant, ...remoteParticipantsSortedBySpeakers]
  const { displayedParticipants, hasNextPage, hasPreviousPage, nextPage, previousPage } = usePagedParticipants(
    allParticipants,
    pageSize
  )

  return (
    <div className='participants-speaker-view'>
      <div
        className={classNames('participants-speaker-view__main-speaker', {
          'participants-speaker-view__only-speaker': !remoteParticipantsExist,
        })}
      >
        {mainParticipant && <MainParticipant mainParticipant={mainParticipant} />}
      </div>
      {remoteParticipantsExist && (
        <>
          {hasPreviousPage && <PreviousButton onClick={previousPage} />}
          <div className='participants-speaker-view__others-container'>
            <div className='participants-speaker-view__others'>
              {displayedParticipants.map((participant) => (
                <Participant key={participant.id} participant={participant} />
              ))}
            </div>
          </div>
          {hasNextPage && <NextButton onClick={nextPage} />}
        </>
      )}
    </div>
  )
}

const MainParticipant: React.FC<{ mainParticipant: ParticipantModel }> = ({ mainParticipant }) => {
  const SPEAKER_CHANGE_DELAY = 500

  const [shouldDisplayMainSpeaker, setShouldDisplayMainSpeaker] = useState(true)
  const [lastSpeaker, setLastSpeaker] = useState(mainParticipant)

  useEffect(() => {
    setShouldDisplayMainSpeaker(false)
    const timeout = setTimeout(() => {
      setShouldDisplayMainSpeaker(true)
      setLastSpeaker(mainParticipant)
    }, SPEAKER_CHANGE_DELAY)
    return () => clearTimeout(timeout)
  }, [mainParticipant])

  return (
    <>
      <div
        className={classNames('main-speaker__participant', {
          'main-speaker__participant--hidden': !shouldDisplayMainSpeaker,
        })}
        aria-hidden={!shouldDisplayMainSpeaker}
        data-testid='mainSpeakerElement'
      >
        <Participant participant={mainParticipant} />
      </div>
      {lastSpeaker && (
        <div
          className={classNames('main-speaker__participant', {
            'main-speaker__participant--hidden': shouldDisplayMainSpeaker,
          })}
          aria-hidden={shouldDisplayMainSpeaker}
          data-testid='lastSpeakerElement'
        >
          <Participant participant={lastSpeaker} />
        </div>
      )}
    </>
  )
}

const PreviousButton: React.FC<{ onClick: () => void }> = ({ onClick }) => (
  <button
    className='participants-speaker-view__button participants-speaker-view__previous-button'
    onClick={onClick}
    aria-label='Previous'
  >
    <div className='participants-speaker-view__previous-button-icon' />
  </button>
)

const NextButton: React.FC<{ onClick: () => void }> = ({ onClick }) => (
  <button
    className='participants-speaker-view__button participants-speaker-view__next-button'
    onClick={onClick}
    aria-label='Next'
  >
    <div className='participants-speaker-view__next-button-icon' />
  </button>
)
