import { render, screen } from '@testing-library/react'
import {
  addRemoteParticipant,
  participantLocalTrackPublished,
  subscribeToParticipantTrack,
} from '../../store/session/sessionSlice'
import { StreamKind, StreamSource } from '@zocks-communications/client-sdk-js'
import { configureStore, EnhancedStore } from '@reduxjs/toolkit'
import { Provider } from 'react-redux'
import { zocksStoreConfig } from '../../app/store'
import { RoomControls } from './RoomControls'
import userEvent from '@testing-library/user-event'
import { ZocksMediaActions } from '../../models'
import { setupMediaDevicesMock } from '../../testUtils'

const mockMediaActions: ZocksMediaActions = {
  publishVideo: jest.fn(),
  unPublishVideo: jest.fn(),
  publishAudio: jest.fn(),
  unPublishAudio: jest.fn(),
  publishScreenShare: jest.fn(),
  unPublishScreenShare: jest.fn(),
  switchActiveDevice: jest.fn(),
}

const mockLeaveAction = jest.fn()

beforeEach(() => setupMediaDevicesMock())

afterEach(() => jest.restoreAllMocks())

const loadDevices = async () => {
  const audioDevicesButton = screen.getByRole('button', { name: 'Audio devices' })
  userEvent.click(audioDevicesButton)
  expect(await screen.findByText('Test Microphone 1')).toBeInTheDocument()
  userEvent.click(audioDevicesButton)
}

const renderWithState = async (store: EnhancedStore) => {
  render(
    <Provider store={store}>
      <RoomControls mediaActions={mockMediaActions} leaveAction={mockLeaveAction} />
    </Provider>
  )
  await loadDevices()
}

test('renders a muted audio control button, a muted video control button, a screen share button, device selectors and a leave button by default', async () => {
  const store = configureStore(zocksStoreConfig)
  await renderWithState(store)

  expect(screen.getByRole('button', { name: 'Unmute' })).toBeInTheDocument()
  expect(screen.getByRole('button', { name: 'Audio devices' })).toBeInTheDocument()
  expect(screen.getByRole('button', { name: 'Enable Video' })).toBeInTheDocument()
  expect(screen.getByRole('button', { name: 'Video devices' })).toBeInTheDocument()
  expect(screen.getByRole('button', { name: 'Share Screen' })).toBeInTheDocument()
  expect(screen.getByRole('button', { name: 'Leave' })).toBeInTheDocument()

  userEvent.click(screen.getByRole('button', { name: 'Audio devices' }))
  expect(await screen.findByText('Test Microphone 1')).toBeInTheDocument()
  userEvent.click(screen.getByRole('button', { name: 'Video devices' }))
  expect(await screen.findByText('Test Camera 1')).toBeInTheDocument()
})

test('clicking on the muted audio button calls the publish audio media action', async () => {
  const store = configureStore(zocksStoreConfig)
  await renderWithState(store)

  const audioButton = screen.getByRole('button', { name: 'Unmute' })
  userEvent.click(audioButton)

  expect(mockMediaActions.publishAudio).toHaveBeenCalledTimes(1)
})

test('when the local participant has unmuted audio, renders an unmuted audio button, which calls the unpublish audio media action', async () => {
  const store = configureStore(zocksStoreConfig)
  store.dispatch(
    participantLocalTrackPublished({
      trackId: 'testTrackId',
      trackKind: StreamKind.AUDIO,
      trackSource: StreamSource.MICROPHONE,
      trackIsMuted: false,
    })
  )
  await renderWithState(store)

  const audioButton = screen.getByRole('button', { name: 'Mute' })
  expect(audioButton).toBeInTheDocument()

  userEvent.click(audioButton)
  expect(mockMediaActions.unPublishAudio).toHaveBeenCalledTimes(1)
})

test('clicking on the muted video button calls the publish video media action', async () => {
  const store = configureStore(zocksStoreConfig)
  await renderWithState(store)

  const videoButton = screen.getByRole('button', { name: 'Enable Video' })
  userEvent.click(videoButton)

  expect(mockMediaActions.publishVideo).toHaveBeenCalledTimes(1)
})

test('when the local participant has unmuted video, renders an unmuted audio button, which calls the unpublish video media action', () => {
  const store = configureStore(zocksStoreConfig)
  store.dispatch(
    participantLocalTrackPublished({
      trackId: 'testTrackId',
      trackKind: StreamKind.VIDEO,
      trackSource: StreamSource.CAMERA,
      trackIsMuted: false,
    })
  )
  renderWithState(store)

  const videoButton = screen.getByRole('button', { name: 'Disable Video' })
  expect(videoButton).toBeInTheDocument()

  userEvent.click(videoButton)
  expect(mockMediaActions.unPublishVideo).toHaveBeenCalledTimes(1)
})

test('clicking on the screen share button calls the publish screen share media action', async () => {
  const store = configureStore(zocksStoreConfig)
  await renderWithState(store)

  const screenShareButton = screen.getByRole('button', { name: 'Share Screen' })
  userEvent.click(screenShareButton)

  expect(mockMediaActions.publishScreenShare).toHaveBeenCalledTimes(1)
})

test('when the local participant is sharing their screen, renders a Stop Sharing button, which calls the unpublish screen share media action', async () => {
  const store = configureStore(zocksStoreConfig)
  store.dispatch(
    participantLocalTrackPublished({
      trackId: 'testTrackId',
      trackKind: StreamKind.VIDEO,
      trackSource: StreamSource.SCREEN_SHARE,
      trackIsMuted: false,
    })
  )
  await renderWithState(store)

  const screenShareButton = screen.getByRole('button', { name: 'Stop Sharing' })
  expect(screenShareButton).toBeInTheDocument()
  expect(screenShareButton).not.toBeDisabled()

  userEvent.click(screenShareButton)
  expect(mockMediaActions.unPublishScreenShare).toHaveBeenCalledTimes(1)
})

test('when a remote participant is sharing their screen, the Share Screen button is disabled', async () => {
  const store = configureStore(zocksStoreConfig)
  store.dispatch(
    addRemoteParticipant({ participantId: 'testRemoteParticipantId', participantName: 'testRemoteParticipantName' })
  )
  store.dispatch(
    subscribeToParticipantTrack({
      participantId: 'testRemoteParticipantId',
      trackId: 'testTrackId',
      trackKind: StreamKind.VIDEO,
      trackSource: StreamSource.SCREEN_SHARE,
      trackIsMuted: false,
    })
  )
  await renderWithState(store)

  expect(screen.getByRole('button', { name: 'Share Screen' })).toBeDisabled()
})

test('clicking on the leave button calls the leave action', async () => {
  const store = configureStore(zocksStoreConfig)
  await renderWithState(store)

  const leaveButton = screen.getByRole('button', { name: 'Leave' })
  userEvent.click(leaveButton)

  expect(mockLeaveAction).toHaveBeenCalledTimes(1)
})

test('clicking on an audio device calls the switchActiveDevice action', async () => {
  const store = configureStore(zocksStoreConfig)
  await renderWithState(store)

  userEvent.click(screen.getByRole('button', { name: 'Audio devices' }))
  userEvent.click(screen.getByText('Test Microphone 2'))

  expect(mockMediaActions.switchActiveDevice).toHaveBeenCalledTimes(1)
  expect(mockMediaActions.switchActiveDevice).toHaveBeenCalledWith('audioinput', 'testMicrophone2MediaDeviceId')
})

test('clicking on a video device calls the switchActiveDevice action', async () => {
  const store = configureStore(zocksStoreConfig)
  await renderWithState(store)

  userEvent.click(screen.getByRole('button', { name: 'Video devices' }))
  userEvent.click(screen.getByText('Test Camera 2'))

  expect(mockMediaActions.switchActiveDevice).toHaveBeenCalledTimes(1)
  expect(mockMediaActions.switchActiveDevice).toHaveBeenCalledWith('videoinput', 'testCamera2MediaDeviceId')
})
