import { useLocalMedia } from './hooks'
import React from 'react'
import { setupMockLocalMediaStream } from '../../testUtils'

beforeEach(() => {
  // @ts-ignore
  jest.spyOn(React, 'useState').mockImplementation((initialState) => [initialState, jest.fn()])
  jest.spyOn(React, 'useEffect').mockImplementation((fn) => fn())
  jest.spyOn(React, 'useCallback').mockImplementation((fn) => fn)
})

afterEach(() => {
  jest.restoreAllMocks()
})

const mockAudioDeviceInfo: MediaDeviceInfo = {
  deviceId: 'testAudioDeviceId2',
  kind: 'audioinput',
  groupId: 'test',
  label: 'test',
  toJSON(): any {},
}

const mockVideoDeviceInfo: MediaDeviceInfo = {
  deviceId: 'testVideoDeviceId2',
  kind: 'videoinput',
  groupId: 'test',
  label: 'test',
  toJSON(): any {},
}

test('toggleLocalAudioStream starts the local audio stream if it is not currently on', () => {
  const { mockCreateMediaStream } = setupMockLocalMediaStream()
  const { toggleLocalAudioStream } = useLocalMedia()

  toggleLocalAudioStream()

  expect(mockCreateMediaStream).toHaveBeenCalledTimes(1)
  expect(mockCreateMediaStream).toHaveBeenCalledWith({ audio: { deviceId: undefined }, video: false })
})

test('toggleLocalAudioStream stops the local audio stream if it is currently on', () => {
  const mockSetState = jest.fn()
  jest.spyOn(React, 'useState').mockImplementation(() => [mockLocalAudioStream, mockSetState])
  const { mockLocalMediaStream: mockLocalAudioStream } = setupMockLocalMediaStream()
  const { toggleLocalAudioStream } = useLocalMedia()

  toggleLocalAudioStream()

  expect(mockLocalAudioStream.stopStream).toHaveBeenCalledTimes(1)
  expect(mockSetState).toHaveBeenCalledTimes(1)
  expect(mockSetState).toHaveBeenCalledWith(undefined)
})

test('toggleLocalVideoStream starts the local video stream if it is not currently on', () => {
  const { mockCreateMediaStream } = setupMockLocalMediaStream()
  const { toggleLocalVideoStream } = useLocalMedia()

  toggleLocalVideoStream()

  expect(mockCreateMediaStream).toHaveBeenCalledTimes(1)
  expect(mockCreateMediaStream).toHaveBeenCalledWith({ audio: false, video: { deviceId: undefined } })
})

test('toggleLocalVideoStream stops the local video stream if it is currently on', () => {
  const { mockLocalMediaStream: mockLocalVideoStream } = setupMockLocalMediaStream()
  const mockSetState = jest.fn()
  jest.spyOn(React, 'useState').mockImplementation(() => [mockLocalVideoStream, mockSetState])
  const { toggleLocalVideoStream } = useLocalMedia()

  toggleLocalVideoStream()

  expect(mockLocalVideoStream.stopStream).toHaveBeenCalledTimes(1)
  expect(mockSetState).toHaveBeenCalledTimes(1)
  expect(mockSetState).toHaveBeenCalledWith(undefined)
})

test('selectAudioDevice starts a new local audio stream on the selected device if there is already an audio stream', () => {
  const { mockLocalMediaStream: mockLocalAudioStream, mockCreateMediaStream } =
    setupMockLocalMediaStream('testAudioDeviceId')
  jest.spyOn(React, 'useState').mockImplementation(() => [mockLocalAudioStream, jest.fn()])
  const { selectAudioDevice } = useLocalMedia()

  selectAudioDevice(mockAudioDeviceInfo)

  expect(mockLocalAudioStream.stopStream).toHaveBeenCalledTimes(1)
  expect(mockCreateMediaStream).toHaveBeenCalledTimes(1)
  expect(mockCreateMediaStream).toHaveBeenCalledWith({ audio: { deviceId: 'testAudioDeviceId2' }, video: false })
})

test('selectAudioDevice does not start a new local stream if the same device was selected', () => {
  const { mockLocalMediaStream: mockLocalAudioStream, mockCreateMediaStream } =
    setupMockLocalMediaStream('testAudioDeviceId')
  const mockSetState = jest.fn()
  jest.spyOn(React, 'useState').mockImplementation(() => [mockLocalAudioStream, mockSetState])
  const { selectAudioDevice } = useLocalMedia()

  selectAudioDevice({ ...mockAudioDeviceInfo, deviceId: 'testAudioDeviceId' })

  expect(mockLocalAudioStream.stopStream).not.toHaveBeenCalled()
  expect(mockCreateMediaStream).not.toHaveBeenCalled()
})

test('selectAudioDevice does not start a new local stream if there is currently no audio stream, but sets the audio device', () => {
  const { mockLocalMediaStream: mockLocalAudioStream, mockCreateMediaStream } =
    setupMockLocalMediaStream('testAudioDeviceId')
  const mockSetState = jest.fn()
  jest.spyOn(React, 'useState').mockImplementation(() => [undefined, mockSetState])
  const { selectAudioDevice } = useLocalMedia()

  selectAudioDevice(mockAudioDeviceInfo)

  expect(mockSetState).toHaveBeenCalledTimes(1)
  expect(mockSetState).toHaveBeenCalledWith(mockAudioDeviceInfo)
  expect(mockLocalAudioStream.stopStream).not.toHaveBeenCalled()
  expect(mockCreateMediaStream).not.toHaveBeenCalled()
})

test('selectVideoDevice starts a new local video stream on the selected device if there is already a video stream', () => {
  const { mockLocalMediaStream: mockLocalVideoStream, mockCreateMediaStream } =
    setupMockLocalMediaStream('testVideoDeviceId')
  jest.spyOn(React, 'useState').mockImplementation(() => [mockLocalVideoStream, jest.fn()])
  const { selectVideoDevice } = useLocalMedia()

  selectVideoDevice(mockVideoDeviceInfo)

  expect(mockLocalVideoStream.stopStream).toHaveBeenCalledTimes(1)
  expect(mockCreateMediaStream).toHaveBeenCalledTimes(1)
  expect(mockCreateMediaStream).toHaveBeenCalledWith({ audio: false, video: { deviceId: 'testVideoDeviceId2' } })
})

test('selectVideoDevice does not start a new local stream if the same device was selected', () => {
  const { mockLocalMediaStream: mockLocalVideoStream, mockCreateMediaStream } =
    setupMockLocalMediaStream('testVideoDeviceId')
  const mockSetState = jest.fn()
  jest.spyOn(React, 'useState').mockImplementation(() => [mockLocalVideoStream, mockSetState])
  const { selectVideoDevice } = useLocalMedia()

  selectVideoDevice({ ...mockVideoDeviceInfo, deviceId: 'testVideoDeviceId' })

  expect(mockLocalVideoStream.stopStream).not.toHaveBeenCalled()
  expect(mockCreateMediaStream).not.toHaveBeenCalled()
})

test('selectVideoDevice does not start a new local stream if there is currently no video stream, but sets the video device', () => {
  const { mockLocalMediaStream: mockLocalVideoStream, mockCreateMediaStream } =
    setupMockLocalMediaStream('testVideoDeviceId')
  const mockSetState = jest.fn()
  jest.spyOn(React, 'useState').mockImplementation(() => [undefined, mockSetState])
  const { selectVideoDevice } = useLocalMedia()

  selectVideoDevice(mockVideoDeviceInfo)

  expect(mockSetState).toHaveBeenCalledTimes(1)
  expect(mockSetState).toHaveBeenCalledWith(mockVideoDeviceInfo)
  expect(mockLocalVideoStream.stopStream).not.toHaveBeenCalled()
  expect(mockCreateMediaStream).not.toHaveBeenCalled()
})

test('automatically start the local audio stream if audio is enabled by default', () => {
  const { mockCreateMediaStream } = setupMockLocalMediaStream()

  useLocalMedia(false, true)

  expect(mockCreateMediaStream).toHaveBeenCalledWith({ audio: { deviceId: undefined }, video: false })
})

test('automatically start the local video stream if video is enabled by default', () => {
  const { mockCreateMediaStream } = setupMockLocalMediaStream()

  useLocalMedia(true, false)

  expect(mockCreateMediaStream).toHaveBeenCalledWith({ audio: false, video: { deviceId: undefined } })
})
