import { useCallback, useEffect, useState } from 'react'
import { LocalMediaStream } from '@zocks-communications/client-sdk-js'

export const useLocalMedia = (videoEnabledByDefault?: boolean, audioEnabledByDefault?: boolean) => {
  const [audioDevice, setAudioDevice] = useState<MediaDeviceInfo>()
  const [videoDevice, setVideoDevice] = useState<MediaDeviceInfo>()
  const [localAudioStream, setLocalAudioStream] = useState<LocalMediaStream>()
  const [localVideoStream, setLocalVideoStream] = useState<LocalMediaStream>()
  const [videoStreamIsLoading, setVideoStreamIsLoading] = useState<boolean>(false)

  const startLocalAudioStream = useCallback((deviceId?: string) => {
    LocalMediaStream.create({ audio: { deviceId }, video: false }).then((audioStream) =>
      setLocalAudioStream(audioStream)
    )
  }, [])

  const startLocalVideoStream = useCallback((deviceId?: string) => {
    setVideoStreamIsLoading(true)
    LocalMediaStream.create({ audio: false, video: { deviceId } })
      .then((videoStream) => setLocalVideoStream(videoStream))
      .finally(() => setVideoStreamIsLoading(false))
  }, [])

  const toggleLocalAudioStream = () => {
    if (localAudioStream) {
      localAudioStream.stopStream()
      setLocalAudioStream(undefined)
    } else {
      startLocalAudioStream(audioDevice?.deviceId)
    }
  }

  const toggleLocalVideoStream = () => {
    if (localVideoStream) {
      localVideoStream.stopStream()
      setLocalVideoStream(undefined)
    } else {
      startLocalVideoStream(videoDevice?.deviceId)
    }
  }

  const selectAudioDevice = (audioDevice: MediaDeviceInfo) => {
    setAudioDevice(audioDevice)
    if (localAudioStream) {
      if (localAudioStream.mediaStreamTrack?.getSettings().deviceId === audioDevice.deviceId) {
        return
      }
      localAudioStream.stopStream()
      startLocalAudioStream(audioDevice.deviceId)
    }
  }

  const selectVideoDevice = (videoDevice: MediaDeviceInfo) => {
    setVideoDevice(videoDevice)
    if (localVideoStream) {
      if (localVideoStream.mediaStreamTrack?.getSettings().deviceId === videoDevice.deviceId) {
        return
      }
      localVideoStream.stopStream()
      setLocalVideoStream(undefined)
      startLocalVideoStream(videoDevice.deviceId)
    }
  }

  useEffect(() => {
    if (videoEnabledByDefault) {
      startLocalVideoStream()
    }
  }, [videoEnabledByDefault, startLocalVideoStream])

  useEffect(() => {
    if (audioEnabledByDefault) {
      startLocalAudioStream()
    }
  }, [audioEnabledByDefault, startLocalAudioStream])

  return {
    localAudioStream,
    localVideoStream,
    videoStreamIsLoading,
    toggleLocalAudioStream,
    toggleLocalVideoStream,
    selectAudioDevice,
    selectVideoDevice,
  }
}
