import {
  ZocksEvent,
  ZocksSDK,
  RoomState,
  Track,
  Session,
  RemoteParticipant,
  ConnectionQuality,
  LocalParticipant,
  Participant,
  RoomInfo,
  ChatMessage,
  StreamKind,
  StreamSource,
} from '@zocks-communications/client-sdk-js'
import { useZocksDispatch, useZocksStore } from '../../app/hooks'
import {
  setLocalParticipantId,
  setRoomState,
  setSessionId,
  addRemoteParticipant,
  subscribeToParticipantTrack,
  connectionQualityChanged,
  participantLocalTrackPublished,
  participantLocalTrackUnPublished,
  removeRemoteParticipant,
  clearSessionId,
  participantTrackMuted,
  participantTrackUnMuted,
  selectTracksForParticipant,
  selectLocalParticipant,
  selectSessionId,
  selectRoomState,
  selectRemoteParticipants,
  selectLocalParticipantIsConnected,
  selectIsConnecting,
  setConnecting,
  setActiveSpeakers,
  selectActiveSpeakers,
  participantTrackUnPublished,
  setLocalParticipantName,
  clearParticipants,
} from './sessionSlice'
import { useEffect, useState } from 'react'
import { ZocksConfig } from '../../app/zocksConfig'
import { JoinOptions, ParticipantModel, ParticipantView, SessionView, ZocksActions } from '../../models'
import {
  clearChatMessages,
  privateChatMessageReceived,
  privateChatMessageSent,
  savePublicChatMessage,
} from '../chat/chatSlice'
import { ChatAttachmentModel } from '../chat/models'
import { getSessionById } from '../utils'

export const useZocksActions = (zocksConfig: ZocksConfig): ZocksActions => {
  const dispatch = useZocksDispatch()

  const [session, setSession] = useState<Session>()

  const setupEventHandlers = (session: Session) => {
    session.on(ZocksEvent.ParticipantLocalTrackPublished, (track: Track) => {
      if (track.isAvailable) {
        dispatch(
          participantLocalTrackPublished({
            trackId: track.id!,
            trackKind: track.kind,
            trackSource: track.source,
            trackIsMuted: track.isMuted,
          })
        )
      }
    })
    session.on(ZocksEvent.ParticipantLocalTrackUnpublished, (track: Track) => {
      if (track.isAvailable) {
        dispatch(
          participantLocalTrackUnPublished({ trackId: track.id!, trackKind: track.kind, trackSource: track.source })
        )
      }
    })
    session.on(ZocksEvent.ParticipantTrackUnPublished, (track: Track, participant: Participant) => {
      if (track.isAvailable) {
        dispatch(
          participantTrackUnPublished({
            participantId: participant.id,
            trackId: track.id!,
            trackKind: track.kind,
            trackSource: track.source,
          })
        )
      }
    })
    session.on(ZocksEvent.SessionConnected, () => {
      dispatch(setSessionId(session.id!))
    })
    session.on(ZocksEvent.SessionStateChanged, (roomState: RoomState) => {
      dispatch(setRoomState(roomState))
      if (roomState === RoomState.DISCONNECTED) {
        dispatch(clearParticipants())
        dispatch(clearSessionId())
        dispatch(clearChatMessages())
      }
    })
    session.on(ZocksEvent.LocalParticipantConnected, (localParticipant: Participant) => {
      dispatch(setLocalParticipantId(localParticipant.id))
      dispatch(setLocalParticipantName(localParticipant.name))
    })
    session.on(ZocksEvent.ParticipantConnected, (participant: RemoteParticipant) => {
      dispatch(addRemoteParticipant({ participantId: participant.id, participantName: participant.name }))
    })
    session.on(ZocksEvent.ParticipantDisconnected, (participantId: string | undefined) => {
      if (participantId) {
        dispatch(removeRemoteParticipant({ participantId }))
      }
    })
    session.on(ZocksEvent.ParticipantTrackSubscribed, (track: Track, participant: RemoteParticipant) => {
      if (participant.id && track.isAvailable) {
        dispatch(
          subscribeToParticipantTrack({
            participantId: participant.id,
            trackId: track.id!,
            trackKind: track.kind,
            trackSource: track.source,
            trackIsMuted: track.isMuted,
          })
        )
      }
    })
    session.on(
      ZocksEvent.ConnectionQualityChanged,
      (connectionQuality: ConnectionQuality, participant: LocalParticipant | RemoteParticipant) => {
        dispatch(connectionQualityChanged({ participantId: participant.id, connectionQuality }))
      }
    )
    session.on(ZocksEvent.ParticipantTrackMuted, (participant: Participant, track: Track) => {
      dispatch(participantTrackMuted({ participantId: participant.id, trackId: track.id }))
    })
    session.on(ZocksEvent.ParticipantTrackUnMuted, (participant: Participant, track: Track) => {
      dispatch(participantTrackUnMuted({ participantId: participant.id, trackId: track.id }))
    })
    session.on(ZocksEvent.ActiveSpeakersChanged, (participants: Participant[]) => {
      dispatch(setActiveSpeakers(participants.filter((p) => !!p.id).map((p) => p.id)))
    })

    session.on(ZocksEvent.PrivateChatMessageReceived, (chatMessage: ChatMessage) => {
      const { message, timestamp, from, attachment } = chatMessage
      dispatch(
        privateChatMessageReceived({
          message,
          timestamp,
          from,
          to: {
            id: session.localParticipant.id,
            name: session.localParticipant.name!,
          },
          attachment,
        })
      )
    })
    session.on(ZocksEvent.PublicChatMessageReceived, (chatMessage: ChatMessage) => {
      const { message, timestamp, from, attachment } = chatMessage

      dispatch(
        savePublicChatMessage({
          message,
          timestamp,
          from,
          attachment,
        })
      )
    })
  }

  const createRoom = (): Promise<RoomInfo> => {
    const zocksSDK = ZocksSDK.init(zocksConfig)
    return zocksSDK.API.room.create()
  }

  const join = (joinOptions: JoinOptions) => {
    const { roomId, autoCreate, audioDeviceSettings, videoDeviceSettings, afterJoinCallback, joinErrorCallback } =
      joinOptions
    const zocksSDK = ZocksSDK.init(zocksConfig)

    const zocksSession = zocksSDK.createNewSession()
    setSession(zocksSession)
    setupEventHandlers(zocksSession)
    const joinRoom = async () => {
      dispatch(setConnecting(true))
      await zocksSession.join(roomId, autoCreate)
    }
    joinRoom()
      .then(() => {
        afterJoinCallback && afterJoinCallback()
        audioDeviceSettings && zocksSession.localParticipant.publishAudio(audioDeviceSettings)
        videoDeviceSettings && zocksSession.localParticipant.publishVideo(videoDeviceSettings)
      })
      .catch((e) => joinErrorCallback && joinErrorCallback(e))
      .finally(() => dispatch(setConnecting(false)))
  }

  const leave = () => {
    session?.disconnect()
  }

  const publishVideo = (mediaTrackSettings?: MediaTrackSettings) => {
    session?.localParticipant.publishVideo(mediaTrackSettings)
  }

  const unPublishVideo = (mediaTrackSettings?: MediaTrackSettings) => {
    session?.localParticipant.unPublishVideo(mediaTrackSettings)
  }

  const publishAudio = (mediaTrackSettings?: MediaTrackSettings) => {
    session?.localParticipant.publishAudio(mediaTrackSettings)
  }

  const unPublishAudio = (mediaTrackSettings?: MediaTrackSettings) => {
    session?.localParticipant.unPublishAudio(mediaTrackSettings)
  }

  const publishScreenShare = () => {
    session?.localParticipant.publishScreenShare().catch(console.error)
  }

  const unPublishScreenShare = () => {
    session?.localParticipant.unPublishScreenShare()
  }

  const switchActiveDevice = (kind: MediaDeviceKind, deviceId: string) => {
    session?.switchActiveDevice(kind, deviceId)
  }

  const sendPublicMessage = (message: string, attachment?: ChatAttachmentModel) => {
    if (!session) {
      throw Error('Cannot send public message without a session!')
    }
    session.sendMessageToRoom(message, attachment)
    dispatch(
      savePublicChatMessage({
        message,
        timestamp: Date.now(),
        from: {
          id: session.localParticipant.id,
          name: session.localParticipant.name!,
        },
        attachment,
      })
    )
  }

  const sendPrivateMessage = (
    message: string,
    remoteParticipant: ParticipantModel,
    attachment?: ChatAttachmentModel
  ) => {
    if (!remoteParticipant.id || !session) {
      throw Error('Cannot send private message without a session or recipient!')
    }
    session.sendMessageToParticipant(message, remoteParticipant.id, attachment)
    dispatch(
      privateChatMessageSent({
        message,
        timestamp: Date.now(),
        from: {
          id: session.localParticipant.id,
          name: session.localParticipant.name!,
        },
        to: {
          id: remoteParticipant.id,
          name: remoteParticipant.name!,
        },
        attachment,
      })
    )
  }

  const uploadAttachment = async (file: File, remoteParticipant?: ParticipantModel) => {
    if ((remoteParticipant && !remoteParticipant.id) || !session) {
      throw Error('Cannot upload attachment without a session or with empty recipient id!')
    }
    const recipientId = remoteParticipant?.id || undefined
    return session.uploadAttachment(file, recipientId)
  }

  const getAttachmentLocation = async (attachmentFileId: string) => {
    if (!session) {
      throw Error('Cannot get attachment location without a session!')
    }
    return session.getFileDownloadLocation(attachmentFileId)
  }

  return {
    session,
    sessionActions: {
      createRoom,
      join,
      leave,
    },
    localMediaActions: {
      publishVideo,
      publishAudio,
      unPublishVideo,
      unPublishAudio,
      publishScreenShare,
      unPublishScreenShare,
      switchActiveDevice,
    },
    chatActions: {
      sendPublicMessage,
      sendPrivateMessage,
      uploadAttachment,
      getAttachmentLocation,
    },
  }
}

export const useZocksSession = (): SessionView => {
  const sessionId = useZocksStore(selectSessionId)
  const roomState = useZocksStore(selectRoomState)
  const isConnecting = useZocksStore(selectIsConnecting)
  const localParticipant = useZocksStore(selectLocalParticipant)
  const remoteParticipants = useZocksStore(selectRemoteParticipants)
  const localParticipantIsConnected = useZocksStore(selectLocalParticipantIsConnected)

  return {
    sessionId,
    roomState,
    isConnecting,
    localParticipant,
    remoteParticipants,
    localParticipantIsConnected,
  }
}

export const useParticipant = (participantModel: ParticipantModel): ParticipantView => {
  const tracks = useZocksStore((state) => selectTracksForParticipant(state, participantModel))
  const localParticipant = useZocksStore(selectLocalParticipant)
  const activeSpeakers = useZocksStore(selectActiveSpeakers)
  const isLocal = participantModel.id === localParticipant.id
  const isSpeaking = activeSpeakers.some((participantId) => participantId === participantModel.id)

  const { id, name, connectionQuality } = participantModel

  return {
    id,
    name,
    tracks,
    isLocal,
    connectionQuality,
    isSpeaking,
  }
}

export const useLeaveOnUnload = (leave: () => void) => {
  useEffect(() => {
    window.onunload = () => {
      leave()
    }
  }, [leave])
}

export const useSynchronizeParticipants = () => {
  const dispatch = useZocksDispatch()
  const stateParticipants = useZocksStore(selectRemoteParticipants)
  const sessionId = useZocksStore(selectSessionId)
  const zocksSession = getSessionById(sessionId)

  const synchronizeParticipants = () => {
    if (!zocksSession) {
      return
    }
    const { participants: zocksParticipants } = zocksSession
    for (const stateParticipant of stateParticipants) {
      if (stateParticipant.id && !zocksParticipants.has(stateParticipant.id)) {
        removeObsoleteParticipant(stateParticipant.id)
      }
    }

    for (const zocksParticipant of zocksParticipants.values()) {
      const { id: zocksParticipantId, name: zocksParticipantName, tracks: zocksParticipantTracks } = zocksParticipant
      const stateParticipant = stateParticipants.find((participant) => participant.id === zocksParticipantId)
      if (!stateParticipant) {
        addMissingParticipant(zocksParticipantTracks, zocksParticipantId, zocksParticipantName)
      } else {
        synchronizeExistingParticipant(stateParticipant, zocksParticipantTracks)
      }
    }
  }

  const removeObsoleteParticipant = (participantId: string) => {
    dispatch(removeRemoteParticipant({ participantId }))
  }

  const addMissingParticipant = (participantTracks: Track[], participantId: string, participantName?: string) => {
    dispatch(addRemoteParticipant({ participantId, participantName }))
    for (const zocksTrack of participantTracks) {
      subscribeToTrack(participantId, zocksTrack)
    }
  }

  const synchronizeExistingParticipant = (stateParticipant: ParticipantModel, zocksTracks: Track[]) => {
    fixMuteStatusOfTracks(stateParticipant, zocksTracks)
    addMissingTracks(stateParticipant, zocksTracks)
    removeObsoleteTracks(stateParticipant, zocksTracks)
  }

  const fixMuteStatusOfTracks = (stateParticipant: ParticipantModel, zocksTracks: Track[]) => {
    const {
      tracks: { videoTracks, audioTracks },
    } = stateParticipant
    for (const stateVideoTrack of videoTracks) {
      const zocksVideoTrack = zocksTracks.find((t) => t.id === stateVideoTrack.id)
      if (zocksVideoTrack && stateVideoTrack.isMuted !== zocksVideoTrack.isMuted) {
        changeMuteStatusOfTrack(stateParticipant.id!, stateVideoTrack.id, zocksVideoTrack.isMuted)
      }
    }
    for (const stateAudioTrack of audioTracks) {
      const zocksAudioTrack = zocksTracks.find((t) => t.id === stateAudioTrack.id)
      if (zocksAudioTrack && stateAudioTrack.isMuted !== zocksAudioTrack.isMuted) {
        changeMuteStatusOfTrack(stateParticipant.id!, stateAudioTrack.id, zocksAudioTrack.isMuted)
      }
    }
  }

  const addMissingTracks = (stateParticipant: ParticipantModel, zocksParticipantTracks: Track[]) => {
    const {
      id,
      tracks: { videoTracks, audioTracks, screenShareTrack },
    } = stateParticipant

    for (const zocksTrack of zocksParticipantTracks) {
      const trackIsMissingFromState =
        !videoTracks.some((t) => t.id === zocksTrack.id) &&
        !audioTracks.some((t) => t.id === zocksTrack.id) &&
        screenShareTrack?.id !== zocksTrack.id
      if (trackIsMissingFromState) {
        subscribeToTrack(id!, zocksTrack)
      }
    }
  }

  const removeObsoleteTracks = (stateParticipant: ParticipantModel, zocksTracks: Track[]) => {
    const {
      tracks: { videoTracks, audioTracks, screenShareTrack },
    } = stateParticipant
    for (const stateVideoTrack of videoTracks) {
      if (!zocksTracks.some((t) => t.id === stateVideoTrack.id)) {
        unsubscribeFromTrack(stateParticipant.id!, stateVideoTrack.id, StreamSource.CAMERA, StreamKind.VIDEO)
      }
    }
    for (const stateAudioTrack of audioTracks) {
      if (!zocksTracks.some((t) => t.id === stateAudioTrack.id)) {
        unsubscribeFromTrack(stateParticipant.id!, stateAudioTrack.id, StreamSource.MICROPHONE, StreamKind.AUDIO)
      }
    }
    if (screenShareTrack && !zocksTracks.some((t) => t.id === screenShareTrack.id)) {
      unsubscribeFromTrack(stateParticipant.id!, screenShareTrack.id, StreamSource.SCREEN_SHARE, StreamKind.VIDEO)
    }
  }

  const subscribeToTrack = (participantId: string, track: Track) => {
    dispatch(
      subscribeToParticipantTrack({
        participantId: participantId,
        trackId: track.id,
        trackKind: track.kind,
        trackSource: track.source,
        trackIsMuted: track.isMuted,
      })
    )
  }

  const unsubscribeFromTrack = (
    participantId: string,
    trackId: string,
    trackSource: StreamSource,
    trackKind: StreamKind
  ) => {
    dispatch(participantTrackUnPublished({ participantId, trackId, trackSource, trackKind }))
  }

  const changeMuteStatusOfTrack = (participantId: string, trackId: string, shouldMute: boolean) => {
    if (shouldMute) {
      dispatch(participantTrackMuted({ participantId, trackId }))
    } else {
      dispatch(participantTrackUnMuted({ participantId, trackId }))
    }
  }

  return { synchronizeParticipants }
}
