import { ConnectionQuality, RoomState, StreamKind, StreamSource } from '@zocks-communications/client-sdk-js'
import sessionReducer, {
  addRemoteParticipant,
  clearParticipants,
  clearSessionId,
  connectionQualityChanged,
  participantLocalTrackPublished,
  participantLocalTrackUnPublished,
  participantTrackMuted,
  participantTrackUnMuted,
  participantTrackUnPublished,
  removeRemoteParticipant,
  selectIsScreenSharing,
  selectScreenSharingParticipant,
  selectSpeakingRemoteParticipantAudioTracks,
  selectTracksForParticipant,
  setActiveSpeakers,
  setConnecting,
  setLocalParticipantId,
  setLocalParticipantName,
  setRoomState,
  setSessionId,
  subscribeToParticipantTrack,
} from './sessionSlice'
import { mockRootState, setupZocksSdkMocks } from '../../testUtils'
import { SessionModel } from '../../models'

afterEach(() => {
  jest.restoreAllMocks()
})

test('room has an initial state', () => {
  expect(sessionReducer(undefined, { type: 'unknown' })).toStrictEqual({
    sessionId: null,
    roomState: RoomState.DISCONNECTED,
    isConnecting: false,
    localParticipant: {
      id: null,
      name: null,
      connectionQuality: ConnectionQuality.EXCELLENT,
      tracks: {
        audioTracks: [],
        videoTracks: [],
        screenShareTrack: null,
      },
    },
    remoteParticipants: {},
    activeSpeakers: [],
  })
})

const state: SessionModel = {
  sessionId: 'testSessionId',
  roomState: RoomState.CONNECTED,
  isConnecting: false,
  localParticipant: {
    id: 'testLocalParticipantId',
    name: 'testLocalParticipantName',
    connectionQuality: ConnectionQuality.EXCELLENT,
    tracks: {
      audioTracks: [
        {
          id: 'testLocalAudioTrackId',
          isMuted: true,
        },
      ],
      videoTracks: [
        {
          id: 'testLocalVideoTrackId',
          isMuted: true,
        },
      ],
      screenShareTrack: null,
    },
  },
  remoteParticipants: {
    testRemoteParticipantId: {
      id: 'testRemoteParticipantId',
      name: 'testRemoteParticipantName',
      connectionQuality: ConnectionQuality.EXCELLENT,
      tracks: {
        audioTracks: [
          {
            id: 'testRemoteAudioTrackId',
            isMuted: true,
          },
        ],
        videoTracks: [
          {
            id: 'testRemoteVideoTrackId',
            isMuted: true,
          },
        ],
        screenShareTrack: null,
      },
    },
  },
  activeSpeakers: [],
}

test('handles room state changes', () => {
  const newState = sessionReducer(state, setRoomState(RoomState.DISCONNECTED))

  expect(newState.roomState).toBe(RoomState.DISCONNECTED)
})

test('handles session id changes', () => {
  const newState = sessionReducer(state, setSessionId('newSessionId'))

  expect(newState.sessionId).toBe('newSessionId')
})

test('handles local participant id changes', () => {
  const newState = sessionReducer(state, setLocalParticipantId('newLocalParticipantId'))

  expect(newState.localParticipant.id).toBe('newLocalParticipantId')
})

test('handles local participant name changes', () => {
  const newState = sessionReducer(state, setLocalParticipantName('newLocalParticipantName'))

  expect(newState.localParticipant.name).toBe('newLocalParticipantName')

  const newStateWithoutName = sessionReducer(state, setLocalParticipantName())

  expect(newStateWithoutName.localParticipant.name).toBeNull()
})

test('sets the active speakers if there are new active speakers', () => {
  const newState = sessionReducer(state, setActiveSpeakers(['testLocalParticipantId']))

  expect(newState).toStrictEqual({ ...state, activeSpeakers: ['testLocalParticipantId'] })
})

test('clears the active speakers if there are no speakers', () => {
  const newState = sessionReducer({ ...state, activeSpeakers: ['testLocalParticipantId'] }, setActiveSpeakers([]))

  expect(newState).toStrictEqual({ ...state, activeSpeakers: [] })
})

test('clears session id', () => {
  const newState = sessionReducer(state, clearSessionId())

  expect(newState.sessionId).toBeNull()
})

test('clears the local participant and remote participants', () => {
  const newState = sessionReducer(state, clearParticipants())

  expect(newState.remoteParticipants).toEqual({})
  expect(newState.localParticipant).toStrictEqual({
    connectionQuality: 'excellent',
    id: null,
    name: null,
    tracks: {
      audioTracks: [],
      screenShareTrack: null,
      videoTracks: [],
    },
  })
})

test('sets if the participant is connecting or not', () => {
  const newState = sessionReducer(state, setConnecting(true))

  expect(newState.isConnecting).toBe(true)
})

test('adds a new local audio track when a new local audio stream is published', () => {
  const newState = sessionReducer(
    state,
    participantLocalTrackPublished({
      trackId: 'newAudioTrackId',
      trackSource: StreamSource.MICROPHONE,
      trackKind: StreamKind.AUDIO,
      trackIsMuted: false,
    })
  )

  expect(newState.localParticipant.tracks.audioTracks).toHaveLength(2)
  expect(newState.localParticipant.tracks.audioTracks[1].id).toBe('newAudioTrackId')
  expect(newState.localParticipant.tracks.audioTracks[1].isMuted).toBe(false)
})

test('adds a new local video track when a new local video track is published', () => {
  const newState = sessionReducer(
    state,
    participantLocalTrackPublished({
      trackId: 'newVideoTrackId',
      trackSource: StreamSource.CAMERA,
      trackKind: StreamKind.VIDEO,
      trackIsMuted: false,
    })
  )

  expect(newState.localParticipant.tracks.videoTracks).toHaveLength(2)
  expect(newState.localParticipant.tracks.videoTracks[1].id).toBe('newVideoTrackId')
  expect(newState.localParticipant.tracks.videoTracks[1].isMuted).toBe(false)
})

test('sets the local screen share track when the local screen share track is published', () => {
  const newState = sessionReducer(
    state,
    participantLocalTrackPublished({
      trackId: 'newScreenShareTrackId',
      trackSource: StreamSource.SCREEN_SHARE,
      trackKind: StreamKind.VIDEO,
      trackIsMuted: false,
    })
  )

  expect(newState.localParticipant.tracks.screenShareTrack).toStrictEqual({
    id: 'newScreenShareTrackId',
    isMuted: false,
  })
})

test('does not change tracks when an unknown local track is published', () => {
  const newState = sessionReducer(
    state,
    participantLocalTrackPublished({
      trackId: 'unknownTrackId',
      trackSource: StreamSource.UNKNOWN,
      trackKind: StreamKind.UNKNOWN,
      trackIsMuted: false,
    })
  )

  expect(newState).toStrictEqual(state)
})

test('removes the local audio track when a local audio track is unpublished', () => {
  const newState = sessionReducer(
    state,
    participantLocalTrackUnPublished({
      trackId: 'testLocalAudioTrackId',
      trackSource: StreamSource.MICROPHONE,
      trackKind: StreamKind.AUDIO,
    })
  )

  expect(newState.localParticipant.tracks.audioTracks).toHaveLength(0)
})

test('removes the local video track when a local video track is unpublished', () => {
  const newState = sessionReducer(
    state,
    participantLocalTrackUnPublished({
      trackId: 'testLocalVideoTrackId',
      trackSource: StreamSource.CAMERA,
      trackKind: StreamKind.VIDEO,
    })
  )

  expect(newState.localParticipant.tracks.videoTracks).toHaveLength(0)
})

test('does not change tracks when an unknown local track is unpublished', () => {
  const newState = sessionReducer(
    state,
    participantLocalTrackUnPublished({
      trackId: 'unknownTrackId',
      trackSource: StreamSource.UNKNOWN,
      trackKind: StreamKind.UNKNOWN,
    })
  )

  expect(newState).toStrictEqual(state)
})

test('removes the remote audio track when a remote audio track is unpublished', () => {
  const newState = sessionReducer(
    state,
    participantTrackUnPublished({
      participantId: 'testRemoteParticipantId',
      trackId: 'testRemoteAudioTrackId',
      trackSource: StreamSource.MICROPHONE,
      trackKind: StreamKind.AUDIO,
    })
  )

  expect(newState.remoteParticipants['testRemoteParticipantId'].tracks.audioTracks).toHaveLength(0)
})

test('removes the remote video track when a remote video track is unpublished', () => {
  const newState = sessionReducer(
    state,
    participantTrackUnPublished({
      participantId: 'testRemoteParticipantId',
      trackId: 'testRemoteVideoTrackId',
      trackSource: StreamSource.CAMERA,
      trackKind: StreamKind.VIDEO,
    })
  )

  expect(newState.remoteParticipants['testRemoteParticipantId'].tracks.videoTracks).toHaveLength(0)
})

test('removes the remote screen share track when a remote screen share track is unpublished', () => {
  const newState = sessionReducer(
    state,
    participantTrackUnPublished({
      participantId: 'testRemoteParticipantId',
      trackId: 'testRemoteScreenShareTrackId',
      trackSource: StreamSource.SCREEN_SHARE,
      trackKind: StreamKind.VIDEO,
    })
  )

  expect(newState.remoteParticipants['testRemoteParticipantId'].tracks.screenShareTrack).toBeNull()
})

test('does not change remote participant tracks when an unknown track is unpublished', () => {
  const newState = sessionReducer(
    state,
    participantTrackUnPublished({
      participantId: 'testRemoteParticipantId',
      trackId: 'unknownTrackId',
      trackSource: StreamSource.UNKNOWN,
      trackKind: StreamKind.UNKNOWN,
    })
  )

  expect(newState).toStrictEqual(state)
})

test('does not change remote participant tracks when a track of an unknown participant is unpublished', () => {
  const newState = sessionReducer(
    state,
    participantTrackUnPublished({
      participantId: 'unknownRemoteParticipantId',
      trackId: 'unknownTrackId',
      trackSource: StreamSource.UNKNOWN,
      trackKind: StreamKind.UNKNOWN,
    })
  )

  expect(newState).toStrictEqual(state)
})

test('adds a new remote participant with default values', () => {
  const newState = sessionReducer(
    state,
    addRemoteParticipant({ participantId: 'newRemoteParticipantId', participantName: 'newRemoteParticipantName' })
  )

  expect(Object.keys(newState.remoteParticipants)).toHaveLength(2)
  expect(newState.remoteParticipants['testRemoteParticipantId']).toStrictEqual(
    state.remoteParticipants['testRemoteParticipantId']
  )
  expect(newState.remoteParticipants['newRemoteParticipantId']).toStrictEqual({
    id: 'newRemoteParticipantId',
    name: 'newRemoteParticipantName',
    connectionQuality: ConnectionQuality.EXCELLENT,
    tracks: {
      audioTracks: [],
      videoTracks: [],
      screenShareTrack: null,
    },
  })
})

test('removes a remote participant', () => {
  const newState = sessionReducer(state, removeRemoteParticipant({ participantId: 'testRemoteParticipantId' }))

  expect(newState.remoteParticipants).toEqual({})
})

test('creates a new remote participant audio track when the local participant subscribes to an audio track', () => {
  const newState = sessionReducer(
    state,
    subscribeToParticipantTrack({
      participantId: 'testRemoteParticipantId',
      trackId: 'newRemoteAudioTrackId',
      trackSource: StreamSource.MICROPHONE,
      trackKind: StreamKind.AUDIO,
      trackIsMuted: false,
    })
  )

  expect(newState.remoteParticipants['testRemoteParticipantId'].tracks.audioTracks).toHaveLength(2)
  expect(newState.remoteParticipants['testRemoteParticipantId'].tracks.audioTracks[1].id).toBe('newRemoteAudioTrackId')
  expect(newState.remoteParticipants['testRemoteParticipantId'].tracks.audioTracks[1].isMuted).toBe(false)
})

test('creates a new remote participant video track when the local participant subscribes to a video track', () => {
  const newState = sessionReducer(
    state,
    subscribeToParticipantTrack({
      participantId: 'testRemoteParticipantId',
      trackId: 'newRemoteVideoTrackId',
      trackSource: StreamSource.CAMERA,
      trackKind: StreamKind.VIDEO,
      trackIsMuted: false,
    })
  )

  expect(newState.remoteParticipants['testRemoteParticipantId'].tracks.videoTracks).toHaveLength(2)
  expect(newState.remoteParticipants['testRemoteParticipantId'].tracks.videoTracks[1].id).toBe('newRemoteVideoTrackId')
  expect(newState.remoteParticipants['testRemoteParticipantId'].tracks.videoTracks[1].isMuted).toBe(false)
})

test('sets the remote participant screen share track when the local participant subscribes to a remote screen share track', () => {
  const newState = sessionReducer(
    state,
    subscribeToParticipantTrack({
      participantId: 'testRemoteParticipantId',
      trackId: 'newRemoteScreenShareTrackId',
      trackSource: StreamSource.SCREEN_SHARE,
      trackKind: StreamKind.VIDEO,
      trackIsMuted: false,
    })
  )

  expect(newState.remoteParticipants['testRemoteParticipantId'].tracks.screenShareTrack).toStrictEqual({
    id: 'newRemoteScreenShareTrackId',
    isMuted: false,
  })
})

test('does not change the remote participant if the local participant subscribes to a track of unknown type', () => {
  const newState = sessionReducer(
    state,
    subscribeToParticipantTrack({
      participantId: 'testRemoteParticipantId',
      trackId: 'unknownTrackId',
      trackSource: StreamSource.UNKNOWN,
      trackKind: StreamKind.UNKNOWN,
      trackIsMuted: false,
    })
  )

  expect(newState).toStrictEqual(state)
})

test('does not change the state when the local participant subscribes to a track of an unknown remote participant', () => {
  const newState = sessionReducer(
    state,
    subscribeToParticipantTrack({
      participantId: 'unknownRemoteParticipant',
      trackId: 'remoteAudioTrackId',
      trackSource: StreamSource.MICROPHONE,
      trackKind: StreamKind.AUDIO,
      trackIsMuted: false,
    })
  )

  expect(newState).toStrictEqual(state)
})

test('mutes the local participant audio track, but not the video', () => {
  const newState = sessionReducer(
    {
      ...state,
      localParticipant: {
        id: 'testLocalParticipantId',
        name: 'testLocalParticipantName',
        connectionQuality: ConnectionQuality.EXCELLENT,
        tracks: {
          audioTracks: [
            {
              id: 'testLocalAudioTrackId',
              isMuted: false,
            },
          ],
          videoTracks: [
            {
              id: 'testLocalVideoTrackId',
              isMuted: false,
            },
          ],
          screenShareTrack: null,
        },
      },
    },
    participantTrackMuted({
      participantId: 'testLocalParticipantId',
      trackId: 'testLocalAudioTrackId',
    })
  )

  expect(newState.localParticipant.tracks.audioTracks[0].isMuted).toBe(true)
  expect(newState.localParticipant.tracks.videoTracks[0].isMuted).toBe(false)
})

test('mutes the local participant video track, but not the audio', () => {
  const newState = sessionReducer(
    {
      ...state,
      localParticipant: {
        id: 'testLocalParticipantId',
        name: 'testLocalParticipantName',
        connectionQuality: ConnectionQuality.EXCELLENT,
        tracks: {
          audioTracks: [
            {
              id: 'testLocalAudioTrackId',
              isMuted: false,
            },
          ],
          videoTracks: [
            {
              id: 'testLocalVideoTrackId',
              isMuted: false,
            },
          ],
          screenShareTrack: null,
        },
      },
    },
    participantTrackMuted({
      participantId: 'testLocalParticipantId',
      trackId: 'testLocalVideoTrackId',
    })
  )

  expect(newState.localParticipant.tracks.videoTracks[0].isMuted).toBe(true)
  expect(newState.localParticipant.tracks.audioTracks[0].isMuted).toBe(false)
})

test('unmutes the local participant audio track, but not the video', () => {
  const newState = sessionReducer(
    state,
    participantTrackUnMuted({
      participantId: 'testLocalParticipantId',
      trackId: 'testLocalAudioTrackId',
    })
  )

  expect(newState.localParticipant.tracks.audioTracks[0].isMuted).toBe(false)
  expect(newState.localParticipant.tracks.videoTracks[0].isMuted).toBe(true)
})

test('unmutes the local participant video track, but not the audio', () => {
  const newState = sessionReducer(
    state,
    participantTrackUnMuted({
      participantId: 'testLocalParticipantId',
      trackId: 'testLocalVideoTrackId',
    })
  )

  expect(newState.localParticipant.tracks.videoTracks[0].isMuted).toBe(false)
  expect(newState.localParticipant.tracks.audioTracks[0].isMuted).toBe(true)
})

test('mutes the remote participant audio track, but not the video', () => {
  const newState = sessionReducer(
    {
      ...state,
      remoteParticipants: {
        testRemoteParticipantId: {
          id: 'testRemoteParticipantId',
          name: 'testRemoteParticipantName',
          connectionQuality: ConnectionQuality.EXCELLENT,
          tracks: {
            audioTracks: [
              {
                id: 'testRemoteAudioTrackId',
                isMuted: false,
              },
            ],
            videoTracks: [
              {
                id: 'testRemoteVideoTrackId',
                isMuted: false,
              },
            ],
            screenShareTrack: null,
          },
        },
      },
    },
    participantTrackMuted({
      participantId: 'testRemoteParticipantId',
      trackId: 'testRemoteAudioTrackId',
    })
  )

  expect(newState.remoteParticipants['testRemoteParticipantId'].tracks.audioTracks[0].isMuted).toBe(true)
  expect(newState.remoteParticipants['testRemoteParticipantId'].tracks.videoTracks[0].isMuted).toBe(false)
})

test('mutes the remote participant video track, but not the audio', () => {
  const newState = sessionReducer(
    {
      ...state,
      remoteParticipants: {
        testRemoteParticipantId: {
          id: 'testRemoteParticipantId',
          name: 'testRemoteParticipantName',
          connectionQuality: ConnectionQuality.EXCELLENT,
          tracks: {
            audioTracks: [
              {
                id: 'testRemoteAudioTrackId',
                isMuted: false,
              },
            ],
            videoTracks: [
              {
                id: 'testRemoteVideoTrackId',
                isMuted: false,
              },
            ],
            screenShareTrack: null,
          },
        },
      },
    },
    participantTrackMuted({
      participantId: 'testRemoteParticipantId',
      trackId: 'testRemoteVideoTrackId',
    })
  )

  expect(newState.remoteParticipants['testRemoteParticipantId'].tracks.videoTracks[0].isMuted).toBe(true)
  expect(newState.remoteParticipants['testRemoteParticipantId'].tracks.audioTracks[0].isMuted).toBe(false)
})

test('unmutes the remote participant audio track, but not the video', () => {
  const newState = sessionReducer(
    state,
    participantTrackUnMuted({
      participantId: 'testRemoteParticipantId',
      trackId: 'testRemoteAudioTrackId',
    })
  )

  expect(newState.remoteParticipants['testRemoteParticipantId'].tracks.audioTracks[0].isMuted).toBe(false)
  expect(newState.remoteParticipants['testRemoteParticipantId'].tracks.videoTracks[0].isMuted).toBe(true)
})

test('unmutes the remote participant video track, but not the audio', () => {
  const newState = sessionReducer(
    state,
    participantTrackUnMuted({
      participantId: 'testRemoteParticipantId',
      trackId: 'testRemoteVideoTrackId',
    })
  )

  expect(newState.remoteParticipants['testRemoteParticipantId'].tracks.videoTracks[0].isMuted).toBe(false)
  expect(newState.remoteParticipants['testRemoteParticipantId'].tracks.audioTracks[0].isMuted).toBe(true)
})

test('does not do anything when a track of an unknown participant is muted', () => {
  const newState = sessionReducer(
    state,
    participantTrackMuted({
      participantId: 'unknownRemoteParticipant',
      trackId: 'unknownRemoteAudioTrackId',
    })
  )

  expect(newState).toStrictEqual(state)
})

test('does not do anything when a track of an unknown participant is unmuted', () => {
  const newState = sessionReducer(
    state,
    participantTrackUnMuted({
      participantId: 'unknownRemoteParticipant',
      trackId: 'unknownRemoteAudioTrackId',
    })
  )

  expect(newState).toStrictEqual(state)
})

test('does not do anything when an unknown track of a participant is muted', () => {
  const newState = sessionReducer(
    state,
    participantTrackMuted({
      participantId: 'testRemoteParticipantId',
      trackId: 'unknownRemoteAudioTrackId',
    })
  )

  expect(newState).toStrictEqual(state)
})

test('changes the connection quality of the local participant', () => {
  const newState = sessionReducer(
    state,
    connectionQualityChanged({ participantId: 'testLocalParticipantId', connectionQuality: ConnectionQuality.POOR })
  )

  expect(newState.localParticipant.connectionQuality).toBe(ConnectionQuality.POOR)
  expect(newState.remoteParticipants).toStrictEqual(state.remoteParticipants)
})

test('does not change state when the connection quality changes for an unknown participant', () => {
  const newState = sessionReducer(
    state,
    connectionQualityChanged({ participantId: 'unknownParticipantId', connectionQuality: ConnectionQuality.POOR })
  )

  expect(newState).toStrictEqual(state)
})

test('changes the connection quality of the remote participant', () => {
  const newState = sessionReducer(
    state,
    connectionQualityChanged({ participantId: 'testRemoteParticipantId', connectionQuality: ConnectionQuality.GOOD })
  )

  expect(newState.remoteParticipants['testRemoteParticipantId'].connectionQuality).toBe(ConnectionQuality.GOOD)
  expect(newState.localParticipant).toStrictEqual(state.localParticipant)
})

test('selectTracksForParticipant selects audio, video and screen share tracks for local participant if available', () => {
  const { mockGetSessionById, mockLocalAudioTrack, mockLocalVideoTrack, mockLocalScreenShareTrack } =
    setupZocksSdkMocks()
  const localParticipantWithAllTracks = {
    ...state.localParticipant,
    tracks: {
      audioTracks: [
        {
          id: 'testLocalAudioTrackId',
          isMuted: false,
        },
      ],
      videoTracks: [
        {
          id: 'testLocalVideoTrackId',
          isMuted: false,
        },
      ],
      screenShareTrack: {
        id: 'testLocalScreenShareTrackId',
        isMuted: false,
      },
    },
  }

  const tracks = selectTracksForParticipant({ ...mockRootState, session: state }, localParticipantWithAllTracks)

  expect(mockGetSessionById).toHaveBeenCalledTimes(1)
  expect(mockGetSessionById).toHaveBeenCalledWith('testSessionId')
  expect(tracks).toStrictEqual({
    audioTracks: [{ track: mockLocalAudioTrack, isMuted: false }],
    videoTracks: [{ track: mockLocalVideoTrack, isMuted: false }],
    primaryAudioTrack: { track: mockLocalAudioTrack, isMuted: false },
    primaryVideoTrack: { track: mockLocalVideoTrack, isMuted: false },
    screenShareTrack: { track: mockLocalScreenShareTrack, isMuted: false },
  })
})

test('selectTracksForParticipant selects audio, video and screen share tracks for remote participant if available', () => {
  const { mockGetSessionById, mockRemoteAudioTrack, mockRemoteVideoTrack, mockRemoteScreenShareTrack } =
    setupZocksSdkMocks()

  const remoteParticipantWithAllTracks = {
    ...state.remoteParticipants['testRemoteParticipantId'],
    tracks: {
      audioTracks: [
        {
          id: 'testRemoteAudioTrackId',
          isMuted: false,
        },
      ],
      videoTracks: [
        {
          id: 'testRemoteVideoTrackId',
          isMuted: false,
        },
      ],
      screenShareTrack: {
        id: 'testRemoteScreenShareTrackId',
        isMuted: false,
      },
    },
  }
  const tracks = selectTracksForParticipant({ ...mockRootState, session: state }, remoteParticipantWithAllTracks)

  expect(mockGetSessionById).toHaveBeenCalledTimes(1)
  expect(mockGetSessionById).toHaveBeenCalledWith('testSessionId')
  expect(tracks).toStrictEqual({
    audioTracks: [{ track: mockRemoteAudioTrack, isMuted: false }],
    videoTracks: [{ track: mockRemoteVideoTrack, isMuted: false }],
    primaryAudioTrack: { track: mockRemoteAudioTrack, isMuted: false },
    primaryVideoTrack: { track: mockRemoteVideoTrack, isMuted: false },
    screenShareTrack: { track: mockRemoteScreenShareTrack, isMuted: false },
  })
})

test('selectTracksForParticipant does not select tracks if tracks are not available', () => {
  const { mockGetSessionById } = setupZocksSdkMocks()
  const localParticipantWithoutTracks = {
    id: 'testLocalParticipantId',
    name: 'testLocalParticipantName',
    connectionQuality: ConnectionQuality.EXCELLENT,
    isSpeaking: false,
    tracks: {
      audioTracks: [
        {
          id: 'unknownAudioTrackId',
          isMuted: false,
        },
      ],
      videoTracks: [
        {
          id: 'unknownVideoTrackId',
          isMuted: false,
        },
      ],
      screenShareTrack: {
        id: 'unknownScreenShareTrackId',
        isMuted: false,
      },
    },
  }

  const tracks = selectTracksForParticipant(
    {
      ...mockRootState,
      session: {
        ...state,
        localParticipant: localParticipantWithoutTracks,
      },
    },
    localParticipantWithoutTracks
  )

  expect(mockGetSessionById).toHaveBeenCalledTimes(1)
  expect(mockGetSessionById).toHaveBeenCalledWith('testSessionId')
  expect(tracks).toStrictEqual({
    audioTracks: [],
    videoTracks: [],
    primaryAudioTrack: null,
    primaryVideoTrack: null,
    screenShareTrack: null,
  })
})

test('screen sharing is on when either the local participant or one of the remote participants share their screen', () => {
  const stateWithLocalParticipantScreenSharing = {
    ...state,
    localParticipant: {
      ...state.localParticipant,
      tracks: {
        audioTracks: [],
        videoTracks: [],
        screenShareTrack: {
          id: 'testLocalScreenShareTrackId',
          isMuted: false,
        },
      },
    },
  }
  const stateWithRemoteParticipantScreenSharing = {
    ...state,
    remoteParticipants: {
      testRemoteParticipantId: {
        ...state.remoteParticipants.testRemoteParticipantId,
        tracks: {
          audioTracks: [],
          videoTracks: [],
          screenShareTrack: {
            id: 'testRemoteScreenShareTrackId',
            isMuted: false,
          },
        },
      },
    },
  }

  expect(selectIsScreenSharing({ ...mockRootState, session: stateWithLocalParticipantScreenSharing })).toBe(true)
  expect(selectIsScreenSharing({ ...mockRootState, session: stateWithRemoteParticipantScreenSharing })).toBe(true)
})

test('screen sharing is off when neither the local nor the remote participants share their screen', () => {
  expect(selectIsScreenSharing({ ...mockRootState, session: state })).toBe(false)
})

test('selectScreenSharingParticipant selects the local participant if the local participant is sharing their screen', () => {
  const stateWithLocalParticipantScreenSharing = {
    ...state,
    localParticipant: {
      ...state.localParticipant,
      tracks: {
        audioTracks: [],
        videoTracks: [],
        screenShareTrack: {
          id: 'testLocalScreenShareTrackId',
          isMuted: false,
        },
      },
    },
  }

  expect(
    selectScreenSharingParticipant({ ...mockRootState, session: stateWithLocalParticipantScreenSharing })
  ).toStrictEqual(stateWithLocalParticipantScreenSharing.localParticipant)
})

test('selectScreenSharingParticipant selects the remote participant if a remote participant is sharing their screen', () => {
  const stateWithRemoteParticipantScreenSharing = {
    ...state,
    remoteParticipants: {
      testRemoteParticipantId: {
        ...state.remoteParticipants.testRemoteParticipantId,
        tracks: {
          audioTracks: [],
          videoTracks: [],
          screenShareTrack: {
            id: 'testRemoteScreenShareTrackId',
            isMuted: false,
          },
        },
      },
    },
  }

  expect(
    selectScreenSharingParticipant({ ...mockRootState, session: stateWithRemoteParticipantScreenSharing })
  ).toStrictEqual(stateWithRemoteParticipantScreenSharing.remoteParticipants.testRemoteParticipantId)
})

test('selectScreenSharingParticipant does not select a participant if noone is sharing their screen', () => {
  expect(selectScreenSharingParticipant({ ...mockRootState, session: state })).toBeNull()
})

test('selectSpeakingRemoteParticipantAudioTracks select remote participant audio track which are not muted', () => {
  const { mockGetSessionById, mockRemoteAudioTrack } = setupZocksSdkMocks()

  const stateWithRemoteParticipantAudio = {
    ...state,
    remoteParticipants: {
      testRemoteParticipantId: {
        ...state.remoteParticipants.testRemoteParticipantId,
        tracks: {
          audioTracks: [
            {
              id: 'testRemoteAudioTrackId',
              isMuted: false,
            },
          ],
          videoTracks: [],
          screenShareTrack: null,
        },
      },
    },
  }
  const tracks = selectSpeakingRemoteParticipantAudioTracks({
    ...mockRootState,
    session: stateWithRemoteParticipantAudio,
  })

  expect(mockGetSessionById).toHaveBeenCalledTimes(1)
  expect(mockGetSessionById).toHaveBeenCalledWith('testSessionId')
  expect(tracks).toStrictEqual([{ track: mockRemoteAudioTrack, isMuted: false }])
})

test('selectSpeakingRemoteParticipantAudioTracks does not select tracks if they are not available', () => {
  const { mockGetSessionById } = setupZocksSdkMocks()

  const stateWithRemoteParticipantAudio = {
    ...state,
    remoteParticipants: {
      testRemoteParticipantId: {
        ...state.remoteParticipants.testRemoteParticipantId,
        tracks: {
          audioTracks: [
            {
              id: 'unknownRemoteAudioTrackId',
              isMuted: false,
            },
          ],
          videoTracks: [],
          screenShareTrack: null,
        },
      },
    },
  }
  const tracks = selectSpeakingRemoteParticipantAudioTracks({
    ...mockRootState,
    session: stateWithRemoteParticipantAudio,
  })

  expect(mockGetSessionById).toHaveBeenCalledTimes(1)
  expect(mockGetSessionById).toHaveBeenCalledWith('testSessionId')
  expect(tracks).toStrictEqual([])
})

test('selectSpeakingRemoteParticipantAudioTracks does not select tracks if there are no audio tracks', () => {
  const { mockGetSessionById } = setupZocksSdkMocks()

  const stateWithRemoteParticipantAudio = {
    ...state,
    remoteParticipants: {
      testRemoteParticipantId: {
        ...state.remoteParticipants.testRemoteParticipantId,
        tracks: {
          audioTracks: [],
          videoTracks: [],
          screenShareTrack: null,
        },
      },
    },
  }
  const tracks = selectSpeakingRemoteParticipantAudioTracks({
    ...mockRootState,
    session: stateWithRemoteParticipantAudio,
  })

  expect(mockGetSessionById).toHaveBeenCalledTimes(1)
  expect(mockGetSessionById).toHaveBeenCalledWith('testSessionId')
  expect(tracks).toStrictEqual([])
})
