/* istanbul ignore file */
import * as zocksSdk from '@zocks-communications/client-sdk-js'
import {
  ConnectionQuality,
  LocalParticipant,
  LocalMediaStream,
  RemoteParticipant,
  Session,
  StreamKind,
  StreamSource,
  Track,
  ZocksVideo,
  ZocksApiRoomService,
  ApiService,
  RoomInfo,
} from '@zocks-communications/client-sdk-js'
import { mock } from 'jest-mock-extended'
import { ParticipantModel, SessionModel } from './models'
import { ViewModel } from './store/view/models'
import { RootState, zocksStoreConfig } from './app/store'
import { configureStore } from '@reduxjs/toolkit'
import {
  addRemoteParticipant,
  setActiveSpeakers,
  setLocalParticipantId,
  setLocalParticipantName,
} from './store/session/sessionSlice'
import { ChatModel } from './store/chat/models'

export const setupZocksSdkMocks = () => {
  const mockLocalAudioTrack = mock<Track>({
    id: 'testLocalAudioTrackId',
    source: StreamSource.MICROPHONE,
    kind: StreamKind.AUDIO,
    isAvailable: true,
    isMuted: false,
  })
  const mockLocalVideoTrack = mock<Track>({
    id: 'testLocalVideoTrackId',
    source: StreamSource.CAMERA,
    kind: StreamKind.VIDEO,
    isAvailable: true,
    isMuted: false,
  })
  const mockRemoteAudioTrack = mock<Track>({
    id: 'testRemoteAudioTrackId',
    source: StreamSource.MICROPHONE,
    kind: StreamKind.AUDIO,
    isAvailable: true,
    isMuted: false,
  })
  const mockRemoteVideoTrack = mock<Track>({
    id: 'testRemoteVideoTrackId',
    source: StreamSource.CAMERA,
    kind: StreamKind.VIDEO,
    isAvailable: true,
    isMuted: false,
  })

  const mockLocalScreenShareTrack = mock<Track>({
    id: 'testLocalScreenShareTrackId',
    source: StreamSource.SCREEN_SHARE,
    kind: StreamKind.VIDEO,
    isAvailable: true,
    isMuted: false,
  })

  const mockRemoteScreenShareTrack = mock<Track>({
    id: 'testRemoteScreenShareTrackId',
    source: StreamSource.SCREEN_SHARE,
    kind: StreamKind.VIDEO,
    isAvailable: true,
    isMuted: false,
  })

  const mockLocalParticipant = mock<LocalParticipant>({
    id: 'testLocalParticipantId',
    name: 'testLocalParticipantName',
    tracks: [mockLocalAudioTrack, mockLocalVideoTrack, mockLocalScreenShareTrack],
  })

  const mockRemoteParticipant = mock<RemoteParticipant>({
    id: 'testRemoteParticipantId',
    name: 'testRemoteParticipantName',
    tracks: [mockRemoteAudioTrack, mockRemoteVideoTrack, mockRemoteScreenShareTrack],
  })

  const remoteParticipants = new Map<string, RemoteParticipant>([[mockRemoteParticipant.id, mockRemoteParticipant]])
  const mockRemoteParticipants = mock<Map<string, RemoteParticipant>>()
  mockRemoteParticipants.get.mockImplementation((participantId: string) => remoteParticipants.get(participantId))
  mockRemoteParticipants.set.mockImplementation((participantId: string, remoteParticipant: RemoteParticipant) =>
    remoteParticipants.set(participantId, remoteParticipant)
  )
  mockRemoteParticipants.values.mockImplementation(() => remoteParticipants.values())
  mockRemoteParticipants.has.mockImplementation((participantId: string) => remoteParticipants.has(participantId))

  const mockSession = mock<Session>({
    id: 'testSessionId',
    localParticipant: mockLocalParticipant,
    participants: mockRemoteParticipants,
  })

  const mockApi = setupApiMock()
  const mockZocksVideo = mock<ZocksVideo>({ API: mockApi })

  mockZocksVideo.createNewSession.mockReturnValue(mockSession)

  const mockGetSessionById = jest.spyOn(zocksSdk.ZocksSDK, 'getSessionById').mockReturnValue(mockSession)
  const mockInitSdk = jest.spyOn(zocksSdk.ZocksSDK, 'init').mockReturnValue(mockZocksVideo)

  const events = {}
  const mockEventBus = {
    // @ts-ignore
    on: jest.fn().mockImplementation((evt, callback) => (events[evt] = callback)),
    // @ts-ignore
    emit: jest.fn().mockImplementation((evt, ...args) => events[evt](...args)),
  }

  mockSession.on.mockImplementation((evt, callback) => mockEventBus.on(evt, callback))
  mockLocalParticipant.on.mockImplementation((evt, callback) => mockEventBus.on(evt, callback))

  mockLocalParticipant.publishScreenShare.mockReturnValue(Promise.resolve())

  const mockStore = {
    session: {
      sessionId: mockSession.id,
      localParticipant: {
        id: mockLocalParticipant.id,
        name: mockLocalParticipant.name,
        tracks: {
          audioTracks: [{ id: mockLocalAudioTrack.id, isMuted: mockLocalAudioTrack.isMuted }],
          videoTracks: [{ id: mockLocalVideoTrack.id, isMuted: mockLocalVideoTrack.isMuted }],
          screenShareTrack: {
            id: mockLocalScreenShareTrack.id,
            isMuted: mockLocalScreenShareTrack.isMuted,
          },
        },
        connectionQuality: ConnectionQuality.EXCELLENT,
      },
      remoteParticipants: {
        [mockRemoteParticipant.id]: {
          id: mockRemoteParticipant.id,
          name: mockRemoteParticipant.name,
          tracks: {
            audioTracks: [{ id: mockRemoteAudioTrack.id, isMuted: mockRemoteAudioTrack.isMuted }],
            videoTracks: [{ id: mockRemoteVideoTrack.id, isMuted: mockRemoteVideoTrack.isMuted }],
            screenShareTrack: {
              id: mockRemoteScreenShareTrack.id,
              isMuted: mockRemoteScreenShareTrack.isMuted,
            },
          },
          connectionQuality: ConnectionQuality.EXCELLENT,
        },
      },
    },
  }

  return {
    mockInitSdk,
    mockGetSessionById,
    mockApi,
    mockSession,
    mockLocalParticipant,
    mockRemoteParticipant,
    mockLocalAudioTrack,
    mockLocalVideoTrack,
    mockLocalScreenShareTrack,
    mockRemoteAudioTrack,
    mockRemoteVideoTrack,
    mockRemoteScreenShareTrack,
    mockEventBus,
    mockStore,
  }
}

const setupApiMock = () => {
  const mockApiRoomService = mock<ZocksApiRoomService>({ create: jest.fn().mockResolvedValue(mock<RoomInfo>()) })

  return mock<ApiService>({ room: mockApiRoomService })
}

export const createMockParticipants = (n: number): ParticipantModel[] =>
  Array.from({ length: n }, (x, i) => {
    return {
      id: `Participant id: ${i.toString(10)}`,
      name: `Participant name: ${i.toString(10)}`,
      tracks: {
        videoTracks: [],
        audioTracks: [],
        screenShareTrack: null,
      },
      connectionQuality: ConnectionQuality.UNKNOWN,
    }
  })

export const initStore = (numberOfRemoteParticipants: number, activeSpeakers?: string[]) => {
  const store = configureStore(zocksStoreConfig)
  store.dispatch(setLocalParticipantId('Participant id: local'))
  store.dispatch(setLocalParticipantName('Participant name: local'))
  const mockRemoteParticipants = createMockParticipants(numberOfRemoteParticipants)
  mockRemoteParticipants.forEach((participant) =>
    store.dispatch(addRemoteParticipant({ participantId: participant.id!, participantName: participant.name! }))
  )
  if (activeSpeakers) {
    store.dispatch(setActiveSpeakers(activeSpeakers))
  }

  return store
}

export const setupMediaDevicesMock = () => {
  const mockNavigatorMediaDevices = { addEventListener: jest.fn(), removeEventListener: jest.fn() }
  const mockMediaDevices: MediaDeviceInfo[] = [
    {
      deviceId: 'testMicrophone1MediaDeviceId',
      label: 'Test Microphone 1',
      kind: 'audioinput',
      groupId: 'test',
      toJSON: jest.fn(),
    },
    {
      deviceId: 'testMicrophone2MediaDeviceId',
      label: 'Test Microphone 2',
      kind: 'audioinput',
      groupId: 'test',
      toJSON: jest.fn(),
    },
    {
      deviceId: 'testCamera1MediaDeviceId',
      label: 'Test Camera 1',
      kind: 'videoinput',
      groupId: 'test',
      toJSON: jest.fn(),
    },
    {
      deviceId: 'testCamera2MediaDeviceId',
      label: 'Test Camera 2',
      kind: 'videoinput',
      groupId: 'test',
      toJSON: jest.fn(),
    },
  ]
  const mockGetDevices = jest.spyOn(zocksSdk.MediaDeviceManager, 'getDevices').mockResolvedValue(mockMediaDevices)
  Object.defineProperty(window.navigator, 'mediaDevices', {
    writable: true,
    value: mockNavigatorMediaDevices,
  })

  return { mockGetDevices, mockMediaDevices, mockNavigatorMediaDevices }
}

export const setupMockLocalMediaStream = (deviceId?: string) => {
  const mockLocalMediaStream = mock<LocalMediaStream>({
    mediaStreamTrack: mock<MediaStreamTrack>({
      getSettings: jest.fn().mockImplementation(() => ({ deviceId })),
    }),
  })
  const mockCreateMediaStream = jest
    .spyOn(LocalMediaStream, 'create')
    .mockReturnValue(Promise.resolve(mockLocalMediaStream))

  return { mockLocalMediaStream, mockCreateMediaStream }
}

export const mockRootState: RootState = {
  session: mock<SessionModel>(),
  view: mock<ViewModel>(),
  chat: mock<ChatModel>(),
}
